package com.wireme.mediaserver;

import android.util.Log;
import org.teleal.cling.model.message.header.EXTHeader;
import org.teleal.cling.support.contentdirectory.AbstractContentDirectoryService;
import org.teleal.cling.support.contentdirectory.ContentDirectoryErrorCode;
import org.teleal.cling.support.contentdirectory.ContentDirectoryException;
import org.teleal.cling.support.contentdirectory.DIDLParser;
import org.teleal.cling.support.model.BrowseFlag;
import org.teleal.cling.support.model.BrowseResult;
import org.teleal.cling.support.model.DIDLContent;
import org.teleal.cling.support.model.SortCriterion;
import org.teleal.cling.support.model.container.Container;
import org.teleal.cling.support.model.item.Item;

public class ContentDirectoryService extends AbstractContentDirectoryService {
    private static final String LOGTAG = "MediaServer-CDS";
    private final String SEARCH_PREFIX = "upnp:class";
    private final int SEARCH_TYPE_EXACT = 1;
    private final int SEARCH_TYPE_PARTIAL = 2;
    private String searchCriteria;
    private int searchType;

    public BrowseResult browse(String objectID, BrowseFlag browseFlag, String filter, long firstResult, long maxResults, SortCriterion[] orderby) throws ContentDirectoryException {
        try {
            DIDLContent didl = new DIDLContent();
            ContentNode contentNode = ContentTree.getNode(objectID);
            Log.v(LOGTAG, "someone's browsing id: " + objectID);
            if (contentNode == null) {
                return new BrowseResult(EXTHeader.DEFAULT_VALUE, 0, 0);
            }
            if (contentNode.isItem()) {
                didl.addItem(contentNode.getItem());
                Log.v(LOGTAG, "returing item: " + contentNode.getItem().getTitle());
                return new BrowseResult(new DIDLParser().generate(didl), 1, 1);
            } else if (browseFlag == BrowseFlag.METADATA) {
                didl.addContainer(contentNode.getContainer());
                Log.v(LOGTAG, "returning metadata of container: " + contentNode.getContainer().getTitle());
                return new BrowseResult(new DIDLParser().generate(didl), 1, 1);
            } else {
                for (Container container : contentNode.getContainer().getContainers()) {
                    didl.addContainer(container);
                    Log.v(LOGTAG, "getting child container: " + container.getTitle());
                }
                for (Item item : contentNode.getContainer().getItems()) {
                    didl.addItem(item);
                    Log.v(LOGTAG, "getting child item: " + item.getTitle());
                }
                return new BrowseResult(new DIDLParser().generate(didl), (long) contentNode.getContainer().getChildCount().intValue(), (long) contentNode.getContainer().getChildCount().intValue());
            }
        } catch (Exception ex) {
            throw new ContentDirectoryException(ContentDirectoryErrorCode.CANNOT_PROCESS, ex.toString());
        }
    }

    private int initSearchInfo(String search) {
        this.searchType = 0;
        if (!search.startsWith("upnp:class")) {
            return -1;
        }
        String tmp = search.substring("upnp:class".length()).trim();
        if (tmp.startsWith("derivedfrom")) {
            tmp = tmp.substring(11).trim();
            if (tmp.charAt(0) == '\"') {
                tmp = tmp.substring(1);
            }
            if (tmp.startsWith("object.item")) {
                this.searchType = 2;
                this.searchCriteria = "object.item";
            }
        } else if (tmp.startsWith("=")) {
            tmp = tmp.substring(1).trim();
            if (tmp.charAt(0) == '\"') {
                tmp = tmp.substring(1);
            }
            if (tmp.startsWith("object.item")) {
                this.searchType = 1;
                if (tmp.startsWith("object.item.audioItem")) {
                    this.searchCriteria = "object.item.audioItem";
                } else if (tmp.startsWith("object.item.imageItem")) {
                    this.searchCriteria = "object.item.imageItem";
                } else if (tmp.startsWith("object.item.videoItem")) {
                    this.searchCriteria = "object.item.videoItem";
                }
            }
        }
        if (this.searchType != 0) {
            return 0;
        }
        return -1;
    }

    private boolean compareSearchItem(Item item) {
        return item.getClazz().getValue().startsWith(this.searchCriteria);
    }

    private final long addContainerItem(DIDLContent didl, Container container, long count, long firstResult, long maxValue) {
        if (count >= maxValue) {
            return count;
        }
        for (Container rcontainer : container.getContainers()) {
            count = addContainerItem(didl, rcontainer, count, firstResult, maxValue);
            if (count >= maxValue) {
                break;
            }
        }
        for (Item ritem : container.getItems()) {
            if (compareSearchItem(ritem)) {
                if (firstResult <= count) {
                    didl.addItem(ritem);
                }
                Log.v(LOGTAG, "getting child item: " + ritem.getTitle());
                count++;
                if (count >= maxValue) {
                    break;
                }
            }
        }
        return count;
    }

    public BrowseResult search(String containerId, String searchStr, String filter, long firstResult, long maxResults, SortCriterion[] orderBy) throws ContentDirectoryException {
        if (initSearchInfo(searchStr) < 0) {
            return new BrowseResult(EXTHeader.DEFAULT_VALUE, 0, 0);
        }
        if (maxResults == 0) {
            maxResults = 10000;
        }
        maxResults += firstResult;
        try {
            DIDLContent didl = new DIDLContent();
            ContentNode contentNode = ContentTree.getNode(containerId);
            Log.v(LOGTAG, "someone's browsing id: " + containerId);
            if (contentNode == null) {
                return new BrowseResult(EXTHeader.DEFAULT_VALUE, null, 0);
            }
            if (contentNode.isItem()) {
                didl.addItem(contentNode.getItem());
                Log.v(LOGTAG, "returing item: " + contentNode.getItem().getTitle());
                return new BrowseResult(new DIDLParser().generate(didl), 1, 1);
            }
            long count = 0;
            for (Container container : contentNode.getContainer().getContainers()) {
                count = addContainerItem(didl, container, count, firstResult, maxResults);
                if (count >= maxResults) {
                    break;
                }
                Log.v(LOGTAG, "getting child container: " + container.getTitle());
            }
            if (count < maxResults) {
                for (Item item : contentNode.getContainer().getItems()) {
                    if (compareSearchItem(item)) {
                        if (count >= firstResult) {
                            didl.addItem(item);
                        }
                        count++;
                        if (count >= maxResults) {
                            break;
                        }
                        Log.v(LOGTAG, "getting child item: " + item.getTitle());
                    }
                }
            }
            count -= firstResult;
            if (count < 0) {
                count = 0;
            }
            Log.d("Router", "Search_CNT:" + Long.valueOf(count).toString());
            return new BrowseResult(new DIDLParser().generate(didl), count, count);
        } catch (Exception ex) {
            throw new ContentDirectoryException(ContentDirectoryErrorCode.CANNOT_PROCESS, ex.toString());
        }
    }
}
