package org.teleal.cling.android;

import android.net.wifi.WifiManager;
import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Logger;
import org.teleal.cling.model.Constants;
import org.teleal.cling.model.ModelUtil;
import org.teleal.cling.transport.spi.InitializationException;
import org.teleal.cling.transport.spi.NetworkAddressFactory;

public class AndroidNetworkAddressFactory implements NetworkAddressFactory {
    private static final Logger log = Logger.getLogger(NetworkAddressFactory.class.getName());
    protected List<InetAddress> bindAddresses = new ArrayList();
    protected NetworkInterface wifiInterface;

    public AndroidNetworkAddressFactory(WifiManager wifiManager) throws InitializationException {
        int count = 0;
        log.info("jangpw : Search WiFi network interface ");
        do {
            this.wifiInterface = getWifiNetworkInterface(wifiManager);
            if (this.wifiInterface == null) {
                if (count > 2) {
                    break;
                }
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                }
            }
            count++;
        } while (this.wifiInterface == null);
        if (this.wifiInterface == null) {
            throw new InitializationException("Could not discover WiFi network interface");
        }
        log.info("Discovered WiFi network interface: " + this.wifiInterface.getDisplayName());
        if (this.wifiInterface.getDisplayName().equals("lo")) {
            discoverBindAddressesWLAN();
        } else {
            discoverBindAddresses();
        }
    }

    protected void discoverBindAddressesWLAN() throws InitializationException {
        try {
            Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
            while (en.hasMoreElements()) {
                NetworkInterface intf = (NetworkInterface) en.nextElement();
                Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses();
                while (enumIpAddr.hasMoreElements()) {
                    InetAddress inetAddress = (InetAddress) enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress() && (inetAddress instanceof Inet4Address)) {
                        if (intf.getName().startsWith("eth") || intf.getName().startsWith("wlan")) {
                            this.bindAddresses.add(inetAddress);
                            this.wifiInterface = intf;
                            break;
                        }
                    }
                }
            }
        } catch (Exception ex) {
            throw new InitializationException("Could not not analyze local network interfaces: " + ex, ex);
        }
    }

    protected void discoverBindAddresses() throws InitializationException {
        try {
            log.finer("Discovering addresses of interface: " + this.wifiInterface.getDisplayName());
            for (InetAddress inetAddress : getInetAddresses(this.wifiInterface)) {
                if (inetAddress == null) {
                    log.warning("Network has a null address: " + this.wifiInterface.getDisplayName());
                } else if (isUsableAddress(inetAddress)) {
                    log.fine("Discovered usable network interface address: " + inetAddress.getHostAddress());
                    this.bindAddresses.add(inetAddress);
                } else {
                    log.finer("Ignoring non-usable network interface address: " + inetAddress.getHostAddress());
                }
            }
        } catch (Exception ex) {
            throw new InitializationException("Could not not analyze local network interfaces: " + ex, ex);
        }
    }

    protected boolean isUsableAddress(InetAddress address) {
        if (address instanceof Inet4Address) {
            return true;
        }
        log.finer("Skipping unsupported non-IPv4 address: " + address);
        return false;
    }

    protected List<InetAddress> getInetAddresses(NetworkInterface networkInterface) {
        return Collections.list(networkInterface.getInetAddresses());
    }

    public InetAddress getMulticastGroup() {
        try {
            return InetAddress.getByName(Constants.IPV4_UPNP_MULTICAST_GROUP);
        } catch (UnknownHostException ex) {
            throw new RuntimeException(ex);
        }
    }

    public int getMulticastPort() {
        return Constants.UPNP_MULTICAST_PORT;
    }

    public int getStreamListenPort() {
        return 0;
    }

    public NetworkInterface[] getNetworkInterfaces() {
        return new NetworkInterface[]{this.wifiInterface};
    }

    public InetAddress[] getBindAddresses() {
        return (InetAddress[]) this.bindAddresses.toArray(new InetAddress[this.bindAddresses.size()]);
    }

    public byte[] getHardwareAddress(InetAddress inetAddress) {
        return null;
    }

    public InetAddress getBroadcastAddress(InetAddress inetAddress) {
        return null;
    }

    public InetAddress getLocalAddress(NetworkInterface networkInterface, boolean isIPv6, InetAddress remoteAddress) {
        for (InetAddress localAddress : getInetAddresses(networkInterface)) {
            if (!isIPv6 || !(localAddress instanceof Inet6Address)) {
                if (isIPv6 || !(localAddress instanceof Inet4Address)) {
                }
            }
            return localAddress;
        }
        throw new IllegalStateException("Can't find any IPv4 or IPv6 address on interface: " + networkInterface.getDisplayName());
    }

    public static NetworkInterface getWifiNetworkInterface(WifiManager manager) {
        if (!ModelUtil.ANDROID_EMULATOR) {
            return getRealWifiNetworkInterface(manager);
        }
        log.info("jangpw: getRealWifiNetworkInterface search enumlator interface ");
        return getEmulatorWifiNetworkInterface(manager);
    }

    public static NetworkInterface getEmulatorWifiNetworkInterface(WifiManager manager) {
        try {
            for (NetworkInterface iface : Collections.list(NetworkInterface.getNetworkInterfaces())) {
                for (InetAddress address : Collections.list(iface.getInetAddresses())) {
                    if (!address.isLoopbackAddress()) {
                        return iface;
                    }
                }
            }
            return null;
        } catch (Exception ex) {
            throw new InitializationException("Could not find emulator's network interface: " + ex, ex);
        }
    }

    public static NetworkInterface getRealWifiNetworkInterface(WifiManager manager) {
        NetworkInterface ifaceExtra = null;
        try {
            Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();
            int wifiIP = manager.getConnectionInfo().getIpAddress();
            int reverseWifiIP = Integer.reverseBytes(wifiIP);
            while (interfaces.hasMoreElements()) {
                NetworkInterface iface = (NetworkInterface) interfaces.nextElement();
                log.info("jangpw: getRealWifiNetworkInterface name = " + iface.getName());
                Enumeration<InetAddress> inetAddresses = iface.getInetAddresses();
                while (inetAddresses.hasMoreElements()) {
                    int byteArrayToInt = byteArrayToInt(((InetAddress) inetAddresses.nextElement()).getAddress(), 0);
                    if (byteArrayToInt == wifiIP) {
                        return iface;
                    }
                    if (byteArrayToInt == reverseWifiIP) {
                        return iface;
                    }
                }
                if (iface.getName().startsWith("eth") || iface.getName().startsWith("wlan")) {
                    ifaceExtra = iface;
                } else {
                    inetAddresses = iface.getInetAddresses();
                    if (inetAddresses.hasMoreElements() && ((InetAddress) inetAddresses.nextElement()).getHostAddress().toString().startsWith("192")) {
                        ifaceExtra = iface;
                    }
                }
            }
            if (ifaceExtra != null) {
                return ifaceExtra;
            }
            return null;
        } catch (SocketException e) {
            log.info("No network interfaces available");
            return null;
        }
    }

    static int byteArrayToInt(byte[] arr, int offset) {
        if (arr == null || arr.length - offset < 4) {
            return -1;
        }
        return ((((arr[offset] & 255) << 24) + ((arr[offset + 1] & 255) << 16)) + ((arr[offset + 2] & 255) << 8)) + (arr[offset + 3] & 255);
    }
}
