package org.teleal.cling.binding.annotations;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.URI;
import java.net.URL;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;
import org.teleal.cling.binding.LocalServiceBinder;
import org.teleal.cling.binding.LocalServiceBindingException;
import org.teleal.cling.model.ValidationError;
import org.teleal.cling.model.ValidationException;
import org.teleal.cling.model.action.ActionExecutor;
import org.teleal.cling.model.action.QueryStateVariableExecutor;
import org.teleal.cling.model.meta.Action;
import org.teleal.cling.model.meta.LocalService;
import org.teleal.cling.model.meta.QueryStateVariableAction;
import org.teleal.cling.model.meta.StateVariable;
import org.teleal.cling.model.state.FieldStateVariableAccessor;
import org.teleal.cling.model.state.GetterStateVariableAccessor;
import org.teleal.cling.model.state.StateVariableAccessor;
import org.teleal.cling.model.types.ServiceId;
import org.teleal.cling.model.types.ServiceType;
import org.teleal.cling.model.types.UDAServiceId;
import org.teleal.cling.model.types.UDAServiceType;
import org.teleal.cling.model.types.csv.CSV;
import org.teleal.common.util.Reflections;

public class AnnotationLocalServiceBinder implements LocalServiceBinder {
    private static Logger log = Logger.getLogger(AnnotationLocalServiceBinder.class.getName());

    public LocalService read(Class<?> clazz) throws LocalServiceBindingException {
        log.fine("Reading and binding annotations of service implementation class: " + clazz);
        if (clazz.isAnnotationPresent(UpnpService.class)) {
            UpnpService annotation = (UpnpService) clazz.getAnnotation(UpnpService.class);
            UpnpServiceId idAnnotation = annotation.serviceId();
            UpnpServiceType typeAnnotation = annotation.serviceType();
            return read((Class) clazz, idAnnotation.namespace().equals(UDAServiceId.DEFAULT_NAMESPACE) ? new UDAServiceId(idAnnotation.value()) : new ServiceId(idAnnotation.namespace(), idAnnotation.value()), typeAnnotation.namespace().equals("schemas-upnp-org") ? new UDAServiceType(typeAnnotation.value(), typeAnnotation.version()) : new ServiceType(typeAnnotation.namespace(), typeAnnotation.value(), typeAnnotation.version()), annotation.supportsQueryStateVariables(), readStringConvertibleTypes(annotation.stringConvertibleTypes()));
        }
        throw new LocalServiceBindingException("Given class is not an @UpnpService");
    }

    public LocalService read(Class<?> clazz, ServiceId id, ServiceType type, boolean supportsQueryStateVariables, Class[] stringConvertibleTypes) throws LocalServiceBindingException {
        return read((Class) clazz, id, type, supportsQueryStateVariables, new HashSet(Arrays.asList(stringConvertibleTypes)));
    }

    public LocalService read(Class<?> clazz, ServiceId id, ServiceType type, boolean supportsQueryStateVariables, Set<Class> stringConvertibleTypes) throws LocalServiceBindingException {
        Map<StateVariable, StateVariableAccessor> stateVariables = readStateVariables(clazz, stringConvertibleTypes);
        Map<Action, ActionExecutor> actions = readActions(clazz, stateVariables, stringConvertibleTypes);
        if (supportsQueryStateVariables) {
            actions.put(new QueryStateVariableAction(), new QueryStateVariableExecutor());
        }
        try {
            return new LocalService(type, id, actions, stateVariables, stringConvertibleTypes, supportsQueryStateVariables);
        } catch (ValidationException ex) {
            log.severe("Could not validate device model: " + ex.toString());
            for (ValidationError validationError : ex.getErrors()) {
                log.severe(validationError.toString());
            }
            throw new LocalServiceBindingException("Validation of model failed, check the log");
        }
    }

    protected Set<Class> readStringConvertibleTypes(Class[] declaredTypes) throws LocalServiceBindingException {
        Class[] arr$ = declaredTypes;
        int len$ = arr$.length;
        int i$ = 0;
        while (i$ < len$) {
            Class stringConvertibleType = arr$[i$];
            if (Modifier.isPublic(stringConvertibleType.getModifiers())) {
                try {
                    stringConvertibleType.getConstructor(new Class[]{String.class});
                    i$++;
                } catch (NoSuchMethodException e) {
                    throw new LocalServiceBindingException("Declared string-convertible type needs a public single-argument String constructor: " + stringConvertibleType);
                }
            }
            throw new LocalServiceBindingException("Declared string-convertible type must be public: " + stringConvertibleType);
        }
        Set<Class> stringConvertibleTypes = new HashSet(Arrays.asList(declaredTypes));
        stringConvertibleTypes.add(URI.class);
        stringConvertibleTypes.add(URL.class);
        stringConvertibleTypes.add(CSV.class);
        return stringConvertibleTypes;
    }

    protected Map<StateVariable, StateVariableAccessor> readStateVariables(Class<?> clazz, Set<Class> stringConvertibleTypes) throws LocalServiceBindingException {
        Method getter;
        Field field;
        StateVariableAccessor accessor;
        Map<StateVariable, StateVariableAccessor> map = new HashMap();
        if (clazz.isAnnotationPresent(UpnpStateVariables.class)) {
            UpnpStateVariables variables = (UpnpStateVariables) clazz.getAnnotation(UpnpStateVariables.class);
            for (UpnpStateVariable v : variables.value()) {
                if (v.name().length() == 0) {
                    throw new LocalServiceBindingException("Class-level @UpnpStateVariable name attribute value required");
                }
                String javaPropertyName = toJavaStateVariableName(v.name());
                getter = Reflections.getGetterMethod(clazz, javaPropertyName);
                field = Reflections.getField(clazz, javaPropertyName);
                accessor = null;
                if (getter != null && field != null) {
                    accessor = variables.preferFields() ? new FieldStateVariableAccessor(field) : new GetterStateVariableAccessor(getter);
                } else if (field != null) {
                    accessor = new FieldStateVariableAccessor(field);
                } else if (getter != null) {
                    accessor = new GetterStateVariableAccessor(getter);
                } else {
                    log.finer("No field or getter found for state variable, skipping accessor: " + v.name());
                }
                map.put(new AnnotationStateVariableBinder(v, v.name(), accessor, stringConvertibleTypes).createStateVariable(), accessor);
            }
        }
        for (Field field2 : Reflections.getFields(clazz, UpnpStateVariable.class)) {
            UpnpStateVariable svAnnotation = (UpnpStateVariable) field2.getAnnotation(UpnpStateVariable.class);
            accessor = new FieldStateVariableAccessor(field2);
            map.put(new AnnotationStateVariableBinder(svAnnotation, svAnnotation.name().length() == 0 ? toUpnpStateVariableName(field2.getName()) : svAnnotation.name(), accessor, stringConvertibleTypes).createStateVariable(), accessor);
        }
        for (Method getter2 : Reflections.getMethods(clazz, UpnpStateVariable.class)) {
            String propertyName = Reflections.getMethodPropertyName(getter2.getName());
            if (propertyName == null) {
                throw new LocalServiceBindingException("Annotated method is not a getter method (: " + getter2);
            } else if (getter2.getParameterTypes().length > 0) {
                throw new LocalServiceBindingException("Getter method defined as @UpnpStateVariable can not have parameters: " + getter2);
            } else {
                svAnnotation = (UpnpStateVariable) getter2.getAnnotation(UpnpStateVariable.class);
                accessor = new GetterStateVariableAccessor(getter2);
                map.put(new AnnotationStateVariableBinder(svAnnotation, svAnnotation.name().length() == 0 ? toUpnpStateVariableName(propertyName) : svAnnotation.name(), accessor, stringConvertibleTypes).createStateVariable(), accessor);
            }
        }
        return map;
    }

    protected Map<Action, ActionExecutor> readActions(Class<?> clazz, Map<StateVariable, StateVariableAccessor> stateVariables, Set<Class> stringConvertibleTypes) throws LocalServiceBindingException {
        Map<Action, ActionExecutor> map = new HashMap();
        for (Method method : Reflections.getMethods(clazz, UpnpAction.class)) {
            new AnnotationActionBinder(method, stateVariables, stringConvertibleTypes).appendAction(map);
        }
        return map;
    }

    static String toUpnpStateVariableName(String javaName) {
        if (javaName.length() >= 1) {
            return javaName.substring(0, 1).toUpperCase() + javaName.substring(1);
        }
        throw new IllegalArgumentException("Variable name must be at least 1 character long");
    }

    static String toJavaStateVariableName(String upnpName) {
        if (upnpName.length() >= 1) {
            return upnpName.substring(0, 1).toLowerCase() + upnpName.substring(1);
        }
        throw new IllegalArgumentException("Variable name must be at least 1 character long");
    }

    static String toUpnpActionName(String javaName) {
        if (javaName.length() >= 1) {
            return javaName.substring(0, 1).toUpperCase() + javaName.substring(1);
        }
        throw new IllegalArgumentException("Action name must be at least 1 character long");
    }

    static String toJavaActionName(String upnpName) {
        if (upnpName.length() >= 1) {
            return upnpName.substring(0, 1).toLowerCase() + upnpName.substring(1);
        }
        throw new IllegalArgumentException("Variable name must be at least 1 character long");
    }
}
