package org.teleal.cling.binding.xml;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilderFactory;
import org.teleal.cling.binding.staging.MutableAction;
import org.teleal.cling.binding.staging.MutableActionArgument;
import org.teleal.cling.binding.staging.MutableAllowedValueRange;
import org.teleal.cling.binding.staging.MutableService;
import org.teleal.cling.binding.staging.MutableStateVariable;
import org.teleal.cling.binding.xml.Descriptor.Service.ATTRIBUTE;
import org.teleal.cling.binding.xml.Descriptor.Service.ELEMENT;
import org.teleal.cling.model.ValidationException;
import org.teleal.cling.model.XMLUtil;
import org.teleal.cling.model.meta.Action;
import org.teleal.cling.model.meta.ActionArgument;
import org.teleal.cling.model.meta.ActionArgument.Direction;
import org.teleal.cling.model.meta.QueryStateVariableAction;
import org.teleal.cling.model.meta.RemoteService;
import org.teleal.cling.model.meta.Service;
import org.teleal.cling.model.meta.StateVariable;
import org.teleal.cling.model.meta.StateVariableEventDetails;
import org.teleal.cling.model.types.CustomDatatype;
import org.teleal.cling.model.types.Datatype.Builtin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class UDA10ServiceDescriptorBinderImpl implements ServiceDescriptorBinder {
    private static Logger log = Logger.getLogger(ServiceDescriptorBinder.class.getName());

    public <S extends Service> S describe(S undescribedService, String descriptorXml) throws DescriptorBindingException, ValidationException {
        if (descriptorXml == null || descriptorXml.length() == 0) {
            throw new DescriptorBindingException("Null or empty descriptor");
        }
        try {
            log.fine("Populating service from XML descriptor: " + undescribedService);
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            return describe((Service) undescribedService, factory.newDocumentBuilder().parse(new InputSource(new StringReader(descriptorXml.trim()))));
        } catch (ValidationException ex) {
            throw ex;
        } catch (Exception ex2) {
            throw new DescriptorBindingException("Could not parse service descriptor: " + ex2.toString(), ex2);
        }
    }

    public <S extends Service> S describe(S undescribedService, Document dom) throws DescriptorBindingException, ValidationException {
        try {
            log.fine("Populating service from DOM: " + undescribedService);
            MutableService descriptor = new MutableService();
            hydrateBasic(descriptor, undescribedService);
            hydrateRoot(descriptor, dom.getDocumentElement());
            return buildInstance(undescribedService, descriptor);
        } catch (ValidationException ex) {
            throw ex;
        } catch (Exception ex2) {
            throw new DescriptorBindingException("Could not parse service DOM: " + ex2.toString(), ex2);
        }
    }

    protected <S extends Service> S buildInstance(S undescribedService, MutableService descriptor) throws ValidationException {
        return descriptor.build(undescribedService.getDevice());
    }

    protected void hydrateBasic(MutableService descriptor, Service undescribedService) {
        descriptor.serviceId = undescribedService.getServiceId();
        descriptor.serviceType = undescribedService.getServiceType();
        if (undescribedService instanceof RemoteService) {
            RemoteService rs = (RemoteService) undescribedService;
            descriptor.controlURI = rs.getControlURI();
            descriptor.eventSubscriptionURI = rs.getEventSubscriptionURI();
            descriptor.descriptorURI = rs.getDescriptorURI();
        }
    }

    protected void hydrateRoot(MutableService descriptor, Element rootElement) throws DescriptorBindingException {
        if (ELEMENT.scpd.equals(rootElement)) {
            NodeList rootChildren = rootElement.getChildNodes();
            for (int i = 0; i < rootChildren.getLength(); i++) {
                Node rootChild = rootChildren.item(i);
                if (rootChild.getNodeType() == (short) 1 && !ELEMENT.specVersion.equals(rootChild)) {
                    if (ELEMENT.actionList.equals(rootChild)) {
                        hydrateActionList(descriptor, rootChild);
                    } else if (ELEMENT.serviceStateTable.equals(rootChild)) {
                        hydrateServiceStateTableList(descriptor, rootChild);
                    } else {
                        log.finer("Ignoring unknown element: " + rootChild.getNodeName());
                    }
                }
            }
            return;
        }
        throw new DescriptorBindingException("Root element name is not <scpd>: " + rootElement.getNodeName());
    }

    public void hydrateActionList(MutableService descriptor, Node actionListNode) throws DescriptorBindingException {
        NodeList actionListChildren = actionListNode.getChildNodes();
        for (int i = 0; i < actionListChildren.getLength(); i++) {
            Node actionListChild = actionListChildren.item(i);
            if (actionListChild.getNodeType() == (short) 1 && ELEMENT.action.equals(actionListChild)) {
                MutableAction action = new MutableAction();
                hydrateAction(action, actionListChild);
                descriptor.actions.add(action);
            }
        }
    }

    public void hydrateAction(MutableAction action, Node actionNode) {
        NodeList actionNodeChildren = actionNode.getChildNodes();
        for (int i = 0; i < actionNodeChildren.getLength(); i++) {
            Node actionNodeChild = actionNodeChildren.item(i);
            if (actionNodeChild.getNodeType() == (short) 1) {
                if (ELEMENT.name.equals(actionNodeChild)) {
                    action.name = XMLUtil.getTextContent(actionNodeChild);
                } else if (ELEMENT.argumentList.equals(actionNodeChild)) {
                    NodeList argumentChildren = actionNodeChild.getChildNodes();
                    for (int j = 0; j < argumentChildren.getLength(); j++) {
                        Node argumentChild = argumentChildren.item(j);
                        if (argumentChild.getNodeType() == (short) 1) {
                            MutableActionArgument actionArgument = new MutableActionArgument();
                            hydrateActionArgument(actionArgument, argumentChild);
                            action.arguments.add(actionArgument);
                        }
                    }
                }
            }
        }
    }

    public void hydrateActionArgument(MutableActionArgument actionArgument, Node actionArgumentNode) {
        NodeList argumentNodeChildren = actionArgumentNode.getChildNodes();
        for (int i = 0; i < argumentNodeChildren.getLength(); i++) {
            Node argumentNodeChild = argumentNodeChildren.item(i);
            if (argumentNodeChild.getNodeType() == (short) 1) {
                if (ELEMENT.name.equals(argumentNodeChild)) {
                    actionArgument.name = XMLUtil.getTextContent(argumentNodeChild);
                } else if (ELEMENT.direction.equals(argumentNodeChild)) {
                    actionArgument.direction = Direction.valueOf(XMLUtil.getTextContent(argumentNodeChild).toUpperCase());
                } else if (ELEMENT.relatedStateVariable.equals(argumentNodeChild)) {
                    actionArgument.relatedStateVariable = XMLUtil.getTextContent(argumentNodeChild);
                } else if (ELEMENT.retval.equals(argumentNodeChild)) {
                    actionArgument.retval = true;
                }
            }
        }
    }

    public void hydrateServiceStateTableList(MutableService descriptor, Node serviceStateTableNode) {
        NodeList serviceStateTableChildren = serviceStateTableNode.getChildNodes();
        for (int i = 0; i < serviceStateTableChildren.getLength(); i++) {
            Node serviceStateTableChild = serviceStateTableChildren.item(i);
            if (serviceStateTableChild.getNodeType() == (short) 1 && ELEMENT.stateVariable.equals(serviceStateTableChild)) {
                MutableStateVariable stateVariable = new MutableStateVariable();
                hydrateStateVariable(stateVariable, (Element) serviceStateTableChild);
                descriptor.stateVariables.add(stateVariable);
            }
        }
    }

    public void hydrateStateVariable(MutableStateVariable stateVariable, Element stateVariableElement) {
        boolean z;
        NodeList stateVariableChildren;
        int i;
        Node stateVariableChild;
        List<String> allowedValues;
        NodeList allowedValueListChildren;
        int j;
        Node allowedValueListChild;
        MutableAllowedValueRange range;
        NodeList allowedValueRangeChildren;
        Node allowedValueRangeChild;
        if (stateVariableElement.getAttribute("sendEvents") != null) {
            if (stateVariableElement.getAttribute(ATTRIBUTE.sendEvents.toString()).toUpperCase().equals("YES")) {
                z = true;
                stateVariable.eventDetails = new StateVariableEventDetails(z);
                stateVariableChildren = stateVariableElement.getChildNodes();
                for (i = 0; i < stateVariableChildren.getLength(); i++) {
                    stateVariableChild = stateVariableChildren.item(i);
                    if (stateVariableChild.getNodeType() == (short) 1) {
                        if (ELEMENT.name.equals(stateVariableChild)) {
                            stateVariable.name = XMLUtil.getTextContent(stateVariableChild);
                        } else if (ELEMENT.dataType.equals(stateVariableChild)) {
                            String dtName = XMLUtil.getTextContent(stateVariableChild);
                            Builtin builtin = Builtin.getByDescriptorName(dtName);
                            stateVariable.dataType = builtin == null ? builtin.getDatatype() : new CustomDatatype(dtName);
                        } else if (ELEMENT.defaultValue.equals(stateVariableChild)) {
                            stateVariable.defaultValue = XMLUtil.getTextContent(stateVariableChild);
                        } else if (ELEMENT.allowedValueList.equals(stateVariableChild)) {
                            allowedValues = new ArrayList();
                            allowedValueListChildren = stateVariableChild.getChildNodes();
                            for (j = 0; j < allowedValueListChildren.getLength(); j++) {
                                allowedValueListChild = allowedValueListChildren.item(j);
                                if (allowedValueListChild.getNodeType() == (short) 1 && ELEMENT.allowedValue.equals(allowedValueListChild)) {
                                    allowedValues.add(XMLUtil.getTextContent(allowedValueListChild));
                                }
                            }
                            stateVariable.allowedValues = allowedValues;
                        } else if (ELEMENT.allowedValueRange.equals(stateVariableChild)) {
                            range = new MutableAllowedValueRange();
                            allowedValueRangeChildren = stateVariableChild.getChildNodes();
                            for (j = 0; j < allowedValueRangeChildren.getLength(); j++) {
                                allowedValueRangeChild = allowedValueRangeChildren.item(j);
                                if (allowedValueRangeChild.getNodeType() == (short) 1) {
                                    if (ELEMENT.minimum.equals(allowedValueRangeChild)) {
                                        try {
                                            range.minimum = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                                        } catch (Exception e) {
                                        }
                                    } else if (ELEMENT.maximum.equals(allowedValueRangeChild)) {
                                        try {
                                            range.maximum = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                                        } catch (Exception e2) {
                                        }
                                    } else if (ELEMENT.step.equals(allowedValueRangeChild)) {
                                        try {
                                            range.step = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                                        } catch (Exception e3) {
                                        }
                                    }
                                }
                            }
                            stateVariable.allowedValueRange = range;
                        }
                    }
                }
            }
        }
        z = false;
        stateVariable.eventDetails = new StateVariableEventDetails(z);
        stateVariableChildren = stateVariableElement.getChildNodes();
        for (i = 0; i < stateVariableChildren.getLength(); i++) {
            stateVariableChild = stateVariableChildren.item(i);
            if (stateVariableChild.getNodeType() == (short) 1) {
                if (ELEMENT.name.equals(stateVariableChild)) {
                    stateVariable.name = XMLUtil.getTextContent(stateVariableChild);
                } else if (ELEMENT.dataType.equals(stateVariableChild)) {
                    String dtName2 = XMLUtil.getTextContent(stateVariableChild);
                    Builtin builtin2 = Builtin.getByDescriptorName(dtName2);
                    if (builtin2 == null) {
                    }
                    stateVariable.dataType = builtin2 == null ? builtin2.getDatatype() : new CustomDatatype(dtName2);
                } else if (ELEMENT.defaultValue.equals(stateVariableChild)) {
                    stateVariable.defaultValue = XMLUtil.getTextContent(stateVariableChild);
                } else if (ELEMENT.allowedValueList.equals(stateVariableChild)) {
                    allowedValues = new ArrayList();
                    allowedValueListChildren = stateVariableChild.getChildNodes();
                    for (j = 0; j < allowedValueListChildren.getLength(); j++) {
                        allowedValueListChild = allowedValueListChildren.item(j);
                        allowedValues.add(XMLUtil.getTextContent(allowedValueListChild));
                    }
                    stateVariable.allowedValues = allowedValues;
                } else if (ELEMENT.allowedValueRange.equals(stateVariableChild)) {
                    range = new MutableAllowedValueRange();
                    allowedValueRangeChildren = stateVariableChild.getChildNodes();
                    for (j = 0; j < allowedValueRangeChildren.getLength(); j++) {
                        allowedValueRangeChild = allowedValueRangeChildren.item(j);
                        if (allowedValueRangeChild.getNodeType() == (short) 1) {
                            if (ELEMENT.minimum.equals(allowedValueRangeChild)) {
                                range.minimum = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                            } else if (ELEMENT.maximum.equals(allowedValueRangeChild)) {
                                range.maximum = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                            } else if (ELEMENT.step.equals(allowedValueRangeChild)) {
                                range.step = Long.valueOf(XMLUtil.getTextContent(allowedValueRangeChild));
                            }
                        }
                    }
                    stateVariable.allowedValueRange = range;
                }
            }
        }
    }

    public String generate(Service service) throws DescriptorBindingException {
        try {
            log.fine("Generating XML descriptor from service model: " + service);
            return XMLUtil.documentToString(buildDOM(service));
        } catch (Exception ex) {
            throw new DescriptorBindingException("Could not build DOM: " + ex.getMessage(), ex);
        }
    }

    public Document buildDOM(Service service) throws DescriptorBindingException {
        try {
            log.fine("Generting XML descriptor from service model: " + service);
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            Document d = factory.newDocumentBuilder().newDocument();
            generateScpd(service, d);
            return d;
        } catch (Exception ex) {
            throw new DescriptorBindingException("Could not generate service descriptor: " + ex.getMessage(), ex);
        }
    }

    private void generateScpd(Service serviceModel, Document descriptor) {
        Element scpdElement = descriptor.createElementNS(Descriptor.Service.NAMESPACE_URI, ELEMENT.scpd.toString());
        descriptor.appendChild(scpdElement);
        generateSpecVersion(serviceModel, descriptor, scpdElement);
        if (serviceModel.hasActions()) {
            generateActionList(serviceModel, descriptor, scpdElement);
        }
        generateServiceStateTable(serviceModel, descriptor, scpdElement);
    }

    private void generateSpecVersion(Service serviceModel, Document descriptor, Element rootElement) {
        Element specVersionElement = XMLUtil.appendNewElement(descriptor, rootElement, ELEMENT.specVersion);
        XMLUtil.appendNewElementIfNotNull(descriptor, specVersionElement, ELEMENT.major, Integer.valueOf(serviceModel.getDevice().getVersion().getMajor()));
        XMLUtil.appendNewElementIfNotNull(descriptor, specVersionElement, ELEMENT.minor, Integer.valueOf(serviceModel.getDevice().getVersion().getMinor()));
    }

    private void generateActionList(Service serviceModel, Document descriptor, Element scpdElement) {
        Element actionListElement = XMLUtil.appendNewElement(descriptor, scpdElement, ELEMENT.actionList);
        for (Action action : serviceModel.getActions()) {
            if (!action.getName().equals(QueryStateVariableAction.ACTION_NAME)) {
                generateAction(action, descriptor, actionListElement);
            }
        }
    }

    private void generateAction(Action action, Document descriptor, Element actionListElement) {
        Element actionElement = XMLUtil.appendNewElement(descriptor, actionListElement, ELEMENT.action);
        XMLUtil.appendNewElementIfNotNull(descriptor, actionElement, ELEMENT.name, action.getName());
        if (action.hasArguments()) {
            Element argumentListElement = XMLUtil.appendNewElement(descriptor, actionElement, ELEMENT.argumentList);
            for (ActionArgument actionArgument : action.getArguments()) {
                generateActionArgument(actionArgument, descriptor, argumentListElement);
            }
        }
    }

    private void generateActionArgument(ActionArgument actionArgument, Document descriptor, Element actionElement) {
        Element actionArgumentElement = XMLUtil.appendNewElement(descriptor, actionElement, ELEMENT.argument);
        XMLUtil.appendNewElementIfNotNull(descriptor, actionArgumentElement, ELEMENT.name, actionArgument.getName());
        XMLUtil.appendNewElementIfNotNull(descriptor, actionArgumentElement, ELEMENT.direction, actionArgument.getDirection().toString().toLowerCase());
        XMLUtil.appendNewElementIfNotNull(descriptor, actionArgumentElement, ELEMENT.relatedStateVariable, actionArgument.getRelatedStateVariableName());
        if (actionArgument.isReturnValue()) {
            XMLUtil.appendNewElement(descriptor, actionArgumentElement, ELEMENT.retval);
        }
    }

    private void generateServiceStateTable(Service serviceModel, Document descriptor, Element scpdElement) {
        Element serviceStateTableElement = XMLUtil.appendNewElement(descriptor, scpdElement, ELEMENT.serviceStateTable);
        for (StateVariable stateVariable : serviceModel.getStateVariables()) {
            generateStateVariable(stateVariable, descriptor, serviceStateTableElement);
        }
    }

    private void generateStateVariable(StateVariable stateVariable, Document descriptor, Element serviveStateTableElement) {
        Element stateVariableElement = XMLUtil.appendNewElement(descriptor, serviveStateTableElement, ELEMENT.stateVariable);
        XMLUtil.appendNewElementIfNotNull(descriptor, stateVariableElement, ELEMENT.name, stateVariable.getName());
        if (stateVariable.getTypeDetails().getDatatype() instanceof CustomDatatype) {
            XMLUtil.appendNewElementIfNotNull(descriptor, stateVariableElement, ELEMENT.dataType, ((CustomDatatype) stateVariable.getTypeDetails().getDatatype()).getName());
        } else {
            XMLUtil.appendNewElementIfNotNull(descriptor, stateVariableElement, ELEMENT.dataType, stateVariable.getTypeDetails().getDatatype().getBuiltin().getDescriptorName());
        }
        XMLUtil.appendNewElementIfNotNull(descriptor, stateVariableElement, ELEMENT.defaultValue, stateVariable.getTypeDetails().getDefaultValue());
        if (stateVariable.getEventDetails().isSendEvents()) {
            stateVariableElement.setAttribute(ATTRIBUTE.sendEvents.toString(), "yes");
        } else {
            stateVariableElement.setAttribute(ATTRIBUTE.sendEvents.toString(), "no");
        }
        if (stateVariable.getTypeDetails().getAllowedValues() != null) {
            Element allowedValueListElement = XMLUtil.appendNewElement(descriptor, stateVariableElement, ELEMENT.allowedValueList);
            for (Object allowedValue : stateVariable.getTypeDetails().getAllowedValues()) {
                XMLUtil.appendNewElementIfNotNull(descriptor, allowedValueListElement, ELEMENT.allowedValue, allowedValue);
            }
        }
        if (stateVariable.getTypeDetails().getAllowedValueRange() != null) {
            Element allowedValueRangeElement = XMLUtil.appendNewElement(descriptor, stateVariableElement, ELEMENT.allowedValueRange);
            XMLUtil.appendNewElementIfNotNull(descriptor, allowedValueRangeElement, ELEMENT.minimum, Long.valueOf(stateVariable.getTypeDetails().getAllowedValueRange().getMinimum()));
            XMLUtil.appendNewElementIfNotNull(descriptor, allowedValueRangeElement, ELEMENT.maximum, Long.valueOf(stateVariable.getTypeDetails().getAllowedValueRange().getMaximum()));
            if (stateVariable.getTypeDetails().getAllowedValueRange().getStep() > 1) {
                XMLUtil.appendNewElementIfNotNull(descriptor, allowedValueRangeElement, ELEMENT.step, Long.valueOf(stateVariable.getTypeDetails().getAllowedValueRange().getStep()));
            }
        }
    }
}
