package org.teleal.cling.binding.xml;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import org.teleal.cling.binding.staging.MutableAction;
import org.teleal.cling.binding.staging.MutableActionArgument;
import org.teleal.cling.binding.staging.MutableAllowedValueRange;
import org.teleal.cling.binding.staging.MutableService;
import org.teleal.cling.binding.staging.MutableStateVariable;
import org.teleal.cling.binding.xml.Descriptor.Service.ATTRIBUTE;
import org.teleal.cling.binding.xml.Descriptor.Service.ELEMENT;
import org.teleal.cling.model.ValidationException;
import org.teleal.cling.model.meta.ActionArgument.Direction;
import org.teleal.cling.model.meta.Service;
import org.teleal.cling.model.meta.StateVariableEventDetails;
import org.teleal.cling.model.types.CustomDatatype;
import org.teleal.cling.model.types.Datatype.Builtin;
import org.teleal.common.xml.SAXParser;
import org.teleal.common.xml.SAXParser.Handler;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class UDA10ServiceDescriptorBinderSAXImpl extends UDA10ServiceDescriptorBinderImpl {
    private static Logger log = Logger.getLogger(ServiceDescriptorBinder.class.getName());

    protected static class ServiceDescriptorHandler<I> extends Handler<I> {
        public ServiceDescriptorHandler(I instance) {
            super(instance);
        }

        public ServiceDescriptorHandler(I instance, SAXParser parser) {
            super((Object) instance, parser);
        }

        public ServiceDescriptorHandler(I instance, ServiceDescriptorHandler parent) {
            super((Object) instance, (Handler) parent);
        }

        public ServiceDescriptorHandler(I instance, SAXParser parser, ServiceDescriptorHandler parent) {
            super(instance, parser, parent);
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            ELEMENT el = ELEMENT.valueOrNullOf(localName);
            if (el != null) {
                startElement(el, attributes);
            }
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            super.endElement(uri, localName, qName);
            ELEMENT el = ELEMENT.valueOrNullOf(localName);
            if (el != null) {
                endElement(el);
            }
        }

        protected boolean isLastElement(String uri, String localName, String qName) {
            ELEMENT el = ELEMENT.valueOrNullOf(localName);
            return el != null && isLastElement(el);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
        }

        public void endElement(ELEMENT element) throws SAXException {
        }

        public boolean isLastElement(ELEMENT element) {
            return false;
        }
    }

    protected static class ActionArgumentHandler extends ServiceDescriptorHandler<MutableActionArgument> {
        public static final ELEMENT EL = ELEMENT.argument;

        public ActionArgumentHandler(MutableActionArgument instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void endElement(ELEMENT element) throws SAXException {
            switch (element) {
                case name:
                    ((MutableActionArgument) getInstance()).name = getCharacters();
                    return;
                case direction:
                    ((MutableActionArgument) getInstance()).direction = Direction.valueOf(getCharacters().toUpperCase());
                    return;
                case relatedStateVariable:
                    ((MutableActionArgument) getInstance()).relatedStateVariable = getCharacters();
                    return;
                case retval:
                    ((MutableActionArgument) getInstance()).retval = true;
                    return;
                default:
                    return;
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class ActionArgumentListHandler extends ServiceDescriptorHandler<List<MutableActionArgument>> {
        public static final ELEMENT EL = ELEMENT.argumentList;

        public ActionArgumentListHandler(List<MutableActionArgument> instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(ActionArgumentHandler.EL)) {
                MutableActionArgument argument = new MutableActionArgument();
                ((List) getInstance()).add(argument);
                ActionArgumentHandler actionArgumentHandler = new ActionArgumentHandler(argument, this);
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class ActionHandler extends ServiceDescriptorHandler<MutableAction> {
        public static final ELEMENT EL = ELEMENT.action;

        public ActionHandler(MutableAction instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(ActionArgumentListHandler.EL)) {
                List<MutableActionArgument> arguments = new ArrayList();
                ((MutableAction) getInstance()).arguments = arguments;
                ActionArgumentListHandler actionArgumentListHandler = new ActionArgumentListHandler(arguments, this);
            }
        }

        public void endElement(ELEMENT element) throws SAXException {
            switch (element) {
                case name:
                    ((MutableAction) getInstance()).name = getCharacters();
                    return;
                default:
                    return;
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class ActionListHandler extends ServiceDescriptorHandler<List<MutableAction>> {
        public static final ELEMENT EL = ELEMENT.actionList;

        public ActionListHandler(List<MutableAction> instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(ActionHandler.EL)) {
                MutableAction action = new MutableAction();
                ((List) getInstance()).add(action);
                ActionHandler actionHandler = new ActionHandler(action, this);
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class AllowedValueListHandler extends ServiceDescriptorHandler<List<String>> {
        public static final ELEMENT EL = ELEMENT.allowedValueList;

        public AllowedValueListHandler(List<String> instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void endElement(ELEMENT element) throws SAXException {
            switch (element) {
                case allowedValue:
                    ((List) getInstance()).add(getCharacters());
                    return;
                default:
                    return;
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class AllowedValueRangeHandler extends ServiceDescriptorHandler<MutableAllowedValueRange> {
        public static final ELEMENT EL = ELEMENT.allowedValueRange;

        public AllowedValueRangeHandler(MutableAllowedValueRange instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void endElement(ELEMENT element) throws SAXException {
            try {
                switch (element) {
                    case minimum:
                        ((MutableAllowedValueRange) getInstance()).minimum = Long.valueOf(getCharacters());
                        return;
                    case maximum:
                        ((MutableAllowedValueRange) getInstance()).maximum = Long.valueOf(getCharacters());
                        return;
                    case step:
                        ((MutableAllowedValueRange) getInstance()).step = Long.valueOf(getCharacters());
                        return;
                    default:
                        return;
                }
            } catch (Exception e) {
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class RootHandler extends ServiceDescriptorHandler<MutableService> {
        public RootHandler(MutableService instance, SAXParser parser) {
            super((Object) instance, parser);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(ActionListHandler.EL)) {
                List<MutableAction> actions = new ArrayList();
                ((MutableService) getInstance()).actions = actions;
                ActionListHandler actionListHandler = new ActionListHandler(actions, this);
            }
            if (element.equals(StateVariableListHandler.EL)) {
                List<MutableStateVariable> stateVariables = new ArrayList();
                ((MutableService) getInstance()).stateVariables = stateVariables;
                StateVariableListHandler stateVariableListHandler = new StateVariableListHandler(stateVariables, this);
            }
        }
    }

    protected static class StateVariableHandler extends ServiceDescriptorHandler<MutableStateVariable> {
        public static final ELEMENT EL = ELEMENT.stateVariable;

        public StateVariableHandler(MutableStateVariable instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(AllowedValueListHandler.EL)) {
                List<String> allowedValues = new ArrayList();
                ((MutableStateVariable) getInstance()).allowedValues = allowedValues;
                AllowedValueListHandler allowedValueListHandler = new AllowedValueListHandler(allowedValues, this);
            }
            if (element.equals(AllowedValueRangeHandler.EL)) {
                MutableAllowedValueRange allowedValueRange = new MutableAllowedValueRange();
                ((MutableStateVariable) getInstance()).allowedValueRange = allowedValueRange;
                AllowedValueRangeHandler allowedValueRangeHandler = new AllowedValueRangeHandler(allowedValueRange, this);
            }
        }

        public void endElement(ELEMENT element) throws SAXException {
            switch (element) {
                case name:
                    ((MutableStateVariable) getInstance()).name = getCharacters();
                    return;
                case dataType:
                    String dtName = getCharacters();
                    Builtin builtin = Builtin.getByDescriptorName(dtName);
                    ((MutableStateVariable) getInstance()).dataType = builtin != null ? builtin.getDatatype() : new CustomDatatype(dtName);
                    return;
                case defaultValue:
                    ((MutableStateVariable) getInstance()).defaultValue = getCharacters();
                    return;
                default:
                    return;
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    protected static class StateVariableListHandler extends ServiceDescriptorHandler<List<MutableStateVariable>> {
        public static final ELEMENT EL = ELEMENT.serviceStateTable;

        public StateVariableListHandler(List<MutableStateVariable> instance, ServiceDescriptorHandler parent) {
            super((Object) instance, parent);
        }

        public void startElement(ELEMENT element, Attributes attributes) throws SAXException {
            if (element.equals(StateVariableHandler.EL)) {
                MutableStateVariable stateVariable = new MutableStateVariable();
                String sendEventsAttributeValue = attributes.getValue(ATTRIBUTE.sendEvents.toString());
                boolean z = sendEventsAttributeValue != null && sendEventsAttributeValue.toUpperCase().equals("YES");
                stateVariable.eventDetails = new StateVariableEventDetails(z);
                ((List) getInstance()).add(stateVariable);
                StateVariableHandler stateVariableHandler = new StateVariableHandler(stateVariable, this);
            }
        }

        public boolean isLastElement(ELEMENT element) {
            return element.equals(EL);
        }
    }

    public <S extends Service> S describe(S undescribedService, String descriptorXml) throws DescriptorBindingException, ValidationException {
        if (descriptorXml == null || descriptorXml.length() == 0) {
            throw new DescriptorBindingException("Null or empty descriptor");
        }
        try {
            log.fine("Reading service from XML descriptor");
            SAXParser parser = new SAXParser();
            MutableService descriptor = new MutableService();
            hydrateBasic(descriptor, undescribedService);
            RootHandler rootHandler = new RootHandler(descriptor, parser);
            parser.parse(new InputSource(new StringReader(descriptorXml.trim())));
            return descriptor.build(undescribedService.getDevice());
        } catch (ValidationException ex) {
            throw ex;
        } catch (Exception ex2) {
            throw new DescriptorBindingException("Could not parse service descriptor: " + ex2.toString(), ex2);
        }
    }
}
