package org.teleal.cling.mock;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Executor;
import org.teleal.cling.DefaultUpnpServiceConfiguration;
import org.teleal.cling.UpnpService;
import org.teleal.cling.UpnpServiceConfiguration;
import org.teleal.cling.controlpoint.ControlPoint;
import org.teleal.cling.controlpoint.ControlPointImpl;
import org.teleal.cling.model.NetworkAddress;
import org.teleal.cling.model.message.IncomingDatagramMessage;
import org.teleal.cling.model.message.OutgoingDatagramMessage;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.header.UpnpHeader;
import org.teleal.cling.model.meta.LocalDevice;
import org.teleal.cling.protocol.ProtocolFactory;
import org.teleal.cling.protocol.ProtocolFactoryImpl;
import org.teleal.cling.protocol.async.SendingNotificationAlive;
import org.teleal.cling.protocol.async.SendingSearch;
import org.teleal.cling.registry.Registry;
import org.teleal.cling.registry.RegistryImpl;
import org.teleal.cling.registry.RegistryMaintainer;
import org.teleal.cling.transport.Router;
import org.teleal.cling.transport.impl.NetworkAddressFactoryImpl;
import org.teleal.cling.transport.spi.NetworkAddressFactory;
import org.teleal.cling.transport.spi.StreamClient;
import org.teleal.cling.transport.spi.UpnpStream;

public class MockUpnpService implements UpnpService {
    private List<byte[]> broadcastedBytes;
    protected final UpnpServiceConfiguration configuration;
    protected final ControlPoint controlPoint;
    private List<IncomingDatagramMessage> incomingDatagramMessages;
    protected final NetworkAddressFactory networkAddressFactory;
    private List<OutgoingDatagramMessage> outgoingDatagramMessages;
    protected final ProtocolFactory protocolFactory;
    private List<UpnpStream> receivedUpnpStreams;
    protected final Registry registry;
    protected final Router router;
    private List<StreamRequestMessage> sentStreamRequestMessages;

    public class MockRouter implements Router {
        int counter = -1;

        public UpnpServiceConfiguration getConfiguration() {
            return MockUpnpService.this.configuration;
        }

        public ProtocolFactory getProtocolFactory() {
            return MockUpnpService.this.protocolFactory;
        }

        public StreamClient getStreamClient() {
            return null;
        }

        public NetworkAddressFactory getNetworkAddressFactory() {
            return MockUpnpService.this.networkAddressFactory;
        }

        public List<NetworkAddress> getActiveStreamServers(InetAddress preferredAddress) {
            try {
                return Arrays.asList(new NetworkAddress[]{new NetworkAddress(InetAddress.getByName("127.0.0.1"), 0)});
            } catch (UnknownHostException ex) {
                throw new RuntimeException(ex);
            }
        }

        public void shutdown() {
        }

        public void received(IncomingDatagramMessage msg) {
            MockUpnpService.this.incomingDatagramMessages.add(msg);
        }

        public void received(UpnpStream stream) {
            MockUpnpService.this.receivedUpnpStreams.add(stream);
        }

        public void send(OutgoingDatagramMessage msg) {
            MockUpnpService.this.outgoingDatagramMessages.add(msg);
        }

        public StreamResponseMessage send(StreamRequestMessage msg) {
            MockUpnpService.this.sentStreamRequestMessages.add(msg);
            this.counter++;
            return MockUpnpService.this.getStreamResponseMessages() != null ? MockUpnpService.this.getStreamResponseMessages()[this.counter] : MockUpnpService.this.getStreamResponseMessage(msg);
        }

        public void broadcast(byte[] bytes) {
            MockUpnpService.this.broadcastedBytes.add(bytes);
        }
    }

    public static class MockProtocolFactory extends ProtocolFactoryImpl {
        private boolean sendsAlive;

        public MockProtocolFactory(UpnpService upnpService, boolean sendsAlive) {
            super(upnpService);
            this.sendsAlive = sendsAlive;
        }

        public SendingNotificationAlive createSendingNotificationAlive(LocalDevice localDevice) {
            return new SendingNotificationAlive(getUpnpService(), localDevice) {
                protected void execute() {
                    if (MockProtocolFactory.this.sendsAlive) {
                        super.execute();
                    }
                }
            };
        }

        public SendingSearch createSendingSearch(UpnpHeader searchTarget, int mxSeconds) {
            return new SendingSearch(getUpnpService(), searchTarget, mxSeconds) {
                public int getBulkIntervalMilliseconds() {
                    return 0;
                }
            };
        }
    }

    public MockUpnpService() {
        this(false, false, false);
    }

    public MockUpnpService(boolean sendsAlive, boolean maintainsRegistry) {
        this(sendsAlive, maintainsRegistry, false);
    }

    public MockUpnpService(boolean sendsAlive, final boolean maintainsRegistry, final boolean multiThreaded) {
        this.incomingDatagramMessages = new ArrayList();
        this.outgoingDatagramMessages = new ArrayList();
        this.receivedUpnpStreams = new ArrayList();
        this.sentStreamRequestMessages = new ArrayList();
        this.broadcastedBytes = new ArrayList();
        this.configuration = new DefaultUpnpServiceConfiguration(false) {

            class C03872 implements Executor {
                C03872() {
                }

                public void execute(Runnable runnable) {
                    new Thread(runnable).start();
                }
            }

            class C03883 implements Executor {
                C03883() {
                }

                public void execute(Runnable runnable) {
                    runnable.run();
                }
            }

            protected NetworkAddressFactory createNetworkAddressFactory(int streamListenPort) {
                return new NetworkAddressFactoryImpl(streamListenPort) {
                    protected boolean isUsableNetworkInterface(NetworkInterface iface) throws Exception {
                        return iface.isLoopback();
                    }

                    protected boolean isUsableAddress(NetworkInterface networkInterface, InetAddress address) {
                        return address.isLoopbackAddress() && (address instanceof Inet4Address);
                    }
                };
            }

            public Executor getRegistryMaintainerExecutor() {
                if (maintainsRegistry) {
                    return new C03872();
                }
                return createDefaultExecutor();
            }

            protected Executor createDefaultExecutor() {
                return multiThreaded ? super.createDefaultExecutor() : new C03883();
            }
        };
        this.protocolFactory = createProtocolFactory(this, sendsAlive);
        this.registry = new RegistryImpl(this) {
            protected RegistryMaintainer createRegistryMaintainer() {
                return maintainsRegistry ? super.createRegistryMaintainer() : null;
            }
        };
        this.networkAddressFactory = this.configuration.createNetworkAddressFactory();
        this.router = createRouter();
        this.controlPoint = new ControlPointImpl(this.configuration, this.protocolFactory, this.registry);
    }

    protected ProtocolFactory createProtocolFactory(UpnpService service, boolean sendsAlive) {
        return new MockProtocolFactory(service, sendsAlive);
    }

    protected Router createRouter() {
        return new MockRouter();
    }

    public UpnpServiceConfiguration getConfiguration() {
        return this.configuration;
    }

    public ControlPoint getControlPoint() {
        return this.controlPoint;
    }

    public ProtocolFactory getProtocolFactory() {
        return this.protocolFactory;
    }

    public Registry getRegistry() {
        return this.registry;
    }

    public Router getRouter() {
        return this.router;
    }

    public void shutdown() {
        getRouter().shutdown();
        getRegistry().shutdown();
        getConfiguration().shutdown();
    }

    public List<IncomingDatagramMessage> getIncomingDatagramMessages() {
        return this.incomingDatagramMessages;
    }

    public List<OutgoingDatagramMessage> getOutgoingDatagramMessages() {
        return this.outgoingDatagramMessages;
    }

    public List<UpnpStream> getReceivedUpnpStreams() {
        return this.receivedUpnpStreams;
    }

    public List<StreamRequestMessage> getSentStreamRequestMessages() {
        return this.sentStreamRequestMessages;
    }

    public List<byte[]> getBroadcastedBytes() {
        return this.broadcastedBytes;
    }

    public StreamResponseMessage[] getStreamResponseMessages() {
        return null;
    }

    public StreamResponseMessage getStreamResponseMessage(StreamRequestMessage request) {
        return null;
    }
}
