package org.teleal.cling.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Logger;
import org.teleal.cling.model.meta.LocalService;
import org.teleal.cling.model.meta.StateVariable;
import org.teleal.cling.model.state.StateVariableAccessor;
import org.teleal.cling.model.state.StateVariableValue;
import org.teleal.common.util.Exceptions;
import org.teleal.common.util.Reflections;

public class DefaultServiceManager<T> implements ServiceManager<T> {
    private static Logger log = Logger.getLogger(DefaultServiceManager.class.getName());
    protected final ReentrantLock lock;
    protected PropertyChangeSupport propertyChangeSupport;
    protected final LocalService<T> service;
    protected final Class<T> serviceClass;
    protected T serviceImpl;

    protected class DefaultPropertyChangeListener implements PropertyChangeListener {
        protected DefaultPropertyChangeListener() {
        }

        public void propertyChange(PropertyChangeEvent e) {
            DefaultServiceManager.log.finer("Property change event on local service: " + e.getPropertyName());
            if (!e.getPropertyName().equals(ServiceManager.EVENTED_STATE_VARIABLES)) {
                StateVariable sv = DefaultServiceManager.this.getService().getStateVariable(e.getPropertyName());
                if (sv != null && sv.getEventDetails().isSendEvents()) {
                    try {
                        DefaultServiceManager.log.fine("Evented state variable value changed, reading state of service: " + sv);
                        DefaultServiceManager.this.getPropertyChangeSupport().firePropertyChange(ServiceManager.EVENTED_STATE_VARIABLES, null, DefaultServiceManager.this.readEventedStateVariableValues());
                    } catch (Exception ex) {
                        DefaultServiceManager.log.severe("Error reading state of service after state variable update event: " + Exceptions.unwrap(ex));
                    }
                }
            }
        }
    }

    protected DefaultServiceManager(LocalService<T> service) {
        this(service, null);
    }

    public DefaultServiceManager(LocalService<T> service, Class<T> serviceClass) {
        this.lock = new ReentrantLock(true);
        this.service = service;
        this.serviceClass = serviceClass;
    }

    protected void lock() {
        try {
            if (this.lock.tryLock((long) getLockTimeoutMillis(), TimeUnit.MILLISECONDS)) {
                log.fine("Acquired lock");
                return;
            }
            throw new RuntimeException("Failed to acquire lock in milliseconds: " + getLockTimeoutMillis());
        } catch (InterruptedException e) {
            throw new RuntimeException("Failed to acquire lock:" + e);
        }
    }

    protected void unlock() {
        log.fine("Releasing lock");
        this.lock.unlock();
    }

    protected int getLockTimeoutMillis() {
        return 500;
    }

    public LocalService<T> getService() {
        return this.service;
    }

    public T getImplementation() {
        lock();
        try {
            if (this.serviceImpl == null) {
                init();
            }
            T t = this.serviceImpl;
            return t;
        } finally {
            unlock();
        }
    }

    public PropertyChangeSupport getPropertyChangeSupport() {
        lock();
        try {
            if (this.propertyChangeSupport == null) {
                init();
            }
            PropertyChangeSupport propertyChangeSupport = this.propertyChangeSupport;
            return propertyChangeSupport;
        } finally {
            unlock();
        }
    }

    public void execute(Command<T> cmd) throws Exception {
        lock();
        try {
            cmd.execute(this);
        } finally {
            unlock();
        }
    }

    public Collection<StateVariableValue> readEventedStateVariableValues() throws Exception {
        lock();
        try {
            Collection<StateVariableValue> values = new ArrayList();
            for (StateVariable stateVariable : getService().getStateVariables()) {
                if (stateVariable.getEventDetails().isSendEvents()) {
                    StateVariableAccessor accessor = getService().getAccessor(stateVariable);
                    if (accessor == null) {
                        throw new IllegalStateException("No accessor for evented state variable");
                    }
                    values.add(accessor.read(stateVariable, getImplementation()));
                }
            }
            return values;
        } finally {
            unlock();
        }
    }

    protected void init() {
        log.fine("No service implementation instance available, initializing...");
        try {
            this.serviceImpl = createServiceInstance();
            this.propertyChangeSupport = createPropertyChangeSupport(this.serviceImpl);
            this.propertyChangeSupport.addPropertyChangeListener(createPropertyChangeListener(this.serviceImpl));
        } catch (Exception ex) {
            throw new RuntimeException("Could not initialize implementation: " + ex, ex);
        }
    }

    protected T createServiceInstance() throws Exception {
        if (this.serviceClass == null) {
            throw new IllegalStateException("Subclass has to provide service class or override createServiceInstance()");
        }
        try {
            return this.serviceClass.getConstructor(new Class[]{LocalService.class}).newInstance(new Object[]{getService()});
        } catch (NoSuchMethodException e) {
            log.fine("Creating new service implementation instance with no-arg constructor: " + this.serviceClass.getName());
            return this.serviceClass.newInstance();
        }
    }

    protected PropertyChangeSupport createPropertyChangeSupport(T serviceImpl) throws Exception {
        Method m = Reflections.getGetterMethod(serviceImpl.getClass(), "propertyChangeSupport");
        if (m == null || !PropertyChangeSupport.class.isAssignableFrom(m.getReturnType())) {
            log.fine("Creating new PropertyChangeSupport for service implementation: " + serviceImpl.getClass().getName());
            return new PropertyChangeSupport(serviceImpl);
        }
        log.fine("Service implementation instance offers PropertyChangeSupport, using that: " + serviceImpl.getClass().getName());
        return (PropertyChangeSupport) m.invoke(serviceImpl, new Object[0]);
    }

    protected PropertyChangeListener createPropertyChangeListener(T t) throws Exception {
        return new DefaultPropertyChangeListener();
    }

    public String toString() {
        return "(" + getClass().getSimpleName() + ") Implementation: " + this.serviceImpl;
    }
}
