package org.teleal.cling.model.gena;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.model.ServiceManager;
import org.teleal.cling.model.meta.LocalService;
import org.teleal.cling.model.meta.StateVariable;
import org.teleal.cling.model.state.StateVariableValue;
import org.teleal.cling.model.types.UnsignedIntegerFourBytes;
import org.teleal.common.util.Exceptions;

public abstract class LocalGENASubscription extends GENASubscription<LocalService> implements PropertyChangeListener {
    private static Logger log = Logger.getLogger(LocalGENASubscription.class.getName());
    final List<URL> callbackURLs;
    final Map<String, Long> lastSentNumericValue = new HashMap();
    final Map<String, Long> lastSentTimestamp = new HashMap();

    public abstract void ended(CancelReason cancelReason);

    protected LocalGENASubscription(LocalService service, List<URL> callbackURLs) throws Exception {
        super(service);
        this.callbackURLs = callbackURLs;
    }

    public LocalGENASubscription(LocalService service, Integer requestedDurationSeconds, List<URL> callbackURLs) throws Exception {
        super(service);
        setSubscriptionDuration(requestedDurationSeconds);
        log.fine("Reading initial state of local service at subscription time");
        long currentTime = new Date().getTime();
        this.currentValues.clear();
        Collection<StateVariableValue> values = ((LocalService) getService()).getManager().readEventedStateVariableValues();
        log.finer("Got evented state variable values: " + values.size());
        for (StateVariableValue value : values) {
            this.currentValues.put(value.getStateVariable().getName(), value);
            if (log.isLoggable(Level.FINEST)) {
                log.finer("Read state variable value '" + value.getStateVariable().getName() + "': " + value.toString());
            }
            this.lastSentTimestamp.put(value.getStateVariable().getName(), Long.valueOf(currentTime));
            if (value.getStateVariable().isModeratedNumericType()) {
                this.lastSentNumericValue.put(value.getStateVariable().getName(), Long.valueOf(value.toString()));
            }
        }
        this.subscriptionId = "uuid:" + UUID.randomUUID();
        this.currentSequence = new UnsignedIntegerFourBytes(0);
        this.callbackURLs = callbackURLs;
    }

    public synchronized List<URL> getCallbackURLs() {
        return this.callbackURLs;
    }

    public synchronized void registerOnService() {
        ((LocalService) getService()).getManager().getPropertyChangeSupport().addPropertyChangeListener(this);
    }

    public synchronized void establish() {
        established();
    }

    public synchronized void end(CancelReason reason) {
        try {
            ((LocalService) getService()).getManager().getPropertyChangeSupport().removePropertyChangeListener(this);
        } catch (Exception ex) {
            log.warning("Removal of local service property change listener failed: " + Exceptions.unwrap(ex));
        }
        ended(reason);
    }

    public synchronized void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equals(ServiceManager.EVENTED_STATE_VARIABLES)) {
            log.fine("Eventing triggered, getting state for subscription: " + getSubscriptionId());
            long currentTime = new Date().getTime();
            Collection<StateVariableValue> newValues = (Collection) e.getNewValue();
            Set<String> excludedVariables = moderateStateVariables(currentTime, newValues);
            this.currentValues.clear();
            for (StateVariableValue newValue : newValues) {
                String name = newValue.getStateVariable().getName();
                if (!excludedVariables.contains(name)) {
                    log.fine("Adding state variable value to current values of event: " + newValue.getStateVariable() + " = " + newValue);
                    this.currentValues.put(newValue.getStateVariable().getName(), newValue);
                    this.lastSentTimestamp.put(name, Long.valueOf(currentTime));
                    if (newValue.getStateVariable().isModeratedNumericType()) {
                        this.lastSentNumericValue.put(name, Long.valueOf(newValue.toString()));
                    }
                }
            }
            if (this.currentValues.size() > 0) {
                log.fine("Propagating new state variable values to subscription: " + this);
                eventReceived();
            } else {
                log.fine("No state variable values for event (all moderated out?), not triggering event");
            }
        }
    }

    protected synchronized Set<String> moderateStateVariables(long currentTime, Collection<StateVariableValue> values) {
        Set<String> excludedVariables;
        excludedVariables = new HashSet();
        for (StateVariableValue stateVariableValue : values) {
            StateVariable stateVariable = stateVariableValue.getStateVariable();
            String stateVariableName = stateVariableValue.getStateVariable().getName();
            if (stateVariable.getEventDetails().getEventMaximumRateMilliseconds() == 0 && stateVariable.getEventDetails().getEventMinimumDelta() == 0) {
                log.finer("Variable is not moderated: " + stateVariable);
            } else if (!this.lastSentTimestamp.containsKey(stateVariableName)) {
                log.finer("Variable is moderated but was never sent before: " + stateVariable);
            } else if (stateVariable.getEventDetails().getEventMaximumRateMilliseconds() > 0 && currentTime <= ((Long) this.lastSentTimestamp.get(stateVariableName)).longValue() + ((long) stateVariable.getEventDetails().getEventMaximumRateMilliseconds())) {
                log.finer("Excluding state variable with maximum rate: " + stateVariable);
                excludedVariables.add(stateVariableName);
            } else if (stateVariable.isModeratedNumericType() && this.lastSentNumericValue.get(stateVariableName) != null) {
                long oldValue = Long.valueOf(((Long) this.lastSentNumericValue.get(stateVariableName)).longValue()).longValue();
                long newValue = Long.valueOf(stateVariableValue.toString()).longValue();
                long minDelta = (long) stateVariable.getEventDetails().getEventMinimumDelta();
                if (newValue > oldValue && newValue - oldValue < minDelta) {
                    log.finer("Excluding state variable with minimum delta: " + stateVariable);
                    excludedVariables.add(stateVariableName);
                } else if (newValue < oldValue && oldValue - newValue < minDelta) {
                    log.finer("Excluding state variable with minimum delta: " + stateVariable);
                    excludedVariables.add(stateVariableName);
                }
            }
        }
        return excludedVariables;
    }

    public synchronized void incrementSequence() {
        this.currentSequence.increment(true);
    }

    public synchronized void setSubscriptionDuration(Integer requestedDurationSeconds) {
        this.requestedDurationSeconds = requestedDurationSeconds == null ? 1800 : requestedDurationSeconds.intValue();
        setActualSubscriptionDurationSeconds(this.requestedDurationSeconds);
    }
}
