package org.teleal.cling.model.message;

import java.io.ByteArrayInputStream;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.model.message.header.UpnpHeader;
import org.teleal.cling.model.message.header.UpnpHeader.Type;
import org.teleal.common.http.Headers;

public class UpnpHeaders extends Headers {
    private static Logger log = Logger.getLogger(UpnpHeaders.class.getName());
    protected Map<Type, List<UpnpHeader>> parsedHeaders;

    public UpnpHeaders(Map<String, List<String>> headers) {
        super((Map) headers);
    }

    public UpnpHeaders(ByteArrayInputStream inputStream) {
        super(inputStream);
    }

    protected void parseHeaders() {
        this.parsedHeaders = new LinkedHashMap();
        log.fine("Parsing all HTTP headers for known UPnP headers: " + size());
        for (Entry<String, List<String>> entry : entrySet()) {
            if (entry.getKey() != null) {
                Type type = Type.getByHttpName((String) entry.getKey());
                if (type == null) {
                    log.fine("Ignoring non-UPNP HTTP header: " + ((String) entry.getKey()));
                } else {
                    for (String value : (List) entry.getValue()) {
                        UpnpHeader upnpHeader = UpnpHeader.newInstance(type, value);
                        if (upnpHeader == null || upnpHeader.getValue() == null) {
                            log.fine("Ignoring known but non-parsable header (value violates the UDA specification?) '" + type.getHttpName() + "': " + value);
                        } else {
                            addParsedValue(type, upnpHeader);
                        }
                    }
                }
            }
        }
    }

    protected void addParsedValue(Type type, UpnpHeader value) {
        log.fine("Adding parsed header: " + value);
        List<UpnpHeader> list = (List) this.parsedHeaders.get(type);
        if (list == null) {
            list = new LinkedList();
            this.parsedHeaders.put(type, list);
        }
        list.add(value);
    }

    public List<String> put(String key, List<String> values) {
        this.parsedHeaders = null;
        return super.put(key, (List) values);
    }

    public void add(String key, String value) {
        this.parsedHeaders = null;
        super.add(key, value);
    }

    public List<String> remove(Object key) {
        this.parsedHeaders = null;
        return super.remove(key);
    }

    public void clear() {
        this.parsedHeaders = null;
        super.clear();
    }

    public boolean containsKey(Type type) {
        if (this.parsedHeaders == null) {
            parseHeaders();
        }
        return this.parsedHeaders.containsKey(type);
    }

    public List<UpnpHeader> get(Type type) {
        if (this.parsedHeaders == null) {
            parseHeaders();
        }
        return (List) this.parsedHeaders.get(type);
    }

    public void add(Type type, UpnpHeader value) {
        super.add(type.getHttpName(), value.getString());
        if (this.parsedHeaders != null) {
            addParsedValue(type, value);
        }
    }

    public void remove(Type type) {
        super.remove(type.getHttpName());
        if (this.parsedHeaders != null) {
            this.parsedHeaders.remove(type);
        }
    }

    public UpnpHeader[] getAsArray(Type type) {
        if (this.parsedHeaders == null) {
            parseHeaders();
        }
        return this.parsedHeaders.get(type) != null ? (UpnpHeader[]) ((List) this.parsedHeaders.get(type)).toArray(new UpnpHeader[((List) this.parsedHeaders.get(type)).size()]) : new UpnpHeader[0];
    }

    public UpnpHeader getFirstHeader(Type type) {
        return getAsArray(type).length > 0 ? getAsArray(type)[0] : null;
    }

    public <H extends UpnpHeader> H getFirstHeader(Type type, Class<H> subtype) {
        UpnpHeader[] headers = getAsArray(type);
        if (headers.length == 0) {
            return null;
        }
        for (H header : headers) {
            if (subtype.isAssignableFrom(header.getClass())) {
                return header;
            }
        }
        return null;
    }

    public void log() {
        if (log.isLoggable(Level.FINE)) {
            log.fine("############################ RAW HEADERS ###########################");
            for (Entry<String, List<String>> entry : entrySet()) {
                log.fine("=== NAME : " + ((String) entry.getKey()));
                for (String v : (List) entry.getValue()) {
                    log.fine("VALUE: " + v);
                }
            }
            if (this.parsedHeaders != null && this.parsedHeaders.size() > 0) {
                log.fine("########################## PARSED HEADERS ##########################");
                for (Entry<Type, List<UpnpHeader>> entry2 : this.parsedHeaders.entrySet()) {
                    log.fine("=== TYPE: " + entry2.getKey());
                    for (UpnpHeader upnpHeader : (List) entry2.getValue()) {
                        log.fine("HEADER: " + upnpHeader);
                    }
                }
            }
            log.fine("####################################################################");
        }
    }
}
