package org.teleal.cling.model.meta;

import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.model.Namespace;
import org.teleal.cling.model.Validatable;
import org.teleal.cling.model.ValidationError;
import org.teleal.cling.model.ValidationException;
import org.teleal.cling.model.message.header.EXTHeader;
import org.teleal.cling.model.profile.ControlPointInfo;
import org.teleal.cling.model.resource.Resource;
import org.teleal.cling.model.types.DeviceType;
import org.teleal.cling.model.types.ServiceId;
import org.teleal.cling.model.types.ServiceType;
import org.teleal.cling.model.types.UDN;

public abstract class Device<DI extends DeviceIdentity, D extends Device, S extends Service> implements Validatable {
    private static final Logger log = Logger.getLogger(Device.class.getName());
    private final DeviceDetails details;
    protected final D[] embeddedDevices;
    private final Icon[] icons;
    private final DI identity;
    private D parentDevice;
    protected final S[] services;
    private final DeviceType type;
    private final UDAVersion version;

    public abstract Resource[] discoverResources(Namespace namespace);

    public abstract D findDevice(UDN udn);

    public abstract D[] getEmbeddedDevices();

    public abstract D getRoot();

    public abstract S[] getServices();

    public abstract D newInstance(UDN udn, UDAVersion uDAVersion, DeviceType deviceType, DeviceDetails deviceDetails, Icon[] iconArr, S[] sArr, List<D> list) throws ValidationException;

    public abstract S newInstance(ServiceType serviceType, ServiceId serviceId, URI uri, URI uri2, URI uri3, Action<S>[] actionArr, StateVariable<S>[] stateVariableArr) throws ValidationException;

    public abstract S[] newServiceArray(int i);

    public abstract D[] toDeviceArray(Collection<D> collection);

    public abstract S[] toServiceArray(Collection<S> collection);

    public Device(DI identity) throws ValidationException {
        this(identity, null, null, null, null, null);
    }

    public Device(DI identity, DeviceType type, DeviceDetails details, Icon[] icons, S[] services) throws ValidationException {
        this(identity, null, type, details, icons, services, null);
    }

    public Device(DI identity, DeviceType type, DeviceDetails details, Icon[] icons, S[] services, D[] embeddedDevices) throws ValidationException {
        this(identity, null, type, details, icons, services, embeddedDevices);
    }

    public Device(DI identity, UDAVersion version, DeviceType type, DeviceDetails details, Icon[] icons, S[] services, D[] embeddedDevices) throws ValidationException {
        this.identity = identity;
        if (version == null) {
            version = new UDAVersion();
        }
        this.version = version;
        this.type = type;
        this.details = details;
        boolean allNullIcons = true;
        if (icons != null) {
            for (Icon icon : icons) {
                if (icon != null) {
                    allNullIcons = false;
                    icon.setDevice(this);
                }
            }
        }
        if (icons == null || allNullIcons) {
            icons = new Icon[0];
        }
        this.icons = icons;
        boolean allNullServices = true;
        if (services != null) {
            for (S service : services) {
                if (service != null) {
                    allNullServices = false;
                    service.setDevice(this);
                }
            }
        }
        if (services == null || allNullServices) {
            services = null;
        }
        this.services = services;
        boolean allNullEmbedded = true;
        if (embeddedDevices != null) {
            for (D embeddedDevice : embeddedDevices) {
                if (embeddedDevice != null) {
                    allNullEmbedded = false;
                    embeddedDevice.setParentDevice(this);
                }
            }
        }
        if (embeddedDevices == null || allNullEmbedded) {
            embeddedDevices = null;
        }
        this.embeddedDevices = embeddedDevices;
        List<ValidationError> errors = validate();
        if (errors.size() > 0) {
            if (log.isLoggable(Level.FINEST)) {
                for (ValidationError error : errors) {
                    log.finest(error.toString());
                }
            }
            throw new ValidationException("Validation of device graph failed, call getErrors() on exception", (List) errors);
        }
    }

    public DI getIdentity() {
        return this.identity;
    }

    public UDAVersion getVersion() {
        return this.version;
    }

    public DeviceType getType() {
        return this.type;
    }

    public DeviceDetails getDetails() {
        return this.details;
    }

    public DeviceDetails getDetails(ControlPointInfo info) {
        return getDetails();
    }

    public Icon[] getIcons() {
        return this.icons;
    }

    public boolean hasIcons() {
        return getIcons() != null && getIcons().length > 0;
    }

    public boolean hasServices() {
        return getServices() != null && getServices().length > 0;
    }

    public boolean hasEmbeddedDevices() {
        return getEmbeddedDevices() != null && getEmbeddedDevices().length > 0;
    }

    public D getParentDevice() {
        return this.parentDevice;
    }

    void setParentDevice(D parentDevice) {
        if (this.parentDevice != null) {
            throw new IllegalStateException("Final value has been set already, model is immutable");
        }
        this.parentDevice = parentDevice;
    }

    public boolean isRoot() {
        return getParentDevice() == null;
    }

    public D[] findEmbeddedDevices() {
        return toDeviceArray(findEmbeddedDevices(this));
    }

    public D[] findDevices(DeviceType deviceType) {
        return toDeviceArray(find(deviceType, this));
    }

    public D[] findDevices(ServiceType serviceType) {
        return toDeviceArray(find(serviceType, this));
    }

    public Icon[] findIcons() {
        List<Icon> icons = new ArrayList();
        if (hasIcons()) {
            icons.addAll(Arrays.asList(getIcons()));
        }
        for (D embeddedDevice : findEmbeddedDevices()) {
            if (embeddedDevice.hasIcons()) {
                icons.addAll(Arrays.asList(embeddedDevice.getIcons()));
            }
        }
        return (Icon[]) icons.toArray(new Icon[icons.size()]);
    }

    public S[] findServices() {
        return toServiceArray(findServices(null, null, this));
    }

    public S[] findServices(ServiceType serviceType) {
        return toServiceArray(findServices(serviceType, null, this));
    }

    protected D find(UDN udn, D current) {
        if (current.getIdentity().getUdn().equals(udn)) {
            return current;
        }
        if (current.hasEmbeddedDevices()) {
            for (Device embeddedDevice : current.getEmbeddedDevices()) {
                D match = find(udn, embeddedDevice);
                if (match != null) {
                    return match;
                }
            }
        }
        return null;
    }

    protected Collection<D> findEmbeddedDevices(D current) {
        Collection<D> devices = new HashSet();
        if (!current.isRoot()) {
            devices.add(current);
        }
        if (current.hasEmbeddedDevices()) {
            for (D embeddedDevice : current.getEmbeddedDevices()) {
                devices.addAll(findEmbeddedDevices(embeddedDevice));
            }
        }
        return devices;
    }

    protected Collection<D> find(DeviceType deviceType, D current) {
        Collection<D> devices = new HashSet();
        if (current.getType() != null && current.getType().implementsVersion(deviceType)) {
            devices.add(current);
        }
        if (current.hasEmbeddedDevices()) {
            for (Device embeddedDevice : current.getEmbeddedDevices()) {
                devices.addAll(find(deviceType, embeddedDevice));
            }
        }
        return devices;
    }

    protected Collection<D> find(ServiceType serviceType, D current) {
        Collection<S> services = findServices(serviceType, null, current);
        Collection<D> devices = new HashSet();
        for (S service : services) {
            devices.add(service.getDevice());
        }
        return devices;
    }

    protected Collection<S> findServices(ServiceType serviceType, ServiceId serviceId, D current) {
        Collection services = new HashSet();
        if (current.hasServices()) {
            for (Service service : current.getServices()) {
                if (isMatch(service, serviceType, serviceId)) {
                    services.add(service);
                }
            }
        }
        Collection<D> embeddedDevices = findEmbeddedDevices(current);
        if (embeddedDevices != null) {
            for (D embeddedDevice : embeddedDevices) {
                if (embeddedDevice.hasServices()) {
                    for (Service service2 : embeddedDevice.getServices()) {
                        if (isMatch(service2, serviceType, serviceId)) {
                            services.add(service2);
                        }
                    }
                }
            }
        }
        return services;
    }

    public S findService(ServiceId serviceId) {
        Collection<S> services = findServices(null, serviceId, this);
        if (services.size() == 1) {
            return (Service) services.iterator().next();
        }
        return null;
    }

    public S findService(ServiceType serviceType) {
        Collection<S> services = findServices(serviceType, null, this);
        if (services.size() > 0) {
            return (Service) services.iterator().next();
        }
        return null;
    }

    public ServiceType[] findServiceTypes() {
        Collection<S> services = findServices(null, null, this);
        Collection<ServiceType> col = new HashSet();
        for (S service : services) {
            col.add(service.getServiceType());
        }
        return (ServiceType[]) col.toArray(new ServiceType[col.size()]);
    }

    private boolean isMatch(Service s, ServiceType serviceType, ServiceId serviceId) {
        boolean matchesType;
        if (serviceType == null || s.getServiceType().implementsVersion(serviceType)) {
            matchesType = true;
        } else {
            matchesType = false;
        }
        boolean matchesId;
        if (serviceId == null || s.getServiceId().equals(serviceId)) {
            matchesId = true;
        } else {
            matchesId = false;
        }
        if (matchesType && matchesId) {
            return true;
        }
        return false;
    }

    public boolean isFullyHydrated() {
        for (S service : findServices()) {
            if (service.hasStateVariables()) {
                return true;
            }
        }
        return false;
    }

    public String getDisplayString() {
        String cleanModelName = null;
        String cleanModelNumber = null;
        if (!(getDetails() == null || getDetails().getModelDetails() == null)) {
            ModelDetails modelDetails = getDetails().getModelDetails();
            if (modelDetails.getModelName() != null) {
                cleanModelName = (modelDetails.getModelNumber() == null || !modelDetails.getModelName().endsWith(modelDetails.getModelNumber())) ? modelDetails.getModelName() : modelDetails.getModelName().substring(0, modelDetails.getModelName().length() - modelDetails.getModelNumber().length());
            }
            cleanModelNumber = cleanModelName != null ? (modelDetails.getModelNumber() == null || cleanModelName.startsWith(modelDetails.getModelNumber())) ? EXTHeader.DEFAULT_VALUE : modelDetails.getModelNumber() : modelDetails.getModelNumber();
        }
        StringBuilder sb = new StringBuilder();
        if (!(getDetails() == null || getDetails().getManufacturerDetails() == null)) {
            if (!(cleanModelName == null || getDetails().getManufacturerDetails().getManufacturer() == null)) {
                cleanModelName = cleanModelName.startsWith(getDetails().getManufacturerDetails().getManufacturer()) ? cleanModelName.substring(getDetails().getManufacturerDetails().getManufacturer().length()).trim() : cleanModelName.trim();
            }
            if (getDetails().getManufacturerDetails().getManufacturer() != null) {
                sb.append(getDetails().getManufacturerDetails().getManufacturer());
            }
        }
        String str = (cleanModelName == null || cleanModelName.length() <= 0) ? EXTHeader.DEFAULT_VALUE : " " + cleanModelName;
        sb.append(str);
        str = (cleanModelNumber == null || cleanModelNumber.length() <= 0) ? EXTHeader.DEFAULT_VALUE : " " + cleanModelNumber.trim();
        sb.append(str);
        return sb.toString();
    }

    public List<ValidationError> validate() {
        List<ValidationError> errors = new ArrayList();
        if (getType() != null) {
            errors.addAll(getVersion().validate());
            if (getDetails() != null) {
                errors.addAll(getDetails().validate());
            }
            if (hasIcons()) {
                for (Icon icon : getIcons()) {
                    if (icon != null) {
                        errors.addAll(icon.validate());
                    }
                }
            }
            if (hasServices()) {
                for (Service service : getServices()) {
                    if (service != null) {
                        errors.addAll(service.validate());
                    }
                }
            }
            if (hasEmbeddedDevices()) {
                for (Device embeddedDevice : getEmbeddedDevices()) {
                    if (embeddedDevice != null) {
                        errors.addAll(embeddedDevice.validate());
                    }
                }
            }
        }
        return errors;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        if (this.identity.equals(((Device) o).identity)) {
            return true;
        }
        return false;
    }

    public int hashCode() {
        return this.identity.hashCode();
    }

    public String toString() {
        return "(" + getClass().getSimpleName() + ") Identity: " + getIdentity().toString() + ", Root: " + isRoot();
    }
}
