package org.teleal.cling.protocol;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.logging.Logger;
import org.teleal.cling.UpnpService;
import org.teleal.cling.binding.xml.DescriptorBindingException;
import org.teleal.cling.model.ValidationError;
import org.teleal.cling.model.ValidationException;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.UpnpRequest.Method;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.meta.Icon;
import org.teleal.cling.model.meta.RemoteDevice;
import org.teleal.cling.model.meta.RemoteDeviceIdentity;
import org.teleal.cling.model.meta.RemoteService;
import org.teleal.cling.model.meta.Service;
import org.teleal.cling.model.types.ServiceType;
import org.teleal.cling.registry.RegistrationException;
import org.teleal.common.util.Exceptions;

public class RetrieveRemoteDescriptors implements Runnable {
    private static final Set<URL> activeRetrievals = new CopyOnWriteArraySet();
    private static final Logger log = Logger.getLogger(RetrieveRemoteDescriptors.class.getName());
    private RemoteDevice rd;
    private final UpnpService upnpService;

    public RetrieveRemoteDescriptors(UpnpService upnpService, RemoteDevice rd) {
        this.upnpService = upnpService;
        this.rd = rd;
    }

    public UpnpService getUpnpService() {
        return this.upnpService;
    }

    public void run() {
        URL deviceURL = ((RemoteDeviceIdentity) this.rd.getIdentity()).getDescriptorURL();
        if (activeRetrievals.contains(deviceURL)) {
            log.finer("Exiting early, active retrieval for URL already in progress: " + deviceURL);
        } else if (getUpnpService().getRegistry().getRemoteDevice(((RemoteDeviceIdentity) this.rd.getIdentity()).getUdn(), true) != null) {
            log.finer("Exiting early, already discovered: " + deviceURL);
        } else {
            try {
                activeRetrievals.add(deviceURL);
                describe();
            } finally {
                activeRetrievals.remove(deviceURL);
            }
        }
    }

    protected void describe() {
        StreamRequestMessage deviceDescRetrievalMsg = new StreamRequestMessage(Method.GET, ((RemoteDeviceIdentity) this.rd.getIdentity()).getDescriptorURL());
        log.fine("Sending device descriptor retrieval message: " + deviceDescRetrievalMsg);
        StreamResponseMessage deviceDescMsg = getUpnpService().getRouter().send(deviceDescRetrievalMsg);
        if (deviceDescMsg == null) {
            log.warning("Device descriptor retrieval failed, no response: " + ((RemoteDeviceIdentity) this.rd.getIdentity()).getDescriptorURL());
        } else if (((UpnpResponse) deviceDescMsg.getOperation()).isFailed()) {
            log.warning("Device descriptor retrieval failed: " + ((RemoteDeviceIdentity) this.rd.getIdentity()).getDescriptorURL() + ", " + ((UpnpResponse) deviceDescMsg.getOperation()).getResponseDetails());
        } else {
            if (!deviceDescMsg.isContentTypeTextUDA()) {
                log.warning("Received device descriptor without or with invalid Content-Type: " + ((RemoteDeviceIdentity) this.rd.getIdentity()).getDescriptorURL());
            }
            log.fine("Received root device descriptor: " + deviceDescMsg);
            describe(deviceDescMsg.getBodyString());
        }
    }

    protected void describe(String descriptorXML) {
        try {
            RemoteDevice describedDevice = (RemoteDevice) getUpnpService().getConfiguration().getDeviceDescriptorBinderUDA10().describe(this.rd, descriptorXML);
            log.fine("Remote device described (without services) notifying listeners: " + describedDevice);
            boolean notifiedStart = getUpnpService().getRegistry().notifyDiscoveryStart(describedDevice);
            log.fine("Hydrating described device's services: " + describedDevice);
            RemoteDevice hydratedDevice = describeServices(describedDevice);
            if (hydratedDevice == null) {
                log.warning("Device service description failed: " + this.rd);
                if (notifiedStart) {
                    getUpnpService().getRegistry().notifyDiscoveryFailure(describedDevice, new DescriptorBindingException("Device service description failed: " + this.rd));
                    return;
                }
                return;
            }
            log.fine("Adding fully hydrated remote device to registry: " + hydratedDevice);
            getUpnpService().getRegistry().addDevice(hydratedDevice);
        } catch (ValidationException ex) {
            log.warning("Could not validate device model: " + this.rd);
            for (ValidationError validationError : ex.getErrors()) {
                log.warning(validationError.toString());
            }
            if (null != null && null != null) {
                getUpnpService().getRegistry().notifyDiscoveryFailure(null, ex);
            }
        } catch (DescriptorBindingException ex2) {
            log.warning("Could not hydrate device or its services from descriptor: " + this.rd);
            log.warning("Cause was: " + Exceptions.unwrap(ex2));
            if (null != null && null != null) {
                getUpnpService().getRegistry().notifyDiscoveryFailure(null, ex2);
            }
        } catch (RegistrationException ex3) {
            log.warning("Adding hydrated device to registry failed: " + this.rd);
            log.warning("Cause was: " + ex3.toString());
            if (null != null && null != null) {
                getUpnpService().getRegistry().notifyDiscoveryFailure(null, ex3);
            }
        }
    }

    protected RemoteDevice describeServices(RemoteDevice currentDevice) throws DescriptorBindingException, ValidationException {
        Collection describedServices = new ArrayList();
        if (currentDevice.hasServices()) {
            for (RemoteService service : filterExclusiveServices(currentDevice.getServices())) {
                RemoteService svc = describeService(service);
                if (svc == null) {
                    return null;
                }
                describedServices.add(svc);
            }
        }
        List describedEmbeddedDevices = new ArrayList();
        if (currentDevice.hasEmbeddedDevices()) {
            for (RemoteDevice embeddedDevice : currentDevice.getEmbeddedDevices()) {
                if (embeddedDevice != null) {
                    RemoteDevice describedEmbeddedDevice = describeServices(embeddedDevice);
                    if (describedEmbeddedDevice == null) {
                        return null;
                    }
                    describedEmbeddedDevices.add(describedEmbeddedDevice);
                }
            }
        }
        Icon[] iconDupes = new Icon[currentDevice.getIcons().length];
        for (int i = 0; i < currentDevice.getIcons().length; i++) {
            iconDupes[i] = currentDevice.getIcons()[i].deepCopy();
        }
        return currentDevice.newInstance(((RemoteDeviceIdentity) currentDevice.getIdentity()).getUdn(), currentDevice.getVersion(), currentDevice.getType(), currentDevice.getDetails(), iconDupes, currentDevice.toServiceArray(describedServices), describedEmbeddedDevices);
    }

    protected RemoteService describeService(RemoteService service) throws DescriptorBindingException, ValidationException {
        URL descriptorURL = ((RemoteDevice) service.getDevice()).normalizeURI(service.getDescriptorURI());
        StreamRequestMessage serviceDescRetrievalMsg = new StreamRequestMessage(Method.GET, descriptorURL);
        log.fine("Sending service descriptor retrieval message: " + serviceDescRetrievalMsg);
        StreamResponseMessage serviceDescMsg = getUpnpService().getRouter().send(serviceDescRetrievalMsg);
        if (serviceDescMsg == null) {
            log.warning("Could not retrieve service descriptor: " + service);
            return null;
        } else if (((UpnpResponse) serviceDescMsg.getOperation()).isFailed()) {
            log.warning("Service descriptor retrieval failed: " + descriptorURL + ", " + ((UpnpResponse) serviceDescMsg.getOperation()).getResponseDetails());
            return null;
        } else {
            if (!serviceDescMsg.isContentTypeTextUDA()) {
                log.warning("Received service descriptor without or with invalid Content-Type: " + descriptorURL);
            }
            String descriptorContent = serviceDescMsg.getBodyString();
            if (descriptorContent == null || descriptorContent.length() == 0) {
                log.warning("Received empty descriptor:" + descriptorURL);
                return null;
            }
            log.fine("Received service descriptor, hydrating service model: " + serviceDescMsg);
            return (RemoteService) getUpnpService().getConfiguration().getServiceDescriptorBinderUDA10().describe((Service) service, serviceDescMsg.getBodyString());
        }
    }

    protected List<RemoteService> filterExclusiveServices(RemoteService[] services) {
        ServiceType[] exclusiveTypes = getUpnpService().getConfiguration().getExclusiveServiceTypes();
        if (exclusiveTypes == null || exclusiveTypes.length == 0) {
            return Arrays.asList(services);
        }
        List<RemoteService> exclusiveServices = new ArrayList();
        for (RemoteService discoveredService : services) {
            for (ServiceType exclusiveType : exclusiveTypes) {
                if (discoveredService.getServiceType().implementsVersion(exclusiveType)) {
                    log.fine("Including exlusive service: " + discoveredService);
                    exclusiveServices.add(discoveredService);
                } else {
                    log.fine("Excluding unwanted service: " + exclusiveType);
                }
            }
        }
        return exclusiveServices;
    }
}
