package org.teleal.cling.protocol.sync;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.UpnpService;
import org.teleal.cling.model.action.ActionException;
import org.teleal.cling.model.action.ActionInvocation;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.UpnpHeaders;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.message.UpnpResponse.Status;
import org.teleal.cling.model.message.control.ActionRequestMessage;
import org.teleal.cling.model.message.control.ActionResponseMessage;
import org.teleal.cling.model.message.control.IncomingActionRequestMessage;
import org.teleal.cling.model.message.control.OutgoingActionResponseMessage;
import org.teleal.cling.model.message.header.ContentTypeHeader;
import org.teleal.cling.model.message.header.UpnpHeader.Type;
import org.teleal.cling.model.meta.LocalService;
import org.teleal.cling.model.resource.ServiceControlResource;
import org.teleal.cling.model.types.ErrorCode;
import org.teleal.cling.protocol.ReceivingSync;
import org.teleal.cling.transport.spi.UnsupportedDataException;
import org.teleal.common.util.Exceptions;

public class ReceivingAction extends ReceivingSync<StreamRequestMessage, StreamResponseMessage> {
    protected static final ThreadLocal<UpnpHeaders> extraResponseHeadersThreadLocal = new ThreadLocal();
    private static final Logger log = Logger.getLogger(ReceivingAction.class.getName());
    protected static final ThreadLocal<IncomingActionRequestMessage> requestThreadLocal = new ThreadLocal();

    public ReceivingAction(UpnpService upnpService, StreamRequestMessage inputMessage) {
        super(upnpService, inputMessage);
    }

    protected StreamResponseMessage executeSync() {
        StreamResponseMessage responseMessage;
        ContentTypeHeader contentTypeHeader = (ContentTypeHeader) ((StreamRequestMessage) getInputMessage()).getHeaders().getFirstHeader(Type.CONTENT_TYPE, ContentTypeHeader.class);
        if (contentTypeHeader == null || contentTypeHeader.isUDACompliantXML()) {
            if (contentTypeHeader == null) {
                log.warning("Received without Content-Type: " + getInputMessage());
            }
            ServiceControlResource resource = (ServiceControlResource) getUpnpService().getRegistry().getResource(ServiceControlResource.class, ((StreamRequestMessage) getInputMessage()).getUri());
            if (resource == null) {
                log.fine("No local resource found: " + getInputMessage());
                return null;
            }
            ActionInvocation invocation;
            ActionResponseMessage responseMessage2;
            log.fine("Found local action resource matching relative request URI: " + ((StreamRequestMessage) getInputMessage()).getUri());
            OutgoingActionResponseMessage responseMessage3 = null;
            Object responseMessage4;
            try {
                ActionRequestMessage requestMessage = new IncomingActionRequestMessage((StreamRequestMessage) getInputMessage(), (LocalService) resource.getModel());
                requestThreadLocal.set(requestMessage);
                extraResponseHeadersThreadLocal.set(new UpnpHeaders());
                log.finer("Created incoming action request message: " + requestMessage);
                invocation = new ActionInvocation(requestMessage.getAction());
                log.fine("Reading body of request message");
                getUpnpService().getConfiguration().getSoapActionProcessor().readBody(requestMessage, invocation);
                log.fine("Executing on local service: " + invocation);
                ((LocalService) resource.getModel()).getExecutor(invocation.getAction()).execute(invocation);
                if (invocation.getFailure() == null) {
                    responseMessage2 = new OutgoingActionResponseMessage(invocation.getAction());
                } else {
                    responseMessage4 = new OutgoingActionResponseMessage(Status.INTERNAL_SERVER_ERROR, invocation.getAction());
                }
                if (!(responseMessage2 == null || extraResponseHeadersThreadLocal.get() == null)) {
                    log.fine("Merging extra headers into action response message: " + ((UpnpHeaders) extraResponseHeadersThreadLocal.get()).size());
                    responseMessage2.getHeaders().putAll((Map) extraResponseHeadersThreadLocal.get());
                }
                requestThreadLocal.set(null);
                extraResponseHeadersThreadLocal.set(null);
            } catch (ActionException ex) {
                log.finer("Error executing local action: " + ex);
                invocation = new ActionInvocation(ex);
                responseMessage = new OutgoingActionResponseMessage(Status.INTERNAL_SERVER_ERROR);
                if (!(responseMessage == null || extraResponseHeadersThreadLocal.get() == null)) {
                    log.fine("Merging extra headers into action response message: " + ((UpnpHeaders) extraResponseHeadersThreadLocal.get()).size());
                    responseMessage.getHeaders().putAll((Map) extraResponseHeadersThreadLocal.get());
                }
                requestThreadLocal.set(null);
                extraResponseHeadersThreadLocal.set(null);
                responseMessage4 = responseMessage;
            } catch (UnsupportedDataException ex2) {
                if (log.isLoggable(Level.FINER)) {
                    log.log(Level.FINER, "Error reading action request XML body: " + ex2.toString(), Exceptions.unwrap(ex2));
                }
                invocation = new ActionInvocation(Exceptions.unwrap(ex2) instanceof ActionException ? (ActionException) Exceptions.unwrap(ex2) : new ActionException(ErrorCode.ACTION_FAILED, ex2.getMessage()));
                responseMessage = new OutgoingActionResponseMessage(Status.INTERNAL_SERVER_ERROR);
                if (!(responseMessage == null || extraResponseHeadersThreadLocal.get() == null)) {
                    log.fine("Merging extra headers into action response message: " + ((UpnpHeaders) extraResponseHeadersThreadLocal.get()).size());
                    responseMessage.getHeaders().putAll((Map) extraResponseHeadersThreadLocal.get());
                }
                requestThreadLocal.set(null);
                extraResponseHeadersThreadLocal.set(null);
                responseMessage4 = responseMessage;
            } catch (Throwable th) {
                Throwable th2 = th;
                if (!(responseMessage3 == null || extraResponseHeadersThreadLocal.get() == null)) {
                    log.fine("Merging extra headers into action response message: " + ((UpnpHeaders) extraResponseHeadersThreadLocal.get()).size());
                    responseMessage3.getHeaders().putAll((Map) extraResponseHeadersThreadLocal.get());
                }
                requestThreadLocal.set(null);
                extraResponseHeadersThreadLocal.set(null);
            }
            try {
                log.fine("Writing body of response message");
                getUpnpService().getConfiguration().getSoapActionProcessor().writeBody(responseMessage2, invocation);
                log.fine("Returning finished response message: " + responseMessage2);
                return responseMessage2;
            } catch (UnsupportedDataException ex22) {
                log.warning("Failure writing body of response message, sending '500 Internal Server Error' without body");
                log.log(Level.WARNING, "Exception root cause: ", Exceptions.unwrap(ex22));
                return new StreamResponseMessage(Status.INTERNAL_SERVER_ERROR);
            }
        }
        log.warning("Received invalid Content-Type '" + contentTypeHeader + "': " + getInputMessage());
        return new StreamResponseMessage(new UpnpResponse(Status.UNSUPPORTED_MEDIA_TYPE));
    }

    public static IncomingActionRequestMessage getRequestMessage() {
        return (IncomingActionRequestMessage) requestThreadLocal.get();
    }

    public static UpnpHeaders getExtraResponseHeaders() {
        return (UpnpHeaders) extraResponseHeadersThreadLocal.get();
    }
}
