package org.teleal.cling.protocol.sync;

import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.UpnpService;
import org.teleal.cling.model.action.ActionException;
import org.teleal.cling.model.action.ActionInvocation;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.message.UpnpResponse.Status;
import org.teleal.cling.model.message.control.ActionRequestMessage;
import org.teleal.cling.model.message.control.ActionResponseMessage;
import org.teleal.cling.model.message.control.IncomingActionResponseMessage;
import org.teleal.cling.model.message.control.OutgoingActionRequestMessage;
import org.teleal.cling.model.types.ErrorCode;
import org.teleal.cling.protocol.SendingSync;
import org.teleal.cling.transport.spi.UnsupportedDataException;
import org.teleal.common.util.Exceptions;

public class SendingAction extends SendingSync<OutgoingActionRequestMessage, IncomingActionResponseMessage> {
    private static final Logger log = Logger.getLogger(SendingAction.class.getName());
    protected final ActionInvocation actionInvocation;

    public SendingAction(UpnpService upnpService, ActionInvocation actionInvocation, URL controlURL) {
        super(upnpService, new OutgoingActionRequestMessage(actionInvocation, controlURL));
        this.actionInvocation = actionInvocation;
    }

    protected IncomingActionResponseMessage executeSync() {
        return invokeRemote((OutgoingActionRequestMessage) getInputMessage());
    }

    protected IncomingActionResponseMessage invokeRemote(OutgoingActionRequestMessage requestMessage) {
        ActionException ex;
        log.fine("Sending outgoing action call '" + this.actionInvocation.getAction().getName() + "' to remote service of: " + this.actionInvocation.getAction().getService().getDevice());
        IncomingActionResponseMessage responseMessage = null;
        try {
            StreamResponseMessage streamResponse = sendRemoteRequest(requestMessage);
            if (streamResponse == null) {
                log.fine("No connection or no no response received, returning null");
                this.actionInvocation.setFailure(new ActionException(ErrorCode.ACTION_FAILED, "Connection error or no response received"));
                return null;
            }
            IncomingActionResponseMessage responseMessage2 = new IncomingActionResponseMessage(streamResponse);
            try {
                if (responseMessage2.isFailedNonRecoverable()) {
                    log.fine("Response was a non-recoverable failure: " + responseMessage2);
                    throw new ActionException(ErrorCode.ACTION_FAILED, "Non-recoverable remote execution failure: " + ((UpnpResponse) responseMessage2.getOperation()).getResponseDetails());
                }
                if (responseMessage2.isFailedRecoverable()) {
                    handleResponseFailure(responseMessage2);
                } else {
                    handleResponse(responseMessage2);
                }
                responseMessage = responseMessage2;
                return responseMessage2;
            } catch (ActionException e) {
                ex = e;
                responseMessage = responseMessage2;
                log.fine("Remote action invocation failed, returning Internal Server Error message: " + ex.getMessage());
                this.actionInvocation.setFailure(ex);
                if (responseMessage == null) {
                }
                return new IncomingActionResponseMessage(new UpnpResponse(Status.INTERNAL_SERVER_ERROR));
            }
        } catch (ActionException e2) {
            ex = e2;
            log.fine("Remote action invocation failed, returning Internal Server Error message: " + ex.getMessage());
            this.actionInvocation.setFailure(ex);
            if (responseMessage == null && ((UpnpResponse) responseMessage.getOperation()).isFailed()) {
                return responseMessage;
            }
            return new IncomingActionResponseMessage(new UpnpResponse(Status.INTERNAL_SERVER_ERROR));
        }
    }

    protected StreamResponseMessage sendRemoteRequest(OutgoingActionRequestMessage requestMessage) throws ActionException {
        try {
            log.fine("Writing SOAP request body of: " + requestMessage);
            getUpnpService().getConfiguration().getSoapActionProcessor().writeBody((ActionRequestMessage) requestMessage, this.actionInvocation);
            log.fine("Sending SOAP body of message as stream to remote device");
            return getUpnpService().getRouter().send((StreamRequestMessage) requestMessage);
        } catch (UnsupportedDataException ex) {
            log.fine("Error writing SOAP body: " + ex);
            log.log(Level.FINE, "Exception root cause: ", Exceptions.unwrap(ex));
            throw new ActionException(ErrorCode.ACTION_FAILED, "Error writing request message. " + ex.getMessage());
        }
    }

    protected void handleResponse(IncomingActionResponseMessage responseMsg) throws ActionException {
        try {
            log.fine("Received response for outgoing call, reading SOAP response body: " + responseMsg);
            getUpnpService().getConfiguration().getSoapActionProcessor().readBody((ActionResponseMessage) responseMsg, this.actionInvocation);
        } catch (UnsupportedDataException ex) {
            log.fine("Error reading SOAP body: " + ex);
            log.log(Level.FINE, "Exception root cause: ", Exceptions.unwrap(ex));
            throw new ActionException(ErrorCode.ACTION_FAILED, "Error reading response message. " + ex.getMessage());
        }
    }

    protected void handleResponseFailure(IncomingActionResponseMessage responseMsg) throws ActionException {
        try {
            log.fine("Received response with Internal Server Error, reading SOAP failure message");
            getUpnpService().getConfiguration().getSoapActionProcessor().readBody((ActionResponseMessage) responseMsg, this.actionInvocation);
        } catch (UnsupportedDataException ex) {
            log.fine("Error reading SOAP body: " + ex);
            log.log(Level.FINE, "Exception root cause: ", Exceptions.unwrap(ex));
            throw new ActionException(ErrorCode.ACTION_FAILED, "Error reading response failure message. " + ex.getMessage());
        }
    }
}
