package org.teleal.cling.registry;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;
import java.util.logging.Logger;
import org.teleal.cling.model.gena.CancelReason;
import org.teleal.cling.model.gena.GENASubscription;
import org.teleal.cling.model.gena.LocalGENASubscription;
import org.teleal.cling.model.meta.LocalDevice;
import org.teleal.cling.model.meta.LocalService;
import org.teleal.cling.model.resource.Resource;
import org.teleal.cling.model.types.UDN;
import org.teleal.cling.protocol.SendingAsync;

class LocalItems extends RegistryItems<LocalDevice, LocalGENASubscription> {
    private static Logger log = Logger.getLogger(Registry.class.getName());
    protected Random randomGenerator = new Random();

    LocalItems(RegistryImpl registry) {
        super(registry);
    }

    void add(LocalDevice localDevice) throws RegistrationException {
        if (this.registry.getDevice(localDevice.getIdentity().getUdn(), false) != null) {
            log.fine("Ignoring addition, device already registered: " + localDevice);
            return;
        }
        log.fine("Adding local device to registry: " + localDevice);
        for (Resource deviceResource : getResources(localDevice)) {
            if (this.registry.getResource(deviceResource.getPathQuery()) != null) {
                throw new RegistrationException("URI namespace conflict with already registered resource: " + deviceResource);
            }
            this.registry.addResource(deviceResource);
            log.fine("Registered resource: " + deviceResource);
        }
        log.fine("Adding item to registry with expiration in seconds: " + localDevice.getIdentity().getMaxAgeSeconds());
        RegistryItem<UDN, LocalDevice> localItem = new RegistryItem(localDevice.getIdentity().getUdn(), localDevice, localDevice.getIdentity().getMaxAgeSeconds().intValue());
        this.deviceItems.add(localItem);
        log.fine("Registered local device: " + localItem);
        advertiseAlive(localDevice);
        for (RegistryListener listener : this.registry.getListeners()) {
            listener.localDeviceAdded(this.registry, localDevice);
        }
    }

    Collection<LocalDevice> get() {
        Set<LocalDevice> c = new HashSet();
        for (RegistryItem<UDN, LocalDevice> item : this.deviceItems) {
            c.add(item.getItem());
        }
        return Collections.unmodifiableCollection(c);
    }

    boolean remove(LocalDevice localDevice) throws RegistrationException {
        return remove(localDevice, false);
    }

    boolean remove(final LocalDevice localDevice, boolean shuttingDown) throws RegistrationException {
        LocalDevice registeredDevice = (LocalDevice) get(localDevice.getIdentity().getUdn(), true);
        if (registeredDevice == null) {
            return false;
        }
        log.fine("Removing local device from registry: " + localDevice);
        this.deviceItems.remove(new RegistryItem(localDevice.getIdentity().getUdn()));
        for (Resource deviceResource : getResources(localDevice)) {
            if (this.registry.removeResource(deviceResource)) {
                log.fine("Unregistered resource: " + deviceResource);
            }
        }
        Iterator<RegistryItem<String, LocalGENASubscription>> it = this.subscriptionItems.iterator();
        while (it.hasNext()) {
            final RegistryItem<String, LocalGENASubscription> incomingSubscription = (RegistryItem) it.next();
            if (((LocalService) ((LocalGENASubscription) incomingSubscription.getItem()).getService()).getDevice().getIdentity().getUdn().equals(registeredDevice.getIdentity().getUdn())) {
                log.fine("Removing incoming subscription: " + ((String) incomingSubscription.getKey()));
                it.remove();
                if (!shuttingDown) {
                    this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                        public void run() {
                            ((LocalGENASubscription) incomingSubscription.getItem()).end(CancelReason.DEVICE_WAS_REMOVED);
                        }
                    });
                }
            }
        }
        advertiseByebye(localDevice, !shuttingDown);
        if (!shuttingDown) {
            for (final RegistryListener listener : this.registry.getListeners()) {
                this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                    public void run() {
                        listener.localDeviceRemoved(LocalItems.this.registry, localDevice);
                    }
                });
            }
        }
        return true;
    }

    void removeAll() {
        removeAll(false);
    }

    void removeAll(boolean shuttingDown) {
        for (LocalDevice device : (LocalDevice[]) get().toArray(new LocalDevice[get().size()])) {
            remove(device, shuttingDown);
        }
    }

    void maintain() {
        if (!this.deviceItems.isEmpty()) {
            Set<RegistryItem<UDN, LocalDevice>> expiredLocalItems = new HashSet();
            for (RegistryItem<UDN, LocalDevice> localItem : this.deviceItems) {
                if (localItem.getExpirationDetails().hasExpired(true)) {
                    log.finer("Local item has expired: " + localItem);
                    expiredLocalItems.add(localItem);
                }
            }
            for (RegistryItem<UDN, LocalDevice> expiredLocalItem : expiredLocalItems) {
                log.fine("Refreshing local device advertisement: " + expiredLocalItem.getItem());
                advertiseAlive((LocalDevice) expiredLocalItem.getItem());
                expiredLocalItem.getExpirationDetails().stampLastRefresh();
            }
            Set<RegistryItem<String, LocalGENASubscription>> expiredIncomingSubscriptions = new HashSet();
            for (RegistryItem<String, LocalGENASubscription> item : this.subscriptionItems) {
                if (item.getExpirationDetails().hasExpired(false)) {
                    expiredIncomingSubscriptions.add(item);
                }
            }
            for (RegistryItem<String, LocalGENASubscription> subscription : expiredIncomingSubscriptions) {
                log.fine("Removing expired: " + subscription);
                removeSubscription((GENASubscription) subscription.getItem());
                ((LocalGENASubscription) subscription.getItem()).end(CancelReason.EXPIRED);
            }
        }
    }

    void shutdown() {
        log.fine("Clearing all registered subscriptions to local devices during shutdown");
        this.subscriptionItems.clear();
        log.fine("Removing all local devices from registry during shutdown");
        removeAll(true);
    }

    protected void advertiseAlive(final LocalDevice localDevice) {
        this.registry.executeAsyncProtocol(new Runnable() {
            public void run() {
                try {
                    LocalItems.log.finer("Sleeping some milliseconds to avoid flooding the network with ALIVE msgs");
                    Thread.sleep((long) LocalItems.this.randomGenerator.nextInt(100));
                } catch (InterruptedException ex) {
                    LocalItems.log.severe("Background execution interrupted: " + ex.getMessage());
                }
                LocalItems.this.registry.getProtocolFactory().createSendingNotificationAlive(localDevice).run();
            }
        });
    }

    protected void advertiseByebye(LocalDevice localDevice, boolean asynchronous) {
        SendingAsync prot = this.registry.getProtocolFactory().createSendingNotificationByebye(localDevice);
        if (asynchronous) {
            this.registry.executeAsyncProtocol(prot);
        } else {
            prot.run();
        }
    }
}
