package org.teleal.cling.registry;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.model.gena.CancelReason;
import org.teleal.cling.model.gena.RemoteGENASubscription;
import org.teleal.cling.model.meta.LocalDevice;
import org.teleal.cling.model.meta.RemoteDevice;
import org.teleal.cling.model.meta.RemoteDeviceIdentity;
import org.teleal.cling.model.meta.RemoteService;
import org.teleal.cling.model.resource.Resource;
import org.teleal.cling.model.types.UDN;

class RemoteItems extends RegistryItems<RemoteDevice, RemoteGENASubscription> {
    private static Logger log = Logger.getLogger(Registry.class.getName());

    RemoteItems(RegistryImpl registry) {
        super(registry);
    }

    void add(final RemoteDevice device) {
        if (update((RemoteDeviceIdentity) device.getIdentity())) {
            log.fine("Ignoring addition, device already registered: " + device);
            return;
        }
        Resource[] resources = getResources(device);
        for (Resource deviceResource : resources) {
            log.fine("Validating remote device resource; " + deviceResource);
            if (this.registry.getResource(deviceResource.getPathQuery()) != null) {
                throw new RegistrationException("URI namespace conflict with already registered resource: " + deviceResource);
            }
        }
        for (Resource validatedResource : resources) {
            this.registry.addResource(validatedResource);
            log.fine("Added remote device resource: " + validatedResource);
        }
        RegistryItem item = new RegistryItem(((RemoteDeviceIdentity) device.getIdentity()).getUdn(), device, ((RemoteDeviceIdentity) device.getIdentity()).getMaxAgeSeconds().intValue());
        log.fine("Adding hydrated remote device to registry with " + item.getExpirationDetails().getMaxAgeSeconds() + " seconds expiration: " + device);
        this.deviceItems.add(item);
        if (log.isLoggable(Level.FINEST)) {
            StringBuilder sb = new StringBuilder();
            sb.append("\n");
            sb.append("-------------------------- START Registry Namespace -----------------------------------\n");
            for (Resource resource : this.registry.getResources()) {
                sb.append(resource).append("\n");
            }
            sb.append("-------------------------- END Registry Namespace -----------------------------------");
            log.finest(sb.toString());
        }
        log.fine("Completely hydrated remote device graph available, calling listeners: " + device);
        for (final RegistryListener listener : this.registry.getListeners()) {
            this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                public void run() {
                    listener.remoteDeviceAdded(RemoteItems.this.registry, device);
                }
            });
        }
    }

    boolean update(RemoteDeviceIdentity rdIdentity) {
        for (LocalDevice localDevice : this.registry.getLocalDevices()) {
            if (localDevice.findDevice(rdIdentity.getUdn()) != null) {
                log.fine("Ignoring update, a local device graph contains UDN");
                return true;
            }
        }
        RemoteDevice registeredRemoteDevice = (RemoteDevice) get(rdIdentity.getUdn(), false);
        if (registeredRemoteDevice == null) {
            return false;
        }
        if (!registeredRemoteDevice.isRoot()) {
            log.fine("Updating root device of embedded: " + registeredRemoteDevice);
            registeredRemoteDevice = registeredRemoteDevice.getRoot();
        }
        final RegistryItem<UDN, RemoteDevice> item = new RegistryItem(((RemoteDeviceIdentity) registeredRemoteDevice.getIdentity()).getUdn(), registeredRemoteDevice, rdIdentity.getMaxAgeSeconds().intValue());
        log.fine("Updating expiration of: " + registeredRemoteDevice);
        this.deviceItems.remove(item);
        this.deviceItems.add(item);
        log.fine("Remote device updated, calling listeners: " + registeredRemoteDevice);
        for (final RegistryListener listener : this.registry.getListeners()) {
            this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                public void run() {
                    listener.remoteDeviceUpdated(RemoteItems.this.registry, (RemoteDevice) item.getItem());
                }
            });
        }
        return true;
    }

    boolean remove(RemoteDevice remoteDevice) {
        return remove(remoteDevice, false);
    }

    boolean remove(RemoteDevice remoteDevice, boolean shuttingDown) throws RegistrationException {
        final RemoteDevice registeredDevice = (RemoteDevice) get(((RemoteDeviceIdentity) remoteDevice.getIdentity()).getUdn(), true);
        if (registeredDevice == null) {
            return false;
        }
        log.fine("Removing remote device from registry: " + remoteDevice);
        for (Resource deviceResource : getResources(registeredDevice)) {
            if (this.registry.removeResource(deviceResource)) {
                log.fine("Unregistered resource: " + deviceResource);
            }
        }
        Iterator<RegistryItem<String, RemoteGENASubscription>> it = this.subscriptionItems.iterator();
        while (it.hasNext()) {
            final RegistryItem<String, RemoteGENASubscription> outgoingSubscription = (RegistryItem) it.next();
            if (((RemoteDeviceIdentity) ((RemoteDevice) ((RemoteService) ((RemoteGENASubscription) outgoingSubscription.getItem()).getService()).getDevice()).getIdentity()).getUdn().equals(((RemoteDeviceIdentity) registeredDevice.getIdentity()).getUdn())) {
                log.fine("Removing outgoing subscription: " + ((String) outgoingSubscription.getKey()));
                it.remove();
                if (!shuttingDown) {
                    this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                        public void run() {
                            ((RemoteGENASubscription) outgoingSubscription.getItem()).end(CancelReason.DEVICE_WAS_REMOVED, null);
                        }
                    });
                }
            }
        }
        if (!shuttingDown) {
            for (final RegistryListener listener : this.registry.getListeners()) {
                this.registry.getConfiguration().getRegistryListenerExecutor().execute(new Runnable() {
                    public void run() {
                        listener.remoteDeviceRemoved(RemoteItems.this.registry, registeredDevice);
                    }
                });
            }
        }
        this.deviceItems.remove(new RegistryItem(((RemoteDeviceIdentity) registeredDevice.getIdentity()).getUdn()));
        return true;
    }

    void removeAll() {
        removeAll(false);
    }

    void removeAll(boolean shuttingDown) {
        for (RemoteDevice device : (RemoteDevice[]) get().toArray(new RemoteDevice[get().size()])) {
            remove(device, shuttingDown);
        }
    }

    void start() {
    }

    void maintain() {
        if (!this.deviceItems.isEmpty()) {
            Map<UDN, RemoteDevice> expiredRemoteDevices = new HashMap();
            for (RegistryItem<UDN, RemoteDevice> remoteItem : this.deviceItems) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Device '" + remoteItem.getItem() + "' expires in seconds: " + remoteItem.getExpirationDetails().getSecondsUntilExpiration());
                }
                if (remoteItem.getExpirationDetails().hasExpired(false)) {
                    expiredRemoteDevices.put(remoteItem.getKey(), remoteItem.getItem());
                }
            }
            for (RemoteDevice remoteDevice : expiredRemoteDevices.values()) {
                if (log.isLoggable(Level.FINE)) {
                    log.fine("Removing expired: " + remoteDevice);
                }
                remove(remoteDevice);
            }
            Set<RemoteGENASubscription> expiredOutgoingSubscriptions = new HashSet();
            for (RegistryItem<String, RemoteGENASubscription> item : this.subscriptionItems) {
                if (item.getExpirationDetails().hasExpired(true)) {
                    expiredOutgoingSubscriptions.add(item.getItem());
                }
            }
            for (RemoteGENASubscription subscription : expiredOutgoingSubscriptions) {
                if (log.isLoggable(Level.FINEST)) {
                    log.fine("Renewing outgoing subscription: " + subscription);
                }
                renewOutgoingSubscription(subscription);
            }
        }
    }

    public void resume() {
        log.fine("Updating remote device expiration timestamps on resume");
        List<RemoteDeviceIdentity> toUpdate = new ArrayList();
        for (RegistryItem<UDN, RemoteDevice> remoteItem : this.deviceItems) {
            toUpdate.add(((RemoteDevice) remoteItem.getItem()).getIdentity());
        }
        for (RemoteDeviceIdentity identity : toUpdate) {
            update(identity);
        }
    }

    void shutdown() {
        log.fine("Cancelling all outgoing subscriptions to remote devices during shutdown");
        List<RemoteGENASubscription> remoteSubscriptions = new ArrayList();
        for (RegistryItem<String, RemoteGENASubscription> item : this.subscriptionItems) {
            remoteSubscriptions.add(item.getItem());
        }
        for (RemoteGENASubscription remoteSubscription : remoteSubscriptions) {
            this.registry.getProtocolFactory().createSendingUnsubscribe(remoteSubscription).run();
        }
        log.fine("Removing all remote devices from registry during shutdown");
        removeAll(true);
    }

    protected void renewOutgoingSubscription(RemoteGENASubscription subscription) {
        this.registry.executeAsyncProtocol(this.registry.getProtocolFactory().createSendingRenewal(subscription));
    }
}
