package org.teleal.cling.support.connectionmanager;

import java.beans.PropertyChangeSupport;
import java.util.logging.Logger;
import org.teleal.cling.binding.annotations.UpnpAction;
import org.teleal.cling.binding.annotations.UpnpInputArgument;
import org.teleal.cling.binding.annotations.UpnpOutputArgument;
import org.teleal.cling.controlpoint.ControlPoint;
import org.teleal.cling.model.ServiceReference;
import org.teleal.cling.model.action.ActionException;
import org.teleal.cling.model.action.ActionInvocation;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.meta.Service;
import org.teleal.cling.model.types.ErrorCode;
import org.teleal.cling.model.types.UnsignedIntegerFourBytes;
import org.teleal.cling.model.types.csv.CSV;
import org.teleal.cling.support.connectionmanager.callback.ConnectionComplete;
import org.teleal.cling.support.connectionmanager.callback.PrepareForConnection;
import org.teleal.cling.support.model.ConnectionInfo;
import org.teleal.cling.support.model.ConnectionInfo.Direction;
import org.teleal.cling.support.model.ConnectionInfo.Status;
import org.teleal.cling.support.model.ProtocolInfo;
import org.teleal.cling.support.model.ProtocolInfos;

public abstract class AbstractPeeringConnectionManagerService extends ConnectionManagerService {
    private static final Logger log = Logger.getLogger(AbstractPeeringConnectionManagerService.class.getName());

    protected abstract void closeConnection(ConnectionInfo connectionInfo);

    protected abstract ConnectionInfo createConnection(int i, int i2, ServiceReference serviceReference, Direction direction, ProtocolInfo protocolInfo) throws ActionException;

    protected abstract void peerFailure(ActionInvocation actionInvocation, UpnpResponse upnpResponse, String str);

    protected AbstractPeeringConnectionManagerService(ConnectionInfo... activeConnections) {
        super(activeConnections);
    }

    protected AbstractPeeringConnectionManagerService(ProtocolInfos sourceProtocolInfo, ProtocolInfos sinkProtocolInfo, ConnectionInfo... activeConnections) {
        super(sourceProtocolInfo, sinkProtocolInfo, activeConnections);
    }

    protected AbstractPeeringConnectionManagerService(PropertyChangeSupport propertyChangeSupport, ProtocolInfos sourceProtocolInfo, ProtocolInfos sinkProtocolInfo, ConnectionInfo... activeConnections) {
        super(propertyChangeSupport, sourceProtocolInfo, sinkProtocolInfo, activeConnections);
    }

    protected synchronized int getNewConnectionId() {
        int currentHighestID;
        currentHighestID = -1;
        for (Integer key : this.activeConnections.keySet()) {
            if (key.intValue() > currentHighestID) {
                currentHighestID = key.intValue();
            }
        }
        return currentHighestID + 1;
    }

    protected synchronized void storeConnection(ConnectionInfo info) {
        CSV<UnsignedIntegerFourBytes> oldConnectionIDs = getCurrentConnectionIDs();
        this.activeConnections.put(Integer.valueOf(info.getConnectionID()), info);
        log.fine("Connection stored, firing event: " + info.getConnectionID());
        getPropertyChangeSupport().firePropertyChange("CurrentConnectionIDs", oldConnectionIDs, getCurrentConnectionIDs());
    }

    protected synchronized void removeConnection(int connectionID) {
        CSV<UnsignedIntegerFourBytes> oldConnectionIDs = getCurrentConnectionIDs();
        this.activeConnections.remove(Integer.valueOf(connectionID));
        log.fine("Connection removed, firing event: " + connectionID);
        getPropertyChangeSupport().firePropertyChange("CurrentConnectionIDs", oldConnectionIDs, getCurrentConnectionIDs());
    }

    @UpnpAction(out = {@UpnpOutputArgument(getterName = "getConnectionID", name = "ConnectionID", stateVariable = "A_ARG_TYPE_ConnectionID"), @UpnpOutputArgument(getterName = "getAvTransportID", name = "AVTransportID", stateVariable = "A_ARG_TYPE_AVTransportID"), @UpnpOutputArgument(getterName = "getRcsID", name = "RcsID", stateVariable = "A_ARG_TYPE_RcsID")})
    public synchronized ConnectionInfo prepareForConnection(@UpnpInputArgument(name = "RemoteProtocolInfo", stateVariable = "A_ARG_TYPE_ProtocolInfo") ProtocolInfo remoteProtocolInfo, @UpnpInputArgument(name = "PeerConnectionManager", stateVariable = "A_ARG_TYPE_ConnectionManager") ServiceReference peerConnectionManager, @UpnpInputArgument(name = "PeerConnectionID", stateVariable = "A_ARG_TYPE_ConnectionID") int peerConnectionId, @UpnpInputArgument(name = "Direction", stateVariable = "A_ARG_TYPE_Direction") String direction) throws ActionException {
        ConnectionInfo newConnectionInfo;
        int connectionId = getNewConnectionId();
        try {
            Direction dir = Direction.valueOf(direction);
            log.fine("Preparing for connection with local new ID " + connectionId + " and peer connection ID: " + peerConnectionId);
            newConnectionInfo = createConnection(connectionId, peerConnectionId, peerConnectionManager, dir, remoteProtocolInfo);
            storeConnection(newConnectionInfo);
        } catch (Exception e) {
            throw new ConnectionManagerException(ErrorCode.ARGUMENT_VALUE_INVALID, "Unsupported direction: " + direction);
        }
        return newConnectionInfo;
    }

    @UpnpAction
    public synchronized void connectionComplete(@UpnpInputArgument(name = "ConnectionID", stateVariable = "A_ARG_TYPE_ConnectionID") int connectionID) throws ActionException {
        ConnectionInfo info = getCurrentConnectionInfo(connectionID);
        log.fine("Closing connection ID " + connectionID);
        closeConnection(info);
        removeConnection(connectionID);
    }

    public synchronized int createConnectionWithPeer(ServiceReference localServiceReference, ControlPoint controlPoint, Service peerService, ProtocolInfo protInfo, Direction direction) {
        int localConnectionID;
        localConnectionID = getNewConnectionId();
        log.fine("Creating new connection ID " + localConnectionID + " with peer: " + peerService);
        final boolean[] failed = new boolean[1];
        final int i = localConnectionID;
        final ProtocolInfo protocolInfo = protInfo;
        final Service service = peerService;
        final Direction direction2 = direction;
        new PrepareForConnection(peerService, controlPoint, protInfo, localServiceReference, localConnectionID, direction) {
            public void received(ActionInvocation invocation, int peerConnectionID, int rcsID, int avTransportID) {
                AbstractPeeringConnectionManagerService.this.storeConnection(new ConnectionInfo(i, rcsID, avTransportID, protocolInfo, service.getReference(), peerConnectionID, direction2.getOpposite(), Status.OK));
            }

            public void failure(ActionInvocation invocation, UpnpResponse operation, String defaultMsg) {
                AbstractPeeringConnectionManagerService.this.peerFailure(invocation, operation, defaultMsg);
                failed[0] = true;
            }
        }.run();
        if (failed[0]) {
            localConnectionID = -1;
        }
        return localConnectionID;
    }

    public synchronized void closeConnectionWithPeer(ControlPoint controlPoint, Service peerService, int connectionID) throws ActionException {
        closeConnectionWithPeer(controlPoint, peerService, getCurrentConnectionInfo(connectionID));
    }

    public synchronized void closeConnectionWithPeer(ControlPoint controlPoint, Service peerService, ConnectionInfo connectionInfo) throws ActionException {
        log.fine("Closing connection ID " + connectionInfo.getConnectionID() + " with peer: " + peerService);
        final ConnectionInfo connectionInfo2 = connectionInfo;
        new ConnectionComplete(peerService, controlPoint, connectionInfo.getPeerConnectionID()) {
            public void success(ActionInvocation invocation) {
                AbstractPeeringConnectionManagerService.this.removeConnection(connectionInfo2.getConnectionID());
            }

            public void failure(ActionInvocation invocation, UpnpResponse operation, String defaultMsg) {
                AbstractPeeringConnectionManagerService.this.peerFailure(invocation, operation, defaultMsg);
            }
        }.run();
    }
}
