package org.teleal.cling.support.lastchange;

import java.io.InputStream;
import java.io.StringReader;
import java.util.Collections;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilderFactory;
import org.teleal.cling.model.XMLUtil;
import org.teleal.cling.model.types.UnsignedIntegerFourBytes;
import org.teleal.cling.support.shared.AbstractMap.SimpleEntry;
import org.teleal.common.io.IO;
import org.teleal.common.util.Exceptions;
import org.teleal.common.xml.SAXParser;
import org.teleal.common.xml.SAXParser.Handler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public abstract class LastChangeParser extends SAXParser {
    private static final Logger log = Logger.getLogger(LastChangeParser.class.getName());

    public enum CONSTANTS {
        Event,
        InstanceID,
        val;

        public boolean equals(String s) {
            return name().equals(s);
        }
    }

    class InstanceIDHandler extends Handler<InstanceID> {
        InstanceIDHandler(InstanceID instance, Handler parent) {
            super((Object) instance, parent);
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            Entry[] attributeMap = new Entry[attributes.getLength()];
            for (int i = 0; i < attributeMap.length; i++) {
                attributeMap[i] = new SimpleEntry(attributes.getLocalName(i), attributes.getValue(i));
            }
            try {
                EventedValue esv = LastChangeParser.this.createValue(localName, attributeMap);
                if (esv != null) {
                    ((InstanceID) getInstance()).getValues().add(esv);
                }
            } catch (Exception ex) {
                LastChangeParser.log.warning("Error reading event XML, ignoring value: " + Exceptions.unwrap(ex));
            }
        }

        protected boolean isLastElement(String uri, String localName, String qName) {
            return CONSTANTS.InstanceID.equals(localName);
        }
    }

    class RootHandler extends Handler<Event> {
        RootHandler(Event instance, SAXParser parser) {
            super((Object) instance, parser);
        }

        RootHandler(Event instance) {
            super(instance);
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            if (CONSTANTS.InstanceID.equals(localName)) {
                String valAttr = attributes.getValue(CONSTANTS.val.name());
                if (valAttr != null) {
                    InstanceID instanceID = new InstanceID(new UnsignedIntegerFourBytes(valAttr));
                    ((Event) getInstance()).getInstanceIDs().add(instanceID);
                    InstanceIDHandler instanceIDHandler = new InstanceIDHandler(instanceID, this);
                }
            }
        }
    }

    protected abstract String getNamespace();

    protected Set<Class<? extends EventedValue>> getEventedVariables() {
        return Collections.EMPTY_SET;
    }

    protected EventedValue createValue(String name, Entry<String, String>[] attributes) throws Exception {
        for (Class<? extends EventedValue> evType : getEventedVariables()) {
            if (evType.getSimpleName().equals(name)) {
                return (EventedValue) evType.getConstructor(new Class[]{Entry[].class}).newInstance(new Object[]{attributes});
            }
        }
        return null;
    }

    public Event parseResource(String resource) throws Exception {
        InputStream is = null;
        try {
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(resource);
            Event parse = parse(IO.readLines(is));
            return parse;
        } finally {
            if (is != null) {
                is.close();
            }
        }
    }

    public Event parse(String xml) throws Exception {
        if (xml == null || xml.length() == 0) {
            throw new RuntimeException("Null or empty XML");
        }
        Event event = new Event();
        RootHandler rootHandler = new RootHandler(event, this);
        log.fine("Parsing 'LastChange' event XML content");
        parse(new InputSource(new StringReader(xml)));
        log.fine("Parsed event with instances IDs: " + event.getInstanceIDs().size());
        if (log.isLoggable(Level.FINEST)) {
            for (InstanceID instanceID : event.getInstanceIDs()) {
                log.finest("InstanceID '" + instanceID.getId() + "' has values: " + instanceID.getValues().size());
                for (EventedValue eventedValue : instanceID.getValues()) {
                    log.finest(eventedValue.getName() + " => " + eventedValue.getValue());
                }
            }
        }
        return event;
    }

    public String generate(Event event) throws Exception {
        return XMLUtil.documentToFragmentString(buildDOM(event));
    }

    protected Document buildDOM(Event event) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        Document d = factory.newDocumentBuilder().newDocument();
        generateRoot(event, d);
        return d;
    }

    protected void generateRoot(Event event, Document descriptor) {
        Element eventElement = descriptor.createElementNS(getNamespace(), CONSTANTS.Event.name());
        descriptor.appendChild(eventElement);
        generateInstanceIDs(event, descriptor, eventElement);
    }

    protected void generateInstanceIDs(Event event, Document descriptor, Element rootElement) {
        for (InstanceID instanceID : event.getInstanceIDs()) {
            if (instanceID.getId() != null) {
                Element instanceIDElement = XMLUtil.appendNewElement(descriptor, rootElement, CONSTANTS.InstanceID.name());
                instanceIDElement.setAttribute(CONSTANTS.val.name(), instanceID.getId().toString());
                for (EventedValue eventedValue : instanceID.getValues()) {
                    generateEventedValue(eventedValue, descriptor, instanceIDElement);
                }
            }
        }
    }

    protected void generateEventedValue(EventedValue eventedValue, Document descriptor, Element parentElement) {
        String name = eventedValue.getName();
        Entry[] attributes = eventedValue.getAttributes();
        if (attributes != null && attributes.length > 0) {
            Element evElement = XMLUtil.appendNewElement(descriptor, parentElement, name);
            for (Entry<String, String> attr : attributes) {
                evElement.setAttribute((String) attr.getKey(), (String) attr.getValue());
            }
        }
    }
}
