package org.teleal.cling.transport;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.UpnpServiceConfiguration;
import org.teleal.cling.model.NetworkAddress;
import org.teleal.cling.model.message.IncomingDatagramMessage;
import org.teleal.cling.model.message.OutgoingDatagramMessage;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.protocol.ProtocolCreationException;
import org.teleal.cling.protocol.ProtocolFactory;
import org.teleal.cling.protocol.ReceivingAsync;
import org.teleal.cling.transport.spi.DatagramIO;
import org.teleal.cling.transport.spi.InitializationException;
import org.teleal.cling.transport.spi.MulticastReceiver;
import org.teleal.cling.transport.spi.NetworkAddressFactory;
import org.teleal.cling.transport.spi.StreamClient;
import org.teleal.cling.transport.spi.StreamServer;
import org.teleal.cling.transport.spi.UpnpStream;
import org.teleal.common.util.Exceptions;

public class RouterImpl implements Router {
    private static Logger log = Logger.getLogger(Router.class.getName());
    protected final UpnpServiceConfiguration configuration;
    protected final Map<InetAddress, DatagramIO> datagramIOs = new HashMap();
    private boolean isShutdown;
    protected final Map<NetworkInterface, MulticastReceiver> multicastReceivers = new HashMap();
    protected final NetworkAddressFactory networkAddressFactory;
    protected final ProtocolFactory protocolFactory;
    protected final StreamClient streamClient;
    protected final Map<InetAddress, StreamServer> streamServers = new HashMap();

    public RouterImpl(UpnpServiceConfiguration configuration, ProtocolFactory protocolFactory) throws InitializationException {
        log.info("Creating Router: " + getClass().getName());
        this.configuration = configuration;
        this.protocolFactory = protocolFactory;
        this.isShutdown = false;
        log.info("jangpw: Starting networking services...");
        this.networkAddressFactory = getConfiguration().createNetworkAddressFactory();
        log.info("jangpw: Starting createNetworkAddressFactory...");
        this.streamClient = getConfiguration().createStreamClient();
        for (NetworkInterface networkInterface : this.networkAddressFactory.getNetworkInterfaces()) {
            MulticastReceiver multicastReceiver = getConfiguration().createMulticastReceiver(this.networkAddressFactory);
            if (multicastReceiver != null) {
                this.multicastReceivers.put(networkInterface, multicastReceiver);
            }
        }
        for (InetAddress inetAddress : this.networkAddressFactory.getBindAddresses()) {
            log.info("jangpw: network address= " + inetAddress.toString());
            DatagramIO datagramIO = getConfiguration().createDatagramIO(this.networkAddressFactory);
            if (datagramIO != null) {
                this.datagramIOs.put(inetAddress, datagramIO);
            }
            StreamServer streamServer = getConfiguration().createStreamServer(this.networkAddressFactory);
            if (streamServer != null) {
                this.streamServers.put(inetAddress, streamServer);
            }
        }
        for (Entry<InetAddress, StreamServer> entry : this.streamServers.entrySet()) {
            log.info("jangpw: Starting stream server on address: " + entry.getKey());
            ((StreamServer) entry.getValue()).init((InetAddress) entry.getKey(), this);
            getConfiguration().getStreamServerExecutor().execute((Runnable) entry.getValue());
            log.info("jangpw: Starting stream server on address: done");
        }
        for (Entry<NetworkInterface, MulticastReceiver> entry2 : this.multicastReceivers.entrySet()) {
            log.info("jangpw: Starting multicast receiver on interface: " + ((NetworkInterface) entry2.getKey()).getDisplayName());
            ((MulticastReceiver) entry2.getValue()).init((NetworkInterface) entry2.getKey(), this, getConfiguration().getDatagramProcessor());
            getConfiguration().getMulticastReceiverExecutor().execute((Runnable) entry2.getValue());
            log.info("jangpw: Starting multicast receiver on interface: done");
        }
        for (Entry<InetAddress, DatagramIO> entry3 : this.datagramIOs.entrySet()) {
            log.info("jangpw: Starting datagram I/O on address: " + entry3.getKey());
            ((DatagramIO) entry3.getValue()).init((InetAddress) entry3.getKey(), this, getConfiguration().getDatagramProcessor());
            getConfiguration().getDatagramIOExecutor().execute((Runnable) entry3.getValue());
            log.info("jangpw: Starting datagram I/O on address: done");
        }
        log.info("jangpw: network address setting done");
    }

    public UpnpServiceConfiguration getConfiguration() {
        return this.configuration;
    }

    public ProtocolFactory getProtocolFactory() {
        return this.protocolFactory;
    }

    public NetworkAddressFactory getNetworkAddressFactory() {
        return this.networkAddressFactory;
    }

    protected Map<NetworkInterface, MulticastReceiver> getMulticastReceivers() {
        return this.multicastReceivers;
    }

    protected Map<InetAddress, DatagramIO> getDatagramIOs() {
        return this.datagramIOs;
    }

    protected StreamClient getStreamClient() {
        return this.streamClient;
    }

    protected Map<InetAddress, StreamServer> getStreamServers() {
        return this.streamServers;
    }

    public synchronized List<NetworkAddress> getActiveStreamServers(InetAddress preferredAddress) {
        List<NetworkAddress> list;
        if (getStreamServers().size() == 0) {
            list = Collections.EMPTY_LIST;
        } else {
            list = new ArrayList();
            if (preferredAddress != null) {
                StreamServer preferredServer = (StreamServer) getStreamServers().get(preferredAddress);
                if (preferredServer != null) {
                    list.add(new NetworkAddress(preferredAddress, preferredServer.getPort(), getNetworkAddressFactory().getHardwareAddress(preferredAddress)));
                }
            }
            for (Entry<InetAddress, StreamServer> entry : getStreamServers().entrySet()) {
                list.add(new NetworkAddress((InetAddress) entry.getKey(), ((StreamServer) entry.getValue()).getPort(), getNetworkAddressFactory().getHardwareAddress((InetAddress) entry.getKey())));
            }
        }
        return list;
    }

    private void internalShutdown() {
        log.info("Shutting down network services");
        if (this.streamClient != null) {
            log.fine("Stopping stream client connection management/pool");
            this.streamClient.stop();
        }
        for (Entry<InetAddress, StreamServer> entry : this.streamServers.entrySet()) {
            log.fine("Stopping stream server on address: " + entry.getKey());
            ((StreamServer) entry.getValue()).stop();
        }
        this.streamServers.clear();
        for (Entry<NetworkInterface, MulticastReceiver> entry2 : this.multicastReceivers.entrySet()) {
            log.fine("Stopping multicast receiver on interface: " + ((NetworkInterface) entry2.getKey()).getDisplayName());
            ((MulticastReceiver) entry2.getValue()).stop();
        }
        this.multicastReceivers.clear();
        for (Entry<InetAddress, DatagramIO> entry3 : this.datagramIOs.entrySet()) {
            log.fine("Stopping datagram I/O on address: " + entry3.getKey());
            ((DatagramIO) entry3.getValue()).stop();
        }
        this.datagramIOs.clear();
    }

    public synchronized void shutdown() {
        this.isShutdown = true;
        internalShutdown();
    }

    public void received(IncomingDatagramMessage msg) {
        if (!this.isShutdown) {
            try {
                ReceivingAsync protocol = getProtocolFactory().createReceivingAsync(msg);
                if (protocol != null) {
                    if (log.isLoggable(Level.FINE)) {
                        log.fine("Received asynchronous message: " + msg);
                    }
                    getConfiguration().getAsyncProtocolExecutor().execute(protocol);
                } else if (log.isLoggable(Level.FINEST)) {
                    log.finest("No protocol, ignoring received message: " + msg);
                }
            } catch (ProtocolCreationException ex) {
                log.warning("Handling received datagram failed - " + Exceptions.unwrap(ex).toString());
            }
        }
    }

    public void received(UpnpStream stream) {
        if (!this.isShutdown) {
            log.fine("Received synchronous stream: " + stream);
            getConfiguration().getSyncProtocolExecutor().execute(stream);
        }
    }

    public void send(OutgoingDatagramMessage msg) {
        if (!this.isShutdown) {
            for (DatagramIO datagramIO : getDatagramIOs().values()) {
                datagramIO.send(msg);
            }
        }
    }

    public StreamResponseMessage send(StreamRequestMessage msg) {
        if (this.isShutdown) {
            return null;
        }
        if (getStreamClient() == null) {
            log.fine("No StreamClient available, ignoring: " + msg);
            return null;
        }
        log.fine("Sending via TCP unicast stream: " + msg);
        return getStreamClient().sendRequest(msg);
    }

    public void broadcast(byte[] bytes) {
        if (!this.isShutdown) {
            for (Entry<InetAddress, DatagramIO> entry : getDatagramIOs().entrySet()) {
                InetAddress broadcast = getNetworkAddressFactory().getBroadcastAddress((InetAddress) entry.getKey());
                if (broadcast != null) {
                    log.fine("Sending UDP datagram to broadcast address: " + broadcast.getHostAddress());
                    ((DatagramIO) entry.getValue()).send(new DatagramPacket(bytes, bytes.length, broadcast, 9));
                }
            }
        }
    }
}
