package org.teleal.cling.transport;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.logging.Logger;
import org.teleal.cling.UpnpServiceConfiguration;
import org.teleal.cling.model.NetworkAddress;
import org.teleal.cling.model.message.IncomingDatagramMessage;
import org.teleal.cling.model.message.OutgoingDatagramMessage;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.protocol.ProtocolFactory;
import org.teleal.cling.transport.spi.InitializationException;
import org.teleal.cling.transport.spi.NetworkAddressFactory;
import org.teleal.cling.transport.spi.UpnpStream;
import org.teleal.common.util.Exceptions;

public class SwitchableRouterImpl implements SwitchableRouter {
    private static final Logger log = Logger.getLogger(Router.class.getName());
    private final UpnpServiceConfiguration configuration;
    private final ProtocolFactory protocolFactory;
    protected Lock readLock = this.routerLock.readLock();
    private Router router;
    protected ReentrantReadWriteLock routerLock = new ReentrantReadWriteLock(true);
    protected Lock writeLock = this.routerLock.writeLock();

    public static class RouterLockAcquisitionException extends RuntimeException {
        public RouterLockAcquisitionException(String s) {
            super(s);
        }

        public RouterLockAcquisitionException(String s, Throwable throwable) {
            super(s, throwable);
        }

        public RouterLockAcquisitionException(Throwable throwable) {
            super(throwable);
        }
    }

    class DisabledNetworkAddressFactory implements NetworkAddressFactory {
        DisabledNetworkAddressFactory() {
        }

        public InetAddress getMulticastGroup() {
            return null;
        }

        public int getMulticastPort() {
            return 0;
        }

        public int getStreamListenPort() {
            return 0;
        }

        public NetworkInterface[] getNetworkInterfaces() {
            return new NetworkInterface[0];
        }

        public InetAddress[] getBindAddresses() {
            return new InetAddress[0];
        }

        public byte[] getHardwareAddress(InetAddress inetAddress) {
            return new byte[0];
        }

        public InetAddress getBroadcastAddress(InetAddress inetAddress) {
            return null;
        }

        public InetAddress getLocalAddress(NetworkInterface networkInterface, boolean isIPv6, InetAddress remoteAddress) throws IllegalStateException {
            return null;
        }
    }

    public SwitchableRouterImpl(UpnpServiceConfiguration configuration, ProtocolFactory protocolFactory) {
        this.configuration = configuration;
        this.protocolFactory = protocolFactory;
    }

    public UpnpServiceConfiguration getConfiguration() {
        return this.configuration;
    }

    public ProtocolFactory getProtocolFactory() {
        return this.protocolFactory;
    }

    public boolean isEnabled() throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            boolean z = this.router != null;
            unlock(this.readLock);
            return z;
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public boolean enable() throws RouterLockAcquisitionException {
        lock(this.writeLock);
        try {
            if (this.router == null) {
                log.fine("Enabling network transport router");
                this.router = new RouterImpl(getConfiguration(), getProtocolFactory());
                unlock(this.writeLock);
                return true;
            }
        } catch (InitializationException ex) {
            handleStartFailure(ex);
        } catch (Throwable th) {
            unlock(this.writeLock);
        }
        unlock(this.writeLock);
        return false;
    }

    public void handleStartFailure(InitializationException ex) {
        log.severe("Unable to initialize network router: " + ex);
        log.severe("Cause: " + Exceptions.unwrap(ex));
    }

    public boolean disable() throws RouterLockAcquisitionException {
        lock(this.writeLock);
        try {
            if (this.router != null) {
                log.fine("Disabling network transport router");
                this.router.shutdown();
                this.router = null;
                return true;
            }
            log.info(" jangpw: Disabling network transport router");
            unlock(this.writeLock);
            return false;
        } finally {
            unlock(this.writeLock);
        }
    }

    public NetworkAddressFactory getNetworkAddressFactory() throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            NetworkAddressFactory networkAddressFactory = this.router != null ? this.router.getNetworkAddressFactory() : new DisabledNetworkAddressFactory();
            unlock(this.readLock);
            return networkAddressFactory;
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public List<NetworkAddress> getActiveStreamServers(InetAddress preferredAddress) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            List<NetworkAddress> activeStreamServers = this.router != null ? this.router.getActiveStreamServers(preferredAddress) : Collections.EMPTY_LIST;
            unlock(this.readLock);
            return activeStreamServers;
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public void shutdown() throws RouterLockAcquisitionException {
        disable();
    }

    public void received(IncomingDatagramMessage msg) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            if (this.router != null) {
                this.router.received(msg);
            }
            unlock(this.readLock);
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public void received(UpnpStream stream) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            if (this.router != null) {
                this.router.received(stream);
            }
            unlock(this.readLock);
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public void send(OutgoingDatagramMessage msg) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            if (this.router != null) {
                this.router.send(msg);
            }
            unlock(this.readLock);
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public StreamResponseMessage send(StreamRequestMessage msg) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            StreamResponseMessage send = this.router != null ? this.router.send(msg) : null;
            unlock(this.readLock);
            return send;
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    public void broadcast(byte[] bytes) throws RouterLockAcquisitionException {
        lock(this.readLock);
        try {
            if (this.router != null) {
                this.router.broadcast(bytes);
            }
            unlock(this.readLock);
        } catch (Throwable th) {
            unlock(this.readLock);
        }
    }

    protected void lock(Lock lock, int timeoutMilliseconds) throws RouterLockAcquisitionException {
        try {
            log.finest("Trying to obtain lock with timeout milliseconds '" + timeoutMilliseconds + "': " + lock.getClass().getSimpleName());
            if (lock.tryLock((long) timeoutMilliseconds, TimeUnit.MILLISECONDS)) {
                log.finest("Acquired router lock: " + lock.getClass().getSimpleName());
                return;
            }
            throw new RouterLockAcquisitionException("Failed to acquire router lock: " + lock.getClass().getSimpleName());
        } catch (InterruptedException e) {
            throw new RuntimeException("Failed to acquire router lock: " + lock.getClass().getSimpleName(), e);
        }
    }

    protected void lock(Lock lock) throws RouterLockAcquisitionException {
        lock(lock, getLockTimeoutMillis());
    }

    protected void unlock(Lock lock) {
        log.finest("Releasing router lock: " + lock.getClass().getSimpleName());
        lock.unlock();
    }

    protected int getLockTimeoutMillis() {
        return 6000;
    }
}
