package org.teleal.cling.transport.impl;

import java.io.StringReader;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilderFactory;
import org.teleal.cling.model.Constants;
import org.teleal.cling.model.XMLUtil;
import org.teleal.cling.model.message.UpnpMessage.BodyType;
import org.teleal.cling.model.message.gena.IncomingEventRequestMessage;
import org.teleal.cling.model.message.gena.OutgoingEventRequestMessage;
import org.teleal.cling.model.meta.StateVariable;
import org.teleal.cling.model.state.StateVariableValue;
import org.teleal.cling.transport.spi.GENAEventProcessor;
import org.teleal.cling.transport.spi.UnsupportedDataException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class GENAEventProcessorImpl implements GENAEventProcessor {
    private static Logger log = Logger.getLogger(GENAEventProcessor.class.getName());

    public void writeBody(OutgoingEventRequestMessage requestMessage) throws UnsupportedDataException {
        log.fine("Writing body of: " + requestMessage);
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            Document d = factory.newDocumentBuilder().newDocument();
            writeProperties(d, writePropertysetElement(d), requestMessage);
            requestMessage.setBody(BodyType.STRING, toString(d));
            if (log.isLoggable(Level.FINER)) {
                log.finer("===================================== GENA BODY BEGIN ============================================");
                log.finer(requestMessage.getBody().toString());
                log.finer("-===================================== GENA BODY END ============================================");
            }
        } catch (Exception ex) {
            throw new UnsupportedDataException("Can't transform message payload: " + ex.getMessage(), ex);
        }
    }

    public void readBody(IncomingEventRequestMessage requestMessage) throws UnsupportedDataException {
        log.fine("Reading body of: " + requestMessage);
        if (log.isLoggable(Level.FINER)) {
            log.finer("===================================== GENA BODY BEGIN ============================================");
            log.finer(requestMessage.getBody().toString());
            log.finer("-===================================== GENA BODY END ============================================");
        }
        if (requestMessage.getBody() == null || !requestMessage.getBodyType().equals(BodyType.STRING)) {
            throw new UnsupportedDataException("Can't transform null or non-string body of: " + requestMessage);
        }
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            readProperties(readPropertysetElement(factory.newDocumentBuilder().parse(new InputSource(new StringReader(requestMessage.getBodyString().trim())))), requestMessage);
        } catch (Exception ex) {
            throw new UnsupportedDataException("Can't transform message payload: " + ex.getMessage(), ex);
        }
    }

    protected Element writePropertysetElement(Document d) {
        Element propertysetElement = d.createElementNS(Constants.NS_UPNP_EVENT_10, "e:propertyset");
        d.appendChild(propertysetElement);
        return propertysetElement;
    }

    protected Element readPropertysetElement(Document d) {
        Element propertysetElement = d.getDocumentElement();
        if (propertysetElement != null && getUnprefixedNodeName(propertysetElement).equals("propertyset")) {
            return propertysetElement;
        }
        throw new RuntimeException("Root element was not 'propertyset'");
    }

    protected void writeProperties(Document d, Element propertysetElement, OutgoingEventRequestMessage message) {
        for (StateVariableValue stateVariableValue : message.getStateVariableValues()) {
            Element propertyElement = d.createElementNS(Constants.NS_UPNP_EVENT_10, "e:property");
            propertysetElement.appendChild(propertyElement);
            XMLUtil.appendNewElement(d, propertyElement, stateVariableValue.getStateVariable().getName(), stateVariableValue.toString());
        }
    }

    protected void readProperties(Element propertysetElement, IncomingEventRequestMessage message) {
        NodeList propertysetElementChildren = propertysetElement.getChildNodes();
        StateVariable[] stateVariables = message.getService().getStateVariables();
        for (int i = 0; i < propertysetElementChildren.getLength(); i++) {
            Node propertysetChild = propertysetElementChildren.item(i);
            if (propertysetChild.getNodeType() == (short) 1 && getUnprefixedNodeName(propertysetChild).equals("property")) {
                NodeList propertyChildren = propertysetChild.getChildNodes();
                for (int j = 0; j < propertyChildren.getLength(); j++) {
                    Node propertyChild = propertyChildren.item(j);
                    if (propertyChild.getNodeType() == (short) 1) {
                        String stateVariableName = getUnprefixedNodeName(propertyChild);
                        for (StateVariable stateVariable : stateVariables) {
                            if (stateVariable.getName().equals(stateVariableName)) {
                                log.fine("Reading state variable value: " + stateVariableName);
                                message.getStateVariableValues().add(new StateVariableValue(stateVariable, XMLUtil.getTextContent(propertyChild)));
                                break;
                            }
                        }
                    }
                }
            }
        }
    }

    protected String toString(Document d) throws Exception {
        String output = XMLUtil.documentToString(d);
        while (true) {
            if (!output.endsWith("\n") && !output.endsWith("\r")) {
                return output;
            }
            output = output.substring(0, output.length() - 1);
        }
    }

    protected String getUnprefixedNodeName(Node node) {
        return node.getPrefix() != null ? node.getNodeName().substring(node.getPrefix().length() + 1) : node.getNodeName();
    }
}
