package org.teleal.cling.transport.impl;

import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.InterfaceAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;
import org.teleal.cling.model.Constants;
import org.teleal.cling.transport.spi.InitializationException;
import org.teleal.cling.transport.spi.NetworkAddressFactory;
import org.teleal.common.util.OS;

public class NetworkAddressFactoryImpl implements NetworkAddressFactory {
    public static final int DEFAULT_TCP_HTTP_LISTEN_PORT = 0;
    private static Logger log = Logger.getLogger(NetworkAddressFactoryImpl.class.getName());
    protected List<InetAddress> bindAddresses;
    protected List<NetworkInterface> networkInterfaces;
    protected int streamListenPort;
    protected Set<String> useAddresses;
    protected Set<String> useInterfaces;

    public NetworkAddressFactoryImpl() throws InitializationException {
        this(0);
    }

    public NetworkAddressFactoryImpl(int streamListenPort) throws InitializationException {
        this.useInterfaces = new HashSet();
        this.useAddresses = new HashSet();
        this.networkInterfaces = new ArrayList();
        this.bindAddresses = new ArrayList();
        String useInterfacesString = System.getProperty(NetworkAddressFactory.SYSTEM_PROPERTY_NET_IFACES);
        if (useInterfacesString != null) {
            this.useInterfaces.addAll(Arrays.asList(useInterfacesString.split(",")));
        }
        String useAddressesString = System.getProperty(NetworkAddressFactory.SYSTEM_PROPERTY_NET_ADDRESSES);
        if (useAddressesString != null) {
            this.useAddresses.addAll(Arrays.asList(useAddressesString.split(",")));
        }
        if (OS.checkForLinux()) {
            Properties props = System.getProperties();
            props.setProperty("java.net.preferIPv6Stack", "true");
            System.setProperties(props);
        }
        discoverNetworkInterfaces();
        discoverBindAddresses();
        if (this.networkInterfaces.size() == 0 || this.bindAddresses.size() == 0) {
            throw new InitializationException("Could not discover any bindable network interfaces and/or addresses");
        }
        this.streamListenPort = streamListenPort;
    }

    public InetAddress getMulticastGroup() {
        try {
            return InetAddress.getByName(Constants.IPV4_UPNP_MULTICAST_GROUP);
        } catch (UnknownHostException ex) {
            throw new RuntimeException(ex);
        }
    }

    public int getMulticastPort() {
        return Constants.UPNP_MULTICAST_PORT;
    }

    public int getStreamListenPort() {
        return this.streamListenPort;
    }

    public NetworkInterface[] getNetworkInterfaces() {
        return (NetworkInterface[]) this.networkInterfaces.toArray(new NetworkInterface[this.networkInterfaces.size()]);
    }

    public InetAddress[] getBindAddresses() {
        return (InetAddress[]) this.bindAddresses.toArray(new InetAddress[this.bindAddresses.size()]);
    }

    public byte[] getHardwareAddress(InetAddress inetAddress) {
        try {
            NetworkInterface iface = NetworkInterface.getByInetAddress(inetAddress);
            return iface != null ? iface.getHardwareAddress() : null;
        } catch (SocketException ex) {
            throw new RuntimeException(ex);
        }
    }

    public InetAddress getBroadcastAddress(InetAddress inetAddress) {
        for (NetworkInterface iface : this.networkInterfaces) {
            for (InterfaceAddress interfaceAddress : getInterfaceAddresses(iface)) {
                if (interfaceAddress != null && interfaceAddress.getAddress().equals(inetAddress)) {
                    return interfaceAddress.getBroadcast();
                }
            }
        }
        return null;
    }

    public InetAddress getLocalAddress(NetworkInterface networkInterface, boolean isIPv6, InetAddress remoteAddress) {
        InetAddress localIPInSubnet = getBindAddressInSubnetOf(remoteAddress);
        if (localIPInSubnet != null) {
            return localIPInSubnet;
        }
        log.finer("Could not find local bind address in same subnet as: " + remoteAddress.getHostAddress());
        for (InetAddress interfaceAddress : getInetAddresses(networkInterface)) {
            if (isIPv6 && (interfaceAddress instanceof Inet6Address)) {
                return interfaceAddress;
            }
            if (!isIPv6 && (interfaceAddress instanceof Inet4Address)) {
                return interfaceAddress;
            }
        }
        throw new IllegalStateException("Can't find any IPv4 or IPv6 address on interface: " + networkInterface.getDisplayName());
    }

    protected List<InterfaceAddress> getInterfaceAddresses(NetworkInterface networkInterface) {
        return networkInterface.getInterfaceAddresses();
    }

    protected List<InetAddress> getInetAddresses(NetworkInterface networkInterface) {
        return Collections.list(networkInterface.getInetAddresses());
    }

    protected InetAddress getBindAddressInSubnetOf(InetAddress inetAddress) {
        for (NetworkInterface iface : this.networkInterfaces) {
            for (InterfaceAddress ifaceAddress : getInterfaceAddresses(iface)) {
                if (this.bindAddresses.contains(ifaceAddress.getAddress()) && isInSubnet(inetAddress.getAddress(), ifaceAddress.getAddress().getAddress(), ifaceAddress.getNetworkPrefixLength())) {
                    return ifaceAddress.getAddress();
                }
            }
        }
        return null;
    }

    protected boolean isInSubnet(byte[] ip, byte[] network, short prefix) {
        boolean z = true;
        if (ip.length != network.length || prefix / 8 > ip.length) {
            return false;
        }
        int i = 0;
        while (prefix >= (short) 8 && i < ip.length) {
            if (ip[i] != network[i]) {
                return false;
            }
            i++;
            prefix = (short) (prefix - 8);
        }
        byte mask = (byte) (((1 << (8 - prefix)) - 1) ^ -1);
        if ((ip[i] & mask) != (network[i] & mask)) {
            z = false;
        }
        return z;
    }

    protected void discoverNetworkInterfaces() throws InitializationException {
        try {
            Iterator i$ = Collections.list(NetworkInterface.getNetworkInterfaces()).iterator();
            while (i$.hasNext()) {
                NetworkInterface iface = (NetworkInterface) i$.next();
                log.finer("Analyzing network interface: " + iface.getDisplayName());
                if (isUsableNetworkInterface(iface)) {
                    log.fine("Discovered usable network interface: " + iface.getDisplayName());
                    this.networkInterfaces.add(iface);
                } else {
                    log.finer("Ignoring non-usable network interface: " + iface.getDisplayName());
                }
            }
        } catch (Exception ex) {
            throw new InitializationException("Could not not analyze local network interfaces: " + ex, ex);
        }
    }

    protected boolean isUsableNetworkInterface(NetworkInterface iface) throws Exception {
        if (!iface.isUp()) {
            log.finer("Skipping network interface (down): " + iface.getDisplayName());
            return false;
        } else if (getInetAddresses(iface).size() == 0) {
            log.finer("Skipping network interface without bound IP addresses: " + iface.getDisplayName());
            return false;
        } else if (iface.getName().toLowerCase().startsWith("vmnet") || iface.getDisplayName().toLowerCase().contains("vmnet")) {
            log.finer("Skipping network interface (VMWare): " + iface.getDisplayName());
            return false;
        } else if (iface.getName().toLowerCase().startsWith("vnic")) {
            log.finer("Skipping network interface (Parallels): " + iface.getDisplayName());
            return false;
        } else if (iface.getName().toLowerCase().startsWith("ppp")) {
            log.finer("Skipping network interface (PPP): " + iface.getDisplayName());
            return false;
        } else if (!iface.supportsMulticast()) {
            log.finer("Skipping network interface (no multicast support): " + iface.getDisplayName());
            return false;
        } else if (iface.isLoopback()) {
            log.finer("Skipping network interface (ignoring loopback): " + iface.getDisplayName());
            return false;
        } else if (this.useInterfaces.size() <= 0 || this.useInterfaces.contains(iface.getName())) {
            return true;
        } else {
            log.finer("Skipping unwanted network interface (-Dorg.teleal.cling.network.useInterfaces): " + iface.getName());
            return false;
        }
    }

    protected void discoverBindAddresses() throws InitializationException {
        try {
            Iterator<NetworkInterface> it = this.networkInterfaces.iterator();
            while (it.hasNext()) {
                NetworkInterface networkInterface = (NetworkInterface) it.next();
                log.finer("Discovering addresses of interface: " + networkInterface.getDisplayName());
                int usableAddresses = 0;
                for (InetAddress inetAddress : getInetAddresses(networkInterface)) {
                    if (inetAddress == null) {
                        log.warning("Network has a null address: " + networkInterface.getDisplayName());
                    } else if (isUsableAddress(networkInterface, inetAddress)) {
                        log.fine("Discovered usable network interface address: " + inetAddress.getHostAddress());
                        usableAddresses++;
                        this.bindAddresses.add(inetAddress);
                    } else {
                        log.finer("Ignoring non-usable network interface address: " + inetAddress.getHostAddress());
                    }
                }
                if (usableAddresses == 0) {
                    log.finer("Network interface has no usable addresses, removing: " + networkInterface.getDisplayName());
                    it.remove();
                }
            }
        } catch (Exception ex) {
            throw new InitializationException("Could not not analyze local network interfaces: " + ex, ex);
        }
    }

    protected boolean isUsableAddress(NetworkInterface networkInterface, InetAddress address) {
        if (!(address instanceof Inet4Address)) {
            log.finer("Skipping unsupported non-IPv4 address: " + address);
            return false;
        } else if (address.isLoopbackAddress()) {
            log.finer("Skipping loopback address: " + address);
            return false;
        } else if (this.useAddresses.size() <= 0 || this.useAddresses.contains(address.getHostAddress())) {
            return true;
        } else {
            log.finer("Skipping unwanted address: " + address);
            return false;
        }
    }

    static void displayInterfaceInformation(NetworkInterface netint) throws SocketException {
        System.out.printf("Parent Info:%s\n", new Object[]{netint.getParent()});
        System.out.printf("Display name: %s\n", new Object[]{netint.getDisplayName()});
        System.out.printf("Name: %s\n", new Object[]{netint.getName()});
        Iterator i$ = Collections.list(netint.getInetAddresses()).iterator();
        while (i$.hasNext()) {
            InetAddress inetAddress = (InetAddress) i$.next();
            System.out.printf("InetAddress: %s\n", new Object[]{inetAddress});
        }
        for (InterfaceAddress ia : netint.getInterfaceAddresses()) {
            System.out.println(" Interface Address");
            System.out.println("  Address: " + ia.getAddress());
            System.out.println("  Broadcast: " + ia.getBroadcast());
            System.out.println("  Prefix length: " + ia.getNetworkPrefixLength());
        }
        i$ = Collections.list(netint.getSubInterfaces()).iterator();
        while (i$.hasNext()) {
            NetworkInterface subIf = (NetworkInterface) i$.next();
            System.out.printf("\tSub Interface Display name: %s\n", new Object[]{subIf.getDisplayName()});
            System.out.printf("\tSub Interface Name: %s\n", new Object[]{subIf.getName()});
        }
        System.out.printf("Up? %s\n", new Object[]{Boolean.valueOf(netint.isUp())});
        System.out.printf("Loopback? %s\n", new Object[]{Boolean.valueOf(netint.isLoopback())});
        System.out.printf("PointToPoint? %s\n", new Object[]{Boolean.valueOf(netint.isPointToPoint())});
        System.out.printf("Supports multicast? %s\n", new Object[]{Boolean.valueOf(netint.supportsMulticast())});
        System.out.printf("Virtual? %s\n", new Object[]{Boolean.valueOf(netint.isVirtual())});
        System.out.printf("Hardware address: %s\n", new Object[]{Arrays.toString(netint.getHardwareAddress())});
        System.out.printf("MTU: %s\n", new Object[]{Integer.valueOf(netint.getMTU())});
        System.out.printf("\n", new Object[0]);
    }
}
