package org.teleal.cling.transport.impl.apache;

import java.io.IOException;
import java.io.InputStream;
import java.net.SocketTimeoutException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.http.ConnectionClosedException;
import org.apache.http.ConnectionReuseStrategy;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpException;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpResponseFactory;
import org.apache.http.HttpServerConnection;
import org.apache.http.MethodNotSupportedException;
import org.apache.http.ProtocolVersion;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.DefaultConnectionReuseStrategy;
import org.apache.http.impl.DefaultHttpResponseFactory;
import org.apache.http.message.BasicStatusLine;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.DefaultedHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.BasicHttpProcessor;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpProcessor;
import org.apache.http.protocol.HttpService;
import org.apache.http.protocol.ResponseConnControl;
import org.apache.http.protocol.ResponseContent;
import org.apache.http.protocol.ResponseDate;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.UpnpHeaders;
import org.teleal.cling.model.message.UpnpMessage.BodyType;
import org.teleal.cling.model.message.UpnpOperation;
import org.teleal.cling.model.message.UpnpRequest;
import org.teleal.cling.model.message.UpnpRequest.Method;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.protocol.ProtocolFactory;
import org.teleal.cling.transport.spi.UnsupportedDataException;
import org.teleal.cling.transport.spi.UpnpStream;
import org.teleal.common.io.IO;
import org.teleal.common.util.Exceptions;

public class HttpServerConnectionUpnpStream extends UpnpStream {
    private static final Logger log = Logger.getLogger(UpnpStream.class.getName());
    protected final HttpServerConnection connection;
    protected final BasicHttpProcessor httpProcessor = new BasicHttpProcessor();
    protected final HttpService httpService;
    protected final HttpParams params;

    protected class UpnpHttpService extends HttpService {
        public UpnpHttpService(HttpProcessor processor, ConnectionReuseStrategy reuse, HttpResponseFactory responseFactory) {
            super(processor, reuse, responseFactory);
        }

        protected void doService(HttpRequest httpRequest, HttpResponse httpResponse, HttpContext ctx) throws HttpException, IOException {
            HttpServerConnectionUpnpStream.log.fine("Processing HTTP request: " + httpRequest.getRequestLine().toString());
            String requestMethod = httpRequest.getRequestLine().getMethod();
            String requestURI = httpRequest.getRequestLine().getUri();
            try {
                StreamRequestMessage requestMessage = new StreamRequestMessage(Method.getByHttpName(requestMethod), URI.create(requestURI));
                if (((UpnpRequest) requestMessage.getOperation()).getMethod().equals(Method.UNKNOWN)) {
                    HttpServerConnectionUpnpStream.log.fine("Method not supported by UPnP stack: " + requestMethod);
                    throw new MethodNotSupportedException("Method not supported: " + requestMethod);
                }
                HttpServerConnectionUpnpStream.log.fine("Created new request message: " + requestMessage);
                ((UpnpRequest) requestMessage.getOperation()).setHttpMinorVersion(httpRequest.getProtocolVersion().getMinor());
                requestMessage.setHeaders(new UpnpHeaders(HeaderUtil.get(httpRequest)));
                if (httpRequest instanceof HttpEntityEnclosingRequest) {
                    HttpServerConnectionUpnpStream.log.fine("Request contains entity body, setting on UPnP message");
                    InputStream inputStream = null;
                    try {
                        inputStream = ((HttpEntityEnclosingRequest) httpRequest).getEntity().getContent();
                        byte[] bodyBytes = IO.readBytes(inputStream);
                        if (bodyBytes.length > 0 && requestMessage.isContentTypeMissingOrText()) {
                            HttpServerConnectionUpnpStream.log.fine("Request contains textual entity body, converting then setting string on message");
                            requestMessage.setBodyCharacters(bodyBytes);
                        } else if (bodyBytes.length > 0) {
                            HttpServerConnectionUpnpStream.log.fine("Request contains binary entity body, setting bytes on message");
                            requestMessage.setBody(BodyType.BYTES, bodyBytes);
                        } else {
                            HttpServerConnectionUpnpStream.log.fine("Request did not contain entity body");
                        }
                    } finally {
                        if (inputStream != null) {
                            inputStream.close();
                        }
                    }
                } else {
                    HttpServerConnectionUpnpStream.log.fine("Request did not contain entity body");
                }
                try {
                    StreamResponseMessage responseMsg = HttpServerConnectionUpnpStream.this.process(requestMessage);
                    if (responseMsg != null) {
                        HttpServerConnectionUpnpStream.log.fine("Sending HTTP response message: " + responseMsg);
                        ProtocolVersion protocolVersion = new ProtocolVersion("HTTP", 1, ((UpnpResponse) responseMsg.getOperation()).getHttpMinorVersion());
                        httpResponse.setStatusLine(new BasicStatusLine(protocolVersion, ((UpnpResponse) responseMsg.getOperation()).getStatusCode(), ((UpnpResponse) responseMsg.getOperation()).getStatusMessage()));
                        HttpServerConnectionUpnpStream.log.fine("Response status line: " + httpResponse.getStatusLine());
                        httpResponse.setParams(getResponseParams(requestMessage.getOperation()));
                        HeaderUtil.add(httpResponse, responseMsg.getHeaders());
                        if (responseMsg.hasBody() && responseMsg.getBodyType().equals(BodyType.BYTES)) {
                            httpResponse.setEntity(new ByteArrayEntity(responseMsg.getBodyBytes()));
                        } else if (responseMsg.hasBody() && responseMsg.getBodyType().equals(BodyType.STRING)) {
                            httpResponse.setEntity(new StringEntity(responseMsg.getBodyString(), "UTF-8"));
                        }
                    } else {
                        HttpServerConnectionUpnpStream.log.fine("Sending HTTP response: 404");
                        httpResponse.setStatusCode(404);
                    }
                    HttpServerConnectionUpnpStream.this.responseSent(responseMsg);
                } catch (RuntimeException ex) {
                    HttpServerConnectionUpnpStream.log.fine("Exception occured during UPnP stream processing: " + ex);
                    if (HttpServerConnectionUpnpStream.log.isLoggable(Level.FINE)) {
                        HttpServerConnectionUpnpStream.log.log(Level.FINE, "Cause: " + Exceptions.unwrap(ex), Exceptions.unwrap(ex));
                    }
                    HttpServerConnectionUpnpStream.log.fine("Sending HTTP response: 500");
                    httpResponse.setStatusCode(500);
                    HttpServerConnectionUpnpStream.this.responseException(ex);
                }
            } catch (IllegalArgumentException e) {
                String msg = "Invalid request URI: " + requestURI + ": " + e.getMessage();
                HttpServerConnectionUpnpStream.log.warning(msg);
                throw new HttpException(msg, e);
            }
        }

        protected HttpParams getResponseParams(UpnpOperation operation) {
            return new DefaultedHttpParams(new BasicHttpParams(), HttpServerConnectionUpnpStream.this.params);
        }
    }

    protected HttpServerConnectionUpnpStream(ProtocolFactory protocolFactory, HttpServerConnection connection, HttpParams params) {
        super(protocolFactory);
        this.connection = connection;
        this.params = params;
        this.httpProcessor.addInterceptor(new ResponseDate());
        this.httpProcessor.addInterceptor(new ResponseContent());
        this.httpProcessor.addInterceptor(new ResponseConnControl());
        this.httpService = new UpnpHttpService(this.httpProcessor, new DefaultConnectionReuseStrategy(), new DefaultHttpResponseFactory());
        this.httpService.setParams(params);
    }

    public HttpServerConnection getConnection() {
        return this.connection;
    }

    public void run() {
        while (!Thread.interrupted() && this.connection.isOpen()) {
            try {
                log.fine("Handling request on open connection...");
                this.httpService.handleRequest(this.connection, new BasicHttpContext(null));
            } catch (ConnectionClosedException ex) {
                log.fine("Client closed connection");
                responseException(ex);
                try {
                    this.connection.shutdown();
                    return;
                } catch (IOException ex2) {
                    log.fine("Error closing connection: " + ex2.getMessage());
                    return;
                }
            } catch (SocketTimeoutException ex3) {
                log.fine("Server-side closed socket (this is 'normal' behavior of Apache HTTP Core!): " + ex3.getMessage());
                try {
                    this.connection.shutdown();
                    return;
                } catch (IOException ex22) {
                    log.fine("Error closing connection: " + ex22.getMessage());
                    return;
                }
            } catch (IOException ex222) {
                log.warning("I/O exception during HTTP request processing: " + ex222.getMessage());
                responseException(ex222);
                try {
                    this.connection.shutdown();
                    return;
                } catch (IOException ex2222) {
                    log.fine("Error closing connection: " + ex2222.getMessage());
                    return;
                }
            } catch (HttpException ex4) {
                throw new UnsupportedDataException("Request malformed: " + ex4.getMessage(), ex4);
            } catch (Throwable th) {
                try {
                    this.connection.shutdown();
                } catch (IOException ex22222) {
                    log.fine("Error closing connection: " + ex22222.getMessage());
                }
            }
        }
        try {
            this.connection.shutdown();
        } catch (IOException ex222222) {
            log.fine("Error closing connection: " + ex222222.getMessage());
        }
    }
}
