package org.teleal.cling.transport.impl.apache;

import java.io.IOException;
import java.util.logging.Logger;
import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.MethodNotSupportedException;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.DefaultedHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.util.EntityUtils;
import org.teleal.cling.model.message.StreamRequestMessage;
import org.teleal.cling.model.message.StreamResponseMessage;
import org.teleal.cling.model.message.UpnpHeaders;
import org.teleal.cling.model.message.UpnpMessage;
import org.teleal.cling.model.message.UpnpMessage.BodyType;
import org.teleal.cling.model.message.UpnpRequest;
import org.teleal.cling.model.message.UpnpRequest.Method;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.transport.spi.InitializationException;
import org.teleal.cling.transport.spi.StreamClient;
import org.teleal.common.mock.http.MockHttpServletRequest;
import org.teleal.common.util.Exceptions;

public class StreamClientImpl implements StreamClient<StreamClientConfigurationImpl> {
    private static final Logger log = Logger.getLogger(StreamClient.class.getName());
    protected final ThreadSafeClientConnManager clientConnectionManager;
    protected final StreamClientConfigurationImpl configuration;
    protected final HttpParams globalParams = new BasicHttpParams();
    protected final DefaultHttpClient httpClient;

    class C04234 implements ResponseHandler<StreamResponseMessage> {
        C04234() {
        }

        public StreamResponseMessage handleResponse(HttpResponse httpResponse) throws IOException {
            StatusLine statusLine = httpResponse.getStatusLine();
            StreamClientImpl.log.fine("Received HTTP response: " + statusLine);
            StreamResponseMessage responseMessage = new StreamResponseMessage(new UpnpResponse(statusLine.getStatusCode(), statusLine.getReasonPhrase()));
            responseMessage.setHeaders(new UpnpHeaders(HeaderUtil.get(httpResponse)));
            HttpEntity entity = httpResponse.getEntity();
            if (!(entity == null || entity.getContentLength() == 0)) {
                if (responseMessage.isContentTypeMissingOrText()) {
                    StreamClientImpl.log.fine("HTTP response message contains text entity");
                    responseMessage.setBody(BodyType.STRING, EntityUtils.toString(entity));
                } else {
                    StreamClientImpl.log.fine("HTTP response message contains binary entity");
                    responseMessage.setBody(BodyType.BYTES, EntityUtils.toByteArray(entity));
                }
            }
            return responseMessage;
        }
    }

    public StreamClientImpl(StreamClientConfigurationImpl configuration) throws InitializationException {
        this.configuration = configuration;
        ConnManagerParams.setMaxTotalConnections(this.globalParams, getConfiguration().getMaxTotalConnections());
        HttpConnectionParams.setConnectionTimeout(this.globalParams, getConfiguration().getConnectionTimeoutSeconds() * 1000);
        HttpConnectionParams.setSoTimeout(this.globalParams, getConfiguration().getDataReadTimeoutSeconds() * 1000);
        HttpProtocolParams.setContentCharset(this.globalParams, getConfiguration().getContentCharset());
        if (getConfiguration().getSocketBufferSize() != -1) {
            HttpConnectionParams.setSocketBufferSize(this.globalParams, getConfiguration().getSocketBufferSize());
        }
        HttpConnectionParams.setStaleCheckingEnabled(this.globalParams, getConfiguration().getStaleCheckingEnabled());
        SchemeRegistry registry = new SchemeRegistry();
        registry.register(new Scheme(MockHttpServletRequest.DEFAULT_PROTOCOL, PlainSocketFactory.getSocketFactory(), 80));
        this.clientConnectionManager = new ThreadSafeClientConnManager(this.globalParams, registry);
        this.httpClient = new DefaultHttpClient(this.clientConnectionManager, this.globalParams);
        if (getConfiguration().getRequestRetryCount() != -1) {
            this.httpClient.setHttpRequestRetryHandler(new DefaultHttpRequestRetryHandler(getConfiguration().getRequestRetryCount(), false));
        }
    }

    public StreamClientConfigurationImpl getConfiguration() {
        return this.configuration;
    }

    public StreamResponseMessage sendRequest(StreamRequestMessage requestMessage) {
        UpnpRequest requestOperation = (UpnpRequest) requestMessage.getOperation();
        log.fine("Preparing HTTP request message with method '" + requestOperation.getHttpMethodName() + "': " + requestMessage);
        try {
            HttpUriRequest httpRequest = createHttpRequest(requestMessage, requestOperation);
            httpRequest.setParams(getRequestParams(requestMessage));
            HeaderUtil.add(httpRequest, requestMessage.getHeaders());
            log.fine("Sending HTTP request: " + httpRequest.getURI());
            return (StreamResponseMessage) this.httpClient.execute(httpRequest, createResponseHandler());
        } catch (MethodNotSupportedException ex) {
            log.warning("Request aborted: " + ex.toString());
            return null;
        } catch (ClientProtocolException ex2) {
            log.warning("HTTP protocol exception executing request: " + requestMessage);
            log.warning("Cause: " + Exceptions.unwrap(ex2));
            return null;
        } catch (IOException ex3) {
            log.fine("Client connection was aborted: " + ex3.getMessage());
            return null;
        }
    }

    public void stop() {
        log.fine("Shutting down HTTP client connection manager/pool");
        this.clientConnectionManager.shutdown();
    }

    protected HttpUriRequest createHttpRequest(UpnpMessage upnpMessage, UpnpRequest upnpRequestOperation) throws MethodNotSupportedException {
        switch (upnpRequestOperation.getMethod()) {
            case GET:
                return new HttpGet(upnpRequestOperation.getURI());
            case SUBSCRIBE:
                return new HttpGet(upnpRequestOperation.getURI()) {
                    public String getMethod() {
                        return Method.SUBSCRIBE.getHttpName();
                    }
                };
            case UNSUBSCRIBE:
                return new HttpGet(upnpRequestOperation.getURI()) {
                    public String getMethod() {
                        return Method.UNSUBSCRIBE.getHttpName();
                    }
                };
            case POST:
                HttpEntityEnclosingRequest post = new HttpPost(upnpRequestOperation.getURI());
                post.setEntity(createHttpRequestEntity(upnpMessage));
                return (HttpUriRequest) post;
            case NOTIFY:
                HttpEntityEnclosingRequest notify = new HttpPost(upnpRequestOperation.getURI()) {
                    public String getMethod() {
                        return Method.NOTIFY.getHttpName();
                    }
                };
                notify.setEntity(createHttpRequestEntity(upnpMessage));
                return (HttpUriRequest) notify;
            default:
                throw new MethodNotSupportedException(upnpRequestOperation.getHttpMethodName());
        }
    }

    protected HttpEntity createHttpRequestEntity(UpnpMessage upnpMessage) {
        if (upnpMessage.getBodyType().equals(BodyType.BYTES)) {
            log.fine("Preparing HTTP request entity as byte[]");
            return new ByteArrayEntity(upnpMessage.getBodyBytes());
        }
        log.fine("Preparing HTTP request entity as string");
        try {
            String charset = upnpMessage.getContentTypeCharset();
            String bodyString = upnpMessage.getBodyString();
            if (charset == null) {
                charset = "UTF-8";
            }
            return new StringEntity(bodyString, charset);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    protected ResponseHandler<StreamResponseMessage> createResponseHandler() {
        return new C04234();
    }

    protected HttpParams getRequestParams(StreamRequestMessage requestMessage) {
        HttpParams localParams = new BasicHttpParams();
        localParams.setParameter("http.protocol.version", ((UpnpRequest) requestMessage.getOperation()).getHttpMinorVersion() == 0 ? HttpVersion.HTTP_1_0 : HttpVersion.HTTP_1_1);
        HttpProtocolParams.setUserAgent(localParams, getConfiguration().getUserAgentValue(requestMessage.getUdaMajorVersion(), requestMessage.getUdaMinorVersion()));
        return new DefaultedHttpParams(localParams, this.globalParams);
    }
}
