package org.teleal.common.http;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.teleal.cling.model.message.header.EXTHeader;

public class CacheControl {
    private Map<String, String> cacheExtensions = new HashMap();
    private int maxAge = -1;
    private boolean mustRevalidate = false;
    private boolean noCache = false;
    private List<String> noCacheFields = new ArrayList();
    private boolean noStore = false;
    private boolean noTransform = true;
    private List<String> privateFields = new ArrayList();
    private boolean privateFlag = false;
    private boolean proxyRevalidate = false;
    private int sharedMaxAge = -1;

    public int getMaxAge() {
        return this.maxAge;
    }

    public void setMaxAge(int maxAge) {
        this.maxAge = maxAge;
    }

    public int getSharedMaxAge() {
        return this.sharedMaxAge;
    }

    public void setSharedMaxAge(int sharedMaxAge) {
        this.sharedMaxAge = sharedMaxAge;
    }

    public boolean isNoCache() {
        return this.noCache;
    }

    public void setNoCache(boolean noCache) {
        this.noCache = noCache;
    }

    public List<String> getNoCacheFields() {
        return this.noCacheFields;
    }

    public void setNoCacheFields(List<String> noCacheFields) {
        this.noCacheFields = noCacheFields;
    }

    public boolean isPrivateFlag() {
        return this.privateFlag;
    }

    public void setPrivateFlag(boolean privateFlag) {
        this.privateFlag = privateFlag;
    }

    public List<String> getPrivateFields() {
        return this.privateFields;
    }

    public void setPrivateFields(List<String> privateFields) {
        this.privateFields = privateFields;
    }

    public boolean isNoStore() {
        return this.noStore;
    }

    public void setNoStore(boolean noStore) {
        this.noStore = noStore;
    }

    public boolean isNoTransform() {
        return this.noTransform;
    }

    public void setNoTransform(boolean noTransform) {
        this.noTransform = noTransform;
    }

    public boolean isMustRevalidate() {
        return this.mustRevalidate;
    }

    public void setMustRevalidate(boolean mustRevalidate) {
        this.mustRevalidate = mustRevalidate;
    }

    public boolean isProxyRevalidate() {
        return this.proxyRevalidate;
    }

    public void setProxyRevalidate(boolean proxyRevalidate) {
        this.proxyRevalidate = proxyRevalidate;
    }

    public Map<String, String> getCacheExtensions() {
        return this.cacheExtensions;
    }

    public void setCacheExtensions(Map<String, String> cacheExtensions) {
        this.cacheExtensions = cacheExtensions;
    }

    public static CacheControl valueOf(String s) throws IllegalArgumentException {
        if (s == null) {
            return null;
        }
        CacheControl result = new CacheControl();
        for (String directive : s.split(",")) {
            String[] nameValue = directive.trim().split("=");
            String name = nameValue[0].trim();
            String value = null;
            if (nameValue.length > 1) {
                value = nameValue[1].trim();
                if (value.startsWith("\"")) {
                    value = value.substring(1);
                }
                if (value.endsWith("\"")) {
                    value = value.substring(0, value.length() - 1);
                }
            }
            String lowercase = name.toLowerCase();
            if ("no-cache".equals(lowercase)) {
                result.setNoCache(true);
                if (!(value == null || EXTHeader.DEFAULT_VALUE.equals(value))) {
                    result.getNoCacheFields().add(value);
                }
            } else if ("private".equals(lowercase)) {
                result.setPrivateFlag(true);
                if (!(value == null || EXTHeader.DEFAULT_VALUE.equals(value))) {
                    result.getPrivateFields().add(value);
                }
            } else if ("no-store".equals(lowercase)) {
                result.setNoStore(true);
            } else if ("max-age".equals(lowercase)) {
                if (value == null) {
                    throw new IllegalArgumentException("CacheControl max-age header does not have a value: " + value);
                }
                result.setMaxAge(Integer.valueOf(value).intValue());
            } else if ("s-maxage".equals(lowercase)) {
                if (value == null) {
                    throw new IllegalArgumentException("CacheControl s-maxage header does not have a value: " + value);
                }
                result.setSharedMaxAge(Integer.valueOf(value).intValue());
            } else if ("no-transform".equals(lowercase)) {
                result.setNoTransform(true);
            } else if ("must-revalidate".equals(lowercase)) {
                result.setMustRevalidate(true);
            } else if ("proxy-revalidate".equals(lowercase)) {
                result.setProxyRevalidate(true);
            } else if (!"public".equals(lowercase)) {
                if (value == null) {
                    value = EXTHeader.DEFAULT_VALUE;
                }
                result.getCacheExtensions().put(name, value);
            }
        }
        return result;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (!isPrivateFlag()) {
            sb.append("public");
        }
        if (isMustRevalidate()) {
            append("must-revalidate", sb);
        }
        if (isNoTransform()) {
            append("no-transform", sb);
        }
        if (isNoStore()) {
            append("no-store", sb);
        }
        if (isProxyRevalidate()) {
            append("proxy-revalidate", sb);
        }
        if (getSharedMaxAge() > -1) {
            append("s-maxage", sb).append("=").append(getSharedMaxAge());
        }
        if (getMaxAge() > -1) {
            append("max-age", sb).append("=").append(getMaxAge());
        }
        if (isNoCache()) {
            if (getNoCacheFields().size() < 1) {
                append("no-cache", sb);
            } else {
                for (String field : getNoCacheFields()) {
                    append("no-cache", sb).append("=\"").append(field).append("\"");
                }
            }
        }
        if (isPrivateFlag()) {
            if (getPrivateFields().size() < 1) {
                append("private", sb);
            } else {
                for (String field2 : getPrivateFields()) {
                    append("private", sb).append("=\"").append(field2).append("\"");
                }
            }
        }
        for (String key : getCacheExtensions().keySet()) {
            String val = (String) getCacheExtensions().get(key);
            append(key, sb);
            if (!(val == null || EXTHeader.DEFAULT_VALUE.equals(val))) {
                sb.append("=\"").append(val).append("\"");
            }
        }
        return sb.toString();
    }

    private StringBuilder append(String s, StringBuilder sb) {
        if (sb.length() > 0) {
            sb.append(", ");
        }
        sb.append(s);
        return sb;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        CacheControl that = (CacheControl) o;
        if (this.maxAge != that.maxAge) {
            return false;
        }
        if (this.mustRevalidate != that.mustRevalidate) {
            return false;
        }
        if (this.noCache != that.noCache) {
            return false;
        }
        if (this.noStore != that.noStore) {
            return false;
        }
        if (this.noTransform != that.noTransform) {
            return false;
        }
        if (this.privateFlag != that.privateFlag) {
            return false;
        }
        if (this.proxyRevalidate != that.proxyRevalidate) {
            return false;
        }
        if (this.sharedMaxAge != that.sharedMaxAge) {
            return false;
        }
        if (!this.cacheExtensions.equals(that.cacheExtensions)) {
            return false;
        }
        if (!this.noCacheFields.equals(that.noCacheFields)) {
            return false;
        }
        if (this.privateFields.equals(that.privateFields)) {
            return true;
        }
        return false;
    }

    public int hashCode() {
        int i;
        int i2 = 1;
        int hashCode = ((((((this.maxAge * 31) + this.sharedMaxAge) * 31) + (this.noCache ? 1 : 0)) * 31) + this.noCacheFields.hashCode()) * 31;
        if (this.privateFlag) {
            i = 1;
        } else {
            i = 0;
        }
        hashCode = (((hashCode + i) * 31) + this.privateFields.hashCode()) * 31;
        if (this.noStore) {
            i = 1;
        } else {
            i = 0;
        }
        hashCode = (hashCode + i) * 31;
        if (this.noTransform) {
            i = 1;
        } else {
            i = 0;
        }
        hashCode = (hashCode + i) * 31;
        if (this.mustRevalidate) {
            i = 1;
        } else {
            i = 0;
        }
        i = (hashCode + i) * 31;
        if (!this.proxyRevalidate) {
            i2 = 0;
        }
        return ((i + i2) * 31) + this.cacheExtensions.hashCode();
    }
}
