package org.teleal.common.http;

import java.io.ByteArrayInputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class Headers implements Map<String, List<String>> {
    static final byte CR = (byte) 13;
    static final byte LF = (byte) 10;
    final Map<String, List<String>> map = new HashMap(32);

    public Headers(Map<String, List<String>> map) {
        putAll(map);
    }

    public Headers(ByteArrayInputStream inputStream) {
        Headers headers = new Headers();
        String line = readLine(inputStream);
        String lastHeader = null;
        if (line.length() != 0) {
            do {
                char firstChar = line.charAt(0);
                if (lastHeader == null || !(firstChar == ' ' || firstChar == '\t')) {
                    String[] header = splitHeader(line);
                    headers.add(header[0], header[1]);
                    lastHeader = header[0];
                } else {
                    List<String> current = headers.get((Object) lastHeader);
                    int lastPos = current.size() - 1;
                    current.set(lastPos, ((String) current.get(lastPos)) + line.trim());
                }
                line = readLine(inputStream);
            } while (line.length() != 0);
        }
        putAll(headers);
    }

    public int size() {
        return this.map.size();
    }

    public boolean isEmpty() {
        return this.map.isEmpty();
    }

    public boolean containsKey(Object key) {
        return key != null && (key instanceof String) && this.map.containsKey(normalize((String) key));
    }

    public boolean containsValue(Object value) {
        return this.map.containsValue(value);
    }

    public List<String> get(Object key) {
        return (List) this.map.get(normalize((String) key));
    }

    public List<String> put(String key, List<String> value) {
        return (List) this.map.put(normalize(key), value);
    }

    public List<String> remove(Object key) {
        return (List) this.map.remove(normalize((String) key));
    }

    public void putAll(Map<? extends String, ? extends List<String>> t) {
        for (Entry<? extends String, ? extends List<String>> entry : t.entrySet()) {
            put((String) entry.getKey(), (List) entry.getValue());
        }
    }

    public void clear() {
        this.map.clear();
    }

    public Set<String> keySet() {
        return this.map.keySet();
    }

    public Collection<List<String>> values() {
        return this.map.values();
    }

    public Set<Entry<String, List<String>>> entrySet() {
        return this.map.entrySet();
    }

    public boolean equals(Object o) {
        return this.map.equals(o);
    }

    public int hashCode() {
        return this.map.hashCode();
    }

    public String getFirstHeader(String key) {
        List<String> l = (List) this.map.get(normalize(key));
        return (l == null || l.size() <= 0) ? null : (String) l.get(0);
    }

    public void add(String key, String value) {
        String k = normalize(key);
        List<String> l = (List) this.map.get(k);
        if (l == null) {
            l = new LinkedList();
            this.map.put(k, l);
        }
        l.add(value);
    }

    public void set(String key, String value) {
        List l = new LinkedList();
        l.add(value);
        put(key, l);
    }

    private String normalize(String key) {
        if (key == null) {
            return null;
        }
        if (key.length() == 0) {
            return key;
        }
        char[] b = key.toCharArray();
        if (b[0] >= 'a' && b[0] <= 'z') {
            b[0] = (char) (b[0] - 32);
        }
        int i = 1;
        while (i < key.length()) {
            if (b[i] >= 'A' && b[i] <= 'Z') {
                b[i] = (char) (b[i] + 32);
            }
            i++;
        }
        return new String(b);
    }

    public static String readLine(ByteArrayInputStream is) {
        StringBuilder sb = new StringBuilder(64);
        while (true) {
            int nextByte = is.read();
            if (nextByte == -1) {
                break;
            }
            char nextChar = (char) nextByte;
            if (nextChar != '\r') {
                if (nextChar == '\n') {
                    break;
                }
            } else if (((char) is.read()) == 10) {
                break;
            }
            sb.append(nextChar);
        }
        return sb.toString();
    }

    protected String[] splitHeader(String sb) {
        String str;
        int nameStart = findNonWhitespace(sb, 0);
        int nameEnd = nameStart;
        while (nameEnd < sb.length()) {
            char ch = sb.charAt(nameEnd);
            if (ch == ':' || Character.isWhitespace(ch)) {
                break;
            }
            nameEnd++;
        }
        int colonEnd = nameEnd;
        while (colonEnd < sb.length()) {
            if (sb.charAt(colonEnd) == ':') {
                colonEnd++;
                break;
            }
            colonEnd++;
        }
        int valueStart = findNonWhitespace(sb, colonEnd);
        int valueEnd = findEndOfString(sb);
        String[] strArr = new String[2];
        strArr[0] = sb.substring(nameStart, nameEnd);
        if (sb.length() < valueStart || sb.length() < valueEnd || valueStart >= valueEnd) {
            str = null;
        } else {
            str = sb.substring(valueStart, valueEnd);
        }
        strArr[1] = str;
        return strArr;
    }

    protected int findNonWhitespace(String sb, int offset) {
        int result = offset;
        while (result < sb.length() && Character.isWhitespace(sb.charAt(result))) {
            result++;
        }
        return result;
    }

    protected int findEndOfString(String sb) {
        int result = sb.length();
        while (result > 0 && Character.isWhitespace(sb.charAt(result - 1))) {
            result--;
        }
        return result;
    }

    public String toString() {
        StringBuilder headerString = new StringBuilder();
        for (Entry<String, List<String>> headerEntry : entrySet()) {
            StringBuilder headerLine = new StringBuilder();
            headerLine.append((String) headerEntry.getKey()).append(": ");
            for (String v : (List) headerEntry.getValue()) {
                headerLine.append(v).append(",");
            }
            headerLine.delete(headerLine.length() - 1, headerLine.length());
            headerString.append(headerLine).append("\r\n");
        }
        return headerString.toString();
    }
}
