package org.teleal.common.jdoc;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.RootDoc;
import com.sun.tools.javac.util.Context;
import com.sun.tools.javac.util.ListBuffer;
import com.sun.tools.javac.util.Options;
import com.sun.tools.javadoc.JavadocTool;
import com.sun.tools.javadoc.ModifierFilter;
import com.sun.tools.javadoc.PublicMessager;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Arrays;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.teleal.cling.model.message.header.EXTHeader;

public class EasyDoclet {
    private final File[] fileNames;
    private final Logger log;
    private final String[] packageNames;
    private final RootDoc rootDoc;
    private final File sourceDirectory;

    protected class LogWriter extends Writer {
        Level level;

        public LogWriter(Level level) {
            this.level = level;
        }

        public void write(char[] chars, int offset, int length) throws IOException {
            String s = new String(Arrays.copyOf(chars, length));
            if (!s.equals("\n") && !s.equals("\r") && !s.equals(" ")) {
                EasyDoclet.this.log.log(this.level, s);
            }
        }

        public void flush() throws IOException {
        }

        public void close() throws IOException {
        }
    }

    public EasyDoclet(File sourceDirectory, Collection<String> packageNames) {
        this(EXTHeader.DEFAULT_VALUE, sourceDirectory, (String[]) packageNames.toArray(new String[packageNames.size()]), new File[0]);
    }

    public EasyDoclet(String locale, File sourceDirectory, Collection<String> packageNames) {
        this(locale, sourceDirectory, (String[]) packageNames.toArray(new String[packageNames.size()]), new File[0]);
    }

    public EasyDoclet(File sourceDirectory, String... packageNames) {
        this(EXTHeader.DEFAULT_VALUE, sourceDirectory, packageNames, new File[0]);
    }

    public EasyDoclet(String locale, File sourceDirectory, String... packageNames) {
        this(locale, sourceDirectory, packageNames, new File[0]);
    }

    public EasyDoclet(File sourceDirectory, File... fileNames) {
        this(EXTHeader.DEFAULT_VALUE, sourceDirectory, new String[0], fileNames);
    }

    public EasyDoclet(String locale, File sourceDirectory, File... fileNames) {
        this(locale, sourceDirectory, new String[0], fileNames);
    }

    protected EasyDoclet(File sourceDirectory, String[] packageNames, File[] fileNames) {
        this(EXTHeader.DEFAULT_VALUE, sourceDirectory, packageNames, fileNames);
    }

    protected EasyDoclet(String locale, File sourceDirectory, String[] packageNames, File[] fileNames) {
        this.log = Logger.getLogger(EasyDoclet.class.getName());
        this.sourceDirectory = sourceDirectory;
        this.packageNames = packageNames;
        this.fileNames = fileNames;
        Context context = new Context();
        Options compOpts = Options.instance(context);
        if (getSourceDirectory().exists()) {
            this.log.fine("Using source path: " + getSourceDirectory().getAbsolutePath());
            compOpts.put("-sourcepath", getSourceDirectory().getAbsolutePath());
        } else {
            this.log.info("Ignoring non-existant source path, check your source directory argument");
        }
        ListBuffer<String> javaNames = new ListBuffer();
        for (File fileName : fileNames) {
            this.log.fine("Adding file to documentation path: " + fileName.getAbsolutePath());
            javaNames.append(fileName.getPath());
        }
        ListBuffer<String> subPackages = new ListBuffer();
        for (String packageName : packageNames) {
            this.log.fine("Adding sub-packages to documentation path: " + packageName);
            subPackages.append(packageName);
        }
        PublicMessager publicMessager = new PublicMessager(context, getApplicationName(), new PrintWriter(new LogWriter(Level.SEVERE), true), new PrintWriter(new LogWriter(Level.WARNING), true), new PrintWriter(new LogWriter(Level.FINE), true));
        try {
            this.rootDoc = JavadocTool.make0(context).getRootDocImpl(locale, null, new ModifierFilter(-9223372036854775801L), javaNames.toList(), new ListBuffer().toList(), false, subPackages.toList(), new ListBuffer().toList(), false, false, false);
            if (this.log.isLoggable(Level.FINEST)) {
                for (ClassDoc classDoc : getRootDoc().classes()) {
                    this.log.finest("Parsed Javadoc class source: " + classDoc.position() + " with inline tags: " + classDoc.inlineTags().length);
                }
            }
        } catch (Throwable ex) {
            throw new RuntimeException(ex);
        }
    }

    public File getSourceDirectory() {
        return this.sourceDirectory;
    }

    public String[] getPackageNames() {
        return this.packageNames;
    }

    public File[] getFileNames() {
        return this.fileNames;
    }

    public RootDoc getRootDoc() {
        return this.rootDoc;
    }

    protected String getApplicationName() {
        return getClass().getSimpleName() + " Application";
    }
}
