package org.teleal.common.util;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import org.teleal.cling.model.ServiceReference;

public class MimeType {
    public static final String WILDCARD = "*";
    private Map<String, String> parameters;
    private String subtype;
    private String type;

    class C04391 implements Comparator<String> {
        C04391() {
        }

        public int compare(String o1, String o2) {
            return o1.compareToIgnoreCase(o2);
        }
    }

    public MimeType() {
        this("*", "*");
    }

    public MimeType(String type, String subtype, Map<String, String> parameters) {
        if (type == null) {
            type = "*";
        }
        this.type = type;
        if (subtype == null) {
            subtype = "*";
        }
        this.subtype = subtype;
        if (parameters == null) {
            this.parameters = Collections.EMPTY_MAP;
            return;
        }
        Map<String, String> map = new TreeMap(new C04391());
        for (Entry<String, String> e : parameters.entrySet()) {
            map.put(((String) e.getKey()).toLowerCase(), e.getValue());
        }
        this.parameters = Collections.unmodifiableMap(map);
    }

    public MimeType(String type, String subtype) {
        this(type, subtype, Collections.EMPTY_MAP);
    }

    public String getType() {
        return this.type;
    }

    public boolean isWildcardType() {
        return getType().equals("*");
    }

    public String getSubtype() {
        return this.subtype;
    }

    public boolean isWildcardSubtype() {
        return getSubtype().equals("*");
    }

    public Map<String, String> getParameters() {
        return this.parameters;
    }

    public boolean isCompatible(MimeType other) {
        if (other == null) {
            return false;
        }
        if (this.type.equals("*") || other.type.equals("*")) {
            return true;
        }
        if (this.type.equalsIgnoreCase(other.type) && (this.subtype.equals("*") || other.subtype.equals("*"))) {
            return true;
        }
        if (this.type.equalsIgnoreCase(other.type) && this.subtype.equalsIgnoreCase(other.subtype)) {
            return true;
        }
        return false;
    }

    public static MimeType valueOf(String stringValue) throws IllegalArgumentException {
        if (stringValue == null) {
            throw new IllegalArgumentException("String value is null");
        }
        String params = null;
        int semicolonIndex = stringValue.indexOf(";");
        if (semicolonIndex > -1) {
            params = stringValue.substring(semicolonIndex + 1).trim();
            stringValue = stringValue.substring(0, semicolonIndex);
        }
        String major = null;
        String subtype = null;
        String[] paths = stringValue.split(ServiceReference.DELIMITER);
        if (paths.length < 2 && stringValue.equals("*")) {
            major = "*";
            subtype = "*";
        } else if (paths.length == 2) {
            major = paths[0].trim();
            subtype = paths[1].trim();
        } else if (paths.length != 2) {
            throw new IllegalArgumentException("Error parsing string: " + stringValue);
        }
        if (params == null || params.length() <= 0) {
            return new MimeType(major, subtype);
        }
        HashMap<String, String> map = new HashMap();
        for (int start = 0; start < params.length(); start = readParamsIntoMap(map, params, start)) {
        }
        return new MimeType(major, subtype, map);
    }

    public static int readParamsIntoMap(Map<String, String> map, String params, int start) {
        boolean quote = false;
        boolean backslash = false;
        int end = getEnd(params, start);
        String name = params.substring(start, end).trim();
        if (end < params.length() && params.charAt(end) == '=') {
            end++;
        }
        StringBuilder buffer = new StringBuilder(params.length() - end);
        int i = end;
        while (i < params.length()) {
            char c = params.charAt(i);
            switch (c) {
                case '\"':
                    if (!backslash) {
                        quote = !quote;
                        break;
                    }
                    backslash = false;
                    buffer.append(c);
                    break;
                case ';':
                    if (quote) {
                        buffer.append(c);
                        break;
                    }
                    map.put(name, buffer.toString().trim());
                    return i + 1;
                case '\\':
                    if (!backslash) {
                        break;
                    }
                    backslash = false;
                    buffer.append(c);
                    break;
                default:
                    buffer.append(c);
                    break;
            }
            i++;
        }
        map.put(name, buffer.toString().trim());
        return i;
    }

    protected static int getEnd(String params, int start) {
        int equals = params.indexOf(61, start);
        int semicolon = params.indexOf(59, start);
        if (equals == -1 && semicolon == -1) {
            return params.length();
        }
        if (equals == -1) {
            return semicolon;
        }
        return (semicolon == -1 || equals < semicolon) ? equals : semicolon;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        MimeType mimeType = (MimeType) o;
        if (this.parameters == null ? mimeType.parameters != null : !this.parameters.equals(mimeType.parameters)) {
            return false;
        }
        if (!this.subtype.equalsIgnoreCase(mimeType.subtype)) {
            return false;
        }
        if (this.type.equalsIgnoreCase(mimeType.type)) {
            return true;
        }
        return false;
    }

    public int hashCode() {
        return (((this.type.toLowerCase().hashCode() * 31) + this.subtype.toLowerCase().hashCode()) * 31) + (this.parameters != null ? this.parameters.hashCode() : 0);
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getType().toLowerCase()).append(ServiceReference.DELIMITER).append(getSubtype().toLowerCase());
        if (getParameters() != null || getParameters().size() > 0) {
            for (String name : getParameters().keySet()) {
                sb.append(";").append(name).append("=\"").append((String) getParameters().get(name)).append("\"");
            }
        }
        return sb.toString();
    }
}
